----------------------------------------------------------------------------------
----------------------------------------------------------------------------------
-- Baumwandler_Package.adb                                                      --
--                                                                              --
----  Mar 2008 by Ronny Friedrich & Christian Schenk                           --
----------------------------------------------------------------------------------
----------------------------------------------------------------------------------
WITH Ada.strings.Unbounded, comtypes, Text_IO;
use Ada.Strings.Unbounded, comtypes, Text_IO;

package body Baumwandler_Package is


----------------------------------------------------------------------------------
-- Dieses Packet wandelt einen Baum von der Form Comtypes in einen Baum         --
--der Form die der While_interpreter                                            --
-- versteht. Er ist in die einzelnen Funktionen Unterteilt,                     --
--die den jeweiligen Typ umwandeln.                                             --
----------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Operatoren - Wandler - Funktion                                              --
----------------------------------------------------------------------------------

function opt_Wandler (op: Unbounded_String) return Operator is
begin
   if    op = "+"   then return Plus;
   elsif op = "-"  then return Minus;
   elsif op = "*"    then return Mal;
   elsif op = "div"    then return Div;
   elsif op = "Div"    then return Div;
   elsif op = "mod" then return Modulo;
   else return Modulo;
   end if;
end opt_Wandler;

----------------------------------------------------------------------------------
-- Ausdrucks - Wandler - Funktion                                               --
----------------------------------------------------------------------------------

function Ausdruck_Wandler (Baum: GrBaum) return Ausdruck is
begin
   case Baum.Knoten.Al is
      when Zahlt =>
         return new Ausdruck_Rec'(Zahl, Integer'value(to_string(Baum.knoten.Exp)));
      when Namet =>
         return new Ausdruck_Rec'(Name, Baum.knoten.Exp);
      when auft =>
         return new Ausdruck_Rec'(Verkn, Opt_Wandler (Baum.Knoten.Rest.Rest.exp),
                                         Ausdruck_Wandler (Baum.Knoten.Rest),
            Ausdruck_Wandler (Baum.Knoten.Rest.Rest.Rest));
      when others =>
         return null;
   end case;
end Ausdruck_Wandler;

----------------------------------------------------------------------------------
-- RelationsOperatoren - Wandler - Funktion                                     --
----------------------------------------------------------------------------------

function Ropt_Wandler (Rop: Unbounded_String) return Vergleicher is
begin
   if    Rop = "<"  then return Kl;
   elsif Rop = "="  then return Gl;
   elsif Rop = ">"  then return Gr;
   elsif Rop = "<=" then return Kg;
   elsif Rop = "/=" then return Ug;
   else                  return Gg;
   end if;
end Ropt_Wandler;

----------------------------------------------------------------------------------
-- Bedingungs - Wandler - Funktion                                              --
----------------------------------------------------------------------------------

function Bedingung_Wandler (Baum: GrBaum) return Bedingung is
begin
   case Baum.Knoten.Al is
      when TrueT =>
         return new Bedingung_Rec'(Wahr, true);
      when FalseT =>
         return new Bedingung_Rec'(Wahr, False);
      when others =>
         return new Bedingung_Rec'(Vergleich, Ropt_Wandler (Baum.Knoten.Rest.exp),
                                       Ausdruck_Wandler (Baum.Knoten),
                                       Ausdruck_Wandler (Baum.Knoten.Rest.Rest));
   end case;
end Bedingung_Wandler;

----------------------------------------------------------------------------------
-- Anweisungs - Wandler - Funktion                                              --
----------------------------------------------------------------------------------

function Anw_Wandler (Baum: GrBaum) return Anweisung is
begin
   case Baum.Al is
      when Zuweisung =>
         return new Anweisung_Rec'(Zuweisung, Baum.Knoten.Exp,
                                        Ausdruck_Wandler (Baum.Knoten.Rest.Rest));
      when Bedingteanweisung =>
         return new Anweisung_Rec'(Fall, Bedingung_Wandler (Baum.Knoten.Rest),
                             Anwf_Wandler (Baum.Knoten.Rest.Rest.Rest),
                             AnwF_Wandler (Baum.Knoten.Rest.Rest.Rest.Rest.Rest));
      when Schleife =>
         return new Anweisung_Rec'(Schleife, Bedingung_Wandler (Baum.Knoten.Rest),
                                       AnwF_Wandler (Baum.Knoten.Rest.Rest.Rest));
      when Leereanweisung =>
         return new Anweisung_Rec'(Form => Skip);
      when others =>
         return null;
   end case;
end Anw_Wandler;

----------------------------------------------------------------------------------
-- AnweisungsFolgen - Wandler - Funktion                                        --
----------------------------------------------------------------------------------

function AnwF_Wandler (Baum: GrBaum) return Anweisung_F is
begin
   if Baum.Knoten.Rest = null then
      return new Anweisung_F_Rec'(Anw_Wandler (Baum.Knoten.Knoten),
                                  null);
   else
      return new Anweisung_F_Rec'(Anw_Wandler (Baum.Knoten.Knoten),
                                  Anwf_Wandler (Baum.Knoten.Rest.Rest));
   end if;
end Anwf_Wandler;

----------------------------------------------------------------------------------
-- Hauptfunktion                                                                --
-- =============                                                                --
-- Wenn der Baum was enthlt, dann starte alles umzuwandeln.                    --
-- (in den Programmkeller)                                                      --
-- Wenn der Baum leer ist, dann gebe auch einen                                 --
-- leeren Baum zurck                                                           --
----------------------------------------------------------------------------------

function Baum_Wandler (Baum: GrBaum) return Teile is
begin
   if Baum.Al = Programm then
      if Baum.Knoten.Knoten.Rest = null then
         return new Teile_Rec'(new Teil_Rec'(Anw_F,
                                 new Anweisung_F_Rec'
                                       (Anw_Wandler (Baum.Knoten.Knoten.Knoten),
                                             null)),
                               null);
      else
         return new Teile_Rec'(new Teil_Rec'(Anw_F,
                      new Anweisung_F_Rec'(Anw_Wandler (Baum.Knoten.Knoten.Knoten),
                           Anwf_Wandler (Baum.Knoten.Knoten.Rest.Rest))),
                               null);
      end if;
   end if;
   return null;
end Baum_Wandler;
   
----------------------------------------------------------------------------------
-- Abfrage ob eine Variable bereits in der Umgebung enthalten ist               --
-- Hilfsfunktion fr Var_Tupel, die True liefert falls Var bereits              --
-- in der Umgebung aufgefhrt ist
----------------------------------------------------------------------------------

Function Wert_Suche (Var : Unbounded_String;
                     Umg : Umgebung) return Boolean is
Begin
   If Umg = null then
      return False;
   Elsif Umg.Elem.Variable_Name = Var then
      return True;
   else
      Return Wert_Suche (Var, Umg.Next);
   end if;
end Wert_Suche;

----------------------------------------------------------------------------------
-- Durchsucht GR nach dem Terminal "nameT" und legt die Umgebung an             --
----------------------------------------------------------------------------------
-- Funktion wird im Tracer aufgerufen und gibt bei leerem Baum leere Umgebung   --
-- zurck.                                                                      --
-- Fr jedes NT ruft sich die Funktion rekursiv auf um dann nach dem            --
-- Terminal "nameT" zu suchen. In nameT steht die Variable die an Umgebung mit  --
-- undef bergeben wird                                                         --
----------------------------------------------------------------------------------

   function Var_Tupel (Gr: Grbaum; Var_Liste: Umgebung) return Umgebung is
      gr_h  : GrBaum := gr;
      var_W : Wert(undef);
   begin
      if Gr_H = null then
         return Var_Liste;
      elsif Gr_H.Al in Comtypes.Programm .. Comtypes.Bedingung then
         return Var_Tupel (Gr_H.Rest, Var_Tupel (Gr_H.Knoten, Var_Liste));
      elsif Gr_H.Al = Namet then
         if not(Wert_Suche(Gr_H.Exp, Var_Liste)) then
            return new Umgebung_Rec'((Gr_H.Exp, Var_W),
                                      Var_Tupel (Gr_H.Rest, Var_Liste));
         else
            return Var_Tupel (Gr_H.Rest, Var_Liste);
         end if;
      else
         return Var_Tupel (Gr_H.Rest, Var_Liste);
      end if;
   end Var_Tupel;

---------------------------------------------------------------------------------
--       Ausgabe des Traces vom While-Programm                                 --
---------------------------------------------------------------------------------   
   procedure Trace (Z: Zustand) is
   begin
      case Z.Form is
         when Akt   => Put_Line(Zeige_Konf(Z.Akt_Zustand));
                       Trace (Delta2(Z));
         when Ende  => Put_Line( "Endspeicherbelegung: {" &
                                 (Zeige_Vars(Z.Ende_Zustand)) & "}");
      end case;
   exception
      when Constraint_Error => raise WhileInit_Error;
   end Trace;

----------------------------------------------------------------------------------
-- Ausgabe des Traces mit dem Zustand an                                        --
-- Schnittstelle zum CompilerInterpreter                                        --
----------------------------------------------------------------------------------
-- Beispiel des WHILE-Programms: " a := 1 ; "                                   --
-- << (a,_) ,  , a:= 1 >>                                                       --
-- << (a,_) ,  , a:= 1&[] >>                                                    --
-- << (a,_) ,  ,  1&:=&a&[] >>                                                  --
-- << (a,_) ,  1 , :=&a&[] >>                                                   --
-- << (a, 1) ,  , [] >>                                                         --
-- << (a, 1) ,  ,  >>                                                           --
-- Endspeicherbelegung: {(a, 1)}                                                --
----------------------------------------------------------------------------------

   procedure macheTrace (Baum: GrBaum) is
   begin
      Trace ((akt, (Var_Tupel (Baum, null), null, Baum_Wandler (Baum))));
   end macheTrace;

----------------------------------------------------------------------------------
-- Trace-Prozedur mit Ausgabe in Textdatei                                      --
----------------------------------------------------------------------------------      

   procedure Trace_File (Z: Zustand; FT: File_Type) is
   begin
      case Z.Form is
         when Akt   => Put_Line(FT, Zeige_Konf(Z.Akt_Zustand));
                       Trace_File (Delta2_FT(Z, ft), FT);
         when Ende  => Put_Line(Ft, "Endspeicherbelegung: {" &
                                    (Zeige_Vars(Z.Ende_Zustand)) & "}");
      end case;
   exception
      when Constraint_Error => raise WhileInit_Error;
   end Trace_File;

----------------------------------------------------------------------------------
-- macheTrace-Prozedur mit Ausgabe in Textdatei                                 --
-- berlagerung der Trace-Prozedur ohne Datei                                   --
---------------------------------------------------------------------------------- 
   procedure macheTrace (Baum: GrBaum; FT: File_Type) is
   begin
      Trace_File ((akt, (Var_Tupel (Baum, null), null, Baum_Wandler (Baum))), FT);
   end macheTrace;


----------------------------------------------------------------------------------
----------------------------------------------------------------------------------
----------Funktionen: kopiert aus While-Interpreter-------------------------------
----------------------------------------------------------------------------------
----------------------------------------------------------------------------------
   function Ada_Op (Op     : Operator;
                    Z1, Z2 : Integer  ) return Integer is

      Erg : Integer;

   begin
      case Op is
         when Plus   => Erg:= Z1 + Z2;

         when Minus  => Erg:= Z1 - Z2;

         when Mal    => Erg:= Z1 * Z2;

         when Div    => Erg:= Z1 / Z2;

         when Modulo => Erg:= Z1 mod Z2;
      end case;
      return Erg;
   end Ada_Op;

   function Ada_Rel_Op (Rop    : Vergleicher;
                        Z1,Z2  : Integer     ) return Boolean is

      Erg : Boolean;

   begin
      case Rop is
         when Kl => Erg := Z1 < Z2;

         when Gl => Erg := Z1 = Z2;

         when Gr => Erg := Z1 > Z2;

         when Kg => Erg := Z1 <= Z2;

         when Ug => Erg := Z1 /= Z2;

         when Gg => Erg := Z1 >= Z2;
      end case;
      return Erg;
   end Ada_Rel_Op;

   function Ist_Undef (W : Wert) return Boolean is

      Wert_Def : Wert := W;

   begin
      if Wert_Def.Form = Undef then
         return True;
      else
         return False;
      end if;
   end Ist_Undef;

   function VarD return Variable is

      Var : Variable;
      Zw  : Wert (Def_Wert);

   begin
      Var.Variable_Name := To_Unbounded_String("d");
      Zw.Zahl_Wert      := 0;
      Var.Variable_Wert := Zw;
      return Var;
   end VarD;

   function Wert_Suche (Var : Unbounded_String;
                        Umg : Umgebung         ) return Wert is

      C  : Unbounded_String := Var;
      U  : Umgebung         := Umg;
      Zw : Wert (Undef);

   begin
      if U = null then
         return Zw;
      elsif U.Elem.Variable_Name = C then
         return U.Elem.Variable_Wert;
      else
         return Wert_Suche (C,U.Next);
      end if;
   end Wert_Suche;

   function Wert_Eintrag (Var      : Unbounded_String;
                          Int_Wert : Integer;
                          Umg      : Umgebung         ) return Umgebung is

      C   : Unbounded_String := Var;
      U   : Umgebung         := Umg;
      Int : Integer          := Int_Wert; -- copyfunction da alte umgebung zerstrt wird
      Zw  : Wert (Def_Wert);

   begin
      if U = null then
         Put("error: Variable " & To_String(C) & " nicht in der Umgebung");
      elsif U.Elem.Variable_Name = C then
         case U.Elem.Variable_Wert.Form is
            when Undef    => U.Elem.Variable_Wert           := Zw ;
                             U.Elem.Variable_Name           := C;
                             U.Elem.Variable_Wert.Zahl_Wert := Int;
            when Def_Wert => U.Elem.Variable_Wert.Zahl_Wert := Int;
         end case;
      else
         U.Next := Wert_Eintrag (C,Int,U.Next);
         return U;
      end if;
      return U;
   end Wert_Eintrag;

-----------------------------------------------------------------------------------------------
---------- Zeige-Funktionen -------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
   function Zeige_Op (A : Operator) return String is

      Erg : Unbounded_String := Null_Unbounded_String;

   begin
      case A is
         when Plus   => Erg := To_Unbounded_String("+");

         when Minus  => Erg := To_Unbounded_String("-");

         when Mal    => Erg := To_Unbounded_String("*");

         when Div    => Erg := To_Unbounded_String("`div`");

         when Modulo => Erg := To_Unbounded_String("`mod`");
      end case;
      return To_String(Erg);
   end Zeige_Op;
-----------------------------------------------------------------------------------------------
   function Zeige_Vergl (A : Vergleicher ) return String is

      Erg : Unbounded_String := Null_Unbounded_String;

   begin
      case A is
         when Kl => Erg := To_Unbounded_String("<");

         when Gl => Erg := To_Unbounded_String("=");

         when Gr => Erg := To_Unbounded_String(">");

         when Kg => Erg := To_Unbounded_String("<=");

         when Ug => Erg := To_Unbounded_String("/=");

         when Gg => Erg := To_Unbounded_String(">=");
      end case;
      return To_String(Erg);
   end Zeige_Vergl;
-----------------------------------------------------------------------------------------------
   function Zeige_Ausdr (A : Ausdruck) return String is

      Erg      : Unbounded_String := Null_Unbounded_String;
      Li,Mi,Re : Unbounded_String := Null_Unbounded_String;

   begin
      case A.Form is
         when Zahl  => Erg := To_Unbounded_String(Integer'Image(A.Zahl_Wert));

         when Name  => Erg := A.Name_Wert;

         when Verkn => Li  := To_Unbounded_String(Zeige_Ausdr(A.Links_Ausdruck));
                       Mi  := To_Unbounded_String(Zeige_Op(A.Operator_Wert));
                       Re  := To_Unbounded_String(Zeige_Ausdr(A.Rechts_Ausdruck));
                       Erg := "(" & Li & Mi & Re & ")";
      end case;
      return To_String(Erg);
   end Zeige_Ausdr;
-----------------------------------------------------------------------------------------------
   function Zeige_Bed (A : Bedingung) return String is

      Erg       : Unbounded_String := Null_Unbounded_String;
      Li,Mi,Re  : Unbounded_String := Null_Unbounded_String;

   begin
      case A.Form is
         when Wahr      => if A.Wahr_Wert then
                              Erg := To_Unbounded_String("true");
                           else
                              Erg := To_Unbounded_String("false");
                           end if;

         when Vergleich => Li  := To_Unbounded_String(Zeige_Ausdr(A.Links_Ausdruck));
                           Mi  := To_Unbounded_String(Zeige_Vergl(A.Vergleichs_Wert));
                           Re  := To_Unbounded_String(Zeige_Ausdr(A.Rechts_Ausdruck));
                           Erg := "(" & Li & Mi & Re & ")";
      end case;
      return To_String(Erg);
   end Zeige_Bed;
-----------------------------------------------------------------------------------------------
   function Zeige_Anw_F (A : Anweisung_F) return String is

      Erg : Unbounded_String := Null_Unbounded_String;

      function Zeige_Anw (A : Anweisung) return String is

         Erglok      : Unbounded_String := Null_Unbounded_String;
         Bed,Afa,Afb : Unbounded_String := Null_Unbounded_String;

      begin
         case A.Form is
            when Fall     => Bed    := To_Unbounded_String(Zeige_Bed(A.Bedingungs_Wert_Fall));
                             Afa    := To_Unbounded_String(Zeige_Anw_F(A.Erst_Anweisung_Fall));
                             Afb    := To_Unbounded_String(Zeige_Anw_F(A.Zweit_Anweisung_Fall));
                             Erglok := " if " & Bed & " then " & Afa & " else " & Afb & " fi";

            when Schleife => Bed := To_Unbounded_String(Zeige_Bed(A.Bedingungs_Wert_Schleife));
                             Afa := To_Unbounded_String(Zeige_Anw_F(A.Erst_Anweisung_Schleife));
                             Erglok := " while " & Bed & " do " & Afa & " od ";

            when Skip      => Erglok := To_Unbounded_String("skip");

            when Zuweisung => Erglok := A.Zuweisungs_Name &
                                        ":=" &
                                        To_Unbounded_String(Zeige_Ausdr(A.Links_Ausdruck));
         end case;
         return To_String(Erglok);
      end Zeige_Anw;

   begin
      if A /= null then
         if A.Rest = null then
            Erg := To_Unbounded_String(Zeige_Anw(A.Anw));
         else
            Erg := To_Unbounded_String(Zeige_Anw(A.Anw)) &
                   ";" &
                   To_Unbounded_String(Zeige_Anw_F(A.Rest));
         end if;
      else
         Erg := To_Unbounded_String("[]");
      end if;
      return To_String(Erg);
   end Zeige_Anw_F;
-----------------------------------------------------------------------------------------------
   function Zeige_Teile (A : Teile) return String is

      Erg : Unbounded_String := Null_Unbounded_String;
      -----------------------------------------------------------------------------------------
      function Zeige_Teil (A : Teil) return String is

         Erglok : Unbounded_String := Null_Unbounded_String;

      begin
         case A.Form is
            when Anw     => Erglok := To_Unbounded_String
                                      (Zeige_Anw_F(new Anweisung_F_Rec'(A.Teil_Anw,null)));

            when Anw_F   => Erglok := To_Unbounded_String(Zeige_Anw_F(A.Teil_Anw_F));

            when Bed     => Erglok := To_Unbounded_String(Zeige_Bed(A.Teil_Bed));

            when Ausdr   => Erglok := To_Unbounded_String(Zeige_Ausdr(A.Teil_Ausdr));

            when Bez     => Erglok := (A.Teil_Bez);

            when Zwert   => Erglok := To_Unbounded_String(Integer'Image(A.Teil_Zwert));

            when Wwert   => if A.Teil_Wwert then
                               Erglok := To_Unbounded_String("true");
                            else
                               Erglok := To_Unbounded_String("false");
                            end if;

            when Op      => Erglok := To_Unbounded_String(Zeige_Op(A.Teil_Op));

            when Rop     => Erglok := To_Unbounded_String(Zeige_Vergl(A.Teil_Rop));

            when If_Case => Erglok := To_Unbounded_String("if");

            when Do_Case => Erglok := To_Unbounded_String("do");

            when Zuw     => Erglok := To_Unbounded_String(":=");
         end case;
         return To_String(Erglok);
      end Zeige_Teil;
      -----------------------------------------------------------------------------------------
   begin
      if A /= null then
         if A.Rest = null then
            Erg := To_Unbounded_String(Zeige_Teil(A.T));
         else
            Erg := To_Unbounded_String(Zeige_Teil(A.T)) &
                   "&" &
                   To_Unbounded_String(Zeige_Teile(A.Rest));
         end if;
      end if;
      return To_String(Erg);
   end Zeige_Teile;
-----------------------------------------------------------------------------------------------
   function Zeige_Wert (A : Wert) return String is

      Erg : Unbounded_String := Null_Unbounded_String;

   begin
      case A.Form is
         when Undef    => Erg := To_Unbounded_String("_");

         when Def_Wert => Erg := To_Unbounded_String(Integer'Image(A.Zahl_Wert));
      end case;
      return To_String(Erg);
   end Zeige_Wert;
-----------------------------------------------------------------------------------------------
   function Zeige_Var (A : Variable) return String is

      Erg : Unbounded_String := Null_Unbounded_String;

   begin
      Erg := To_Unbounded_String("(" & To_String(A.Variable_Name) &
                                 "," & Zeige_Wert(A.Variable_Wert) &
                                 ")");
      return To_String(Erg);
   end Zeige_Var;
-----------------------------------------------------------------------------------------------
   function Zeige_Vars (A : Umgebung) return String is

      Erg : Unbounded_String := Null_Unbounded_String;

   begin
      if A /= null then
         Erg := To_Unbounded_String(Zeige_Var (A.Elem) & Zeige_Vars (A.Next));
      end if;
      return To_String(Erg);
   end Zeige_Vars;
-----------------------------------------------------------------------------------------------
   function Rev (T : Teile) return Teile is

      Tx     : Teile := T;
      Result : Teile;

   begin
      while Tx /= null loop
         Result := new Teile_Rec'(Tx.T, Result);
         Tx     := Tx.Rest;
      end loop;
      return Result;
   end Rev;
-----------------------------------------------------------------------------------------------
   function Zeige_Konf (A : Konfiguration) return String is

      Erg : Unbounded_String := Null_Unbounded_String;

   begin
      Erg := To_Unbounded_String("<< " & Zeige_Vars (A.Um) &
                                 " , " & Zeige_Teile (Rev(A.Wk)) &
                                 " , " & Zeige_Teile (A.Zk) &
                                 " >>");
      return To_String(Erg);
   end Zeige_Konf;

-----------------------------------------------------------------------------------------------
---------- DELTA-Funktion (Zustandsbergangsfunktion) -----------------------------------------
--------------------------------------------------------------- teilweise eingerckt (***) ----
-----------------------------------------------------------------------------------------------
   function Delta2 (Z : Zustand) return Zustand is

      Result      : Zustand        := Z;
      EndOfResult : Zustand (Ende);

   begin
      New_Line;
      if (not (Result.Form = Ende)) and then (not (Result.Akt_Zustand.Zk = null)) then
         case Result.Akt_Zustand.Zk.T.Form is
            --Anw
            when Anw    => case Result.Akt_Zustand.Zk.T.Teil_Anw.Form is
                             --Anw: Fall
                             when Fall => Put_Line("-- (12 : If-Anweisung aufloesen)--");
      --(***)
      -- lse Fall auf und hnge die Komponenten vor zk (ohne Anw)
      Result.Akt_Zustand.Zk := new Teile_Rec'(new Teil_Rec'
                                (Bed,Result.Akt_Zustand.Zk.T.Teil_Anw.Bedingungs_Wert_Fall),
                                new Teile_Rec'(new Teil_Rec'(Form=>If_Case),
                                 new Teile_Rec'(new Teil_Rec'
                                  (Anw_F,Result.Akt_Zustand.Zk.T.Teil_Anw.Erst_Anweisung_Fall),
                                  new Teile_Rec'(new Teil_Rec'
                                   (Anw_F,Result.Akt_Zustand.Zk.T.Teil_Anw.Zweit_Anweisung_Fall),
                                    Result.Akt_Zustand.Zk.Rest))));

                             --Anw: Schleife
                             when Schleife => Put_Line("-- (15 : Schleife aufloesen)--");

      --(***)
      -- lse schleife auf und hnge die Teile vor zk (ohne die Schleife)
      Result.Akt_Zustand.Zk := new Teile_Rec'(new Teil_Rec'
                                (Bed,Result.Akt_Zustand.Zk.T.Teil_Anw.Bedingungs_Wert_Schleife),
                                new Teile_Rec'(new Teil_Rec'(Form=>Do_Case),
                                 new Teile_Rec'(new Teil_Rec'
                                  (Bed,Result.Akt_Zustand.Zk.T.Teil_Anw.Bedingungs_Wert_Schleife),
                                  new Teile_Rec'(new Teil_Rec'
                                   (Anw_F,Result.Akt_Zustand.Zk.T.Teil_Anw.Erst_Anweisung_Schleife),
                                   Result.Akt_Zustand.Zk.Rest))));

                           --               if result.akt_zustand.zk /= null then
                           --                 result.akt_zustand.zk := result.akt_zustand.zk.rest;
                           --               end if;

                             -- Anw: Skip
                             when Skip => Put_Line("-- (8 : Skipanweisung)--");
                                        -- mache nichts..
                                        if Result.Akt_Zustand.Zk /= null then
                                        Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                                        end if;
                             -- Anw: Zuweisung
                             when Zuweisung => Put_Line("-- (9 : Zuweisung aufloesen)--");

      --(***)
      -- Lse Zuweisung und ersetze sie durch ihren Inhalt
      Result.Akt_Zustand.Zk := new Teile_Rec'(new Teil_Rec'
                                (Ausdr,Result.Akt_Zustand.Zk.T.Teil_Anw.Links_Ausdruck),
                                new Teile_Rec'(new Teil_Rec'(Form=>Zuw),
                                 new Teile_Rec'(new Teil_Rec'
                                  (Bez,Result.Akt_Zustand.Zk.T.Teil_Anw.Zuweisungs_Name),
                                  Result.Akt_Zustand.Zk.Rest)));

                           end case;
            --Anw_F
            when Anw_F  => -- unterscheidung von 11) und 11)' (Teil_anw_f leer oder nicht)
                           if Result.Akt_Zustand.Zk.T.Teil_Anw_F = null then
                              Put_Line("-- (11' : leere Anweisungsfolge aufloesen)--");
                              -- leere Anweisungsfolge: wird gelscht
                              Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                           else
                              Put_Line("-- (11 : nichtleere Anweisungsfolge aufloesen)--");
                              -- hnge 1.Anweisung aus Anw_f vor Anw_f
                              Result.Akt_Zustand.Zk := new Teile_Rec'(new Teil_Rec'
                                                       (Anw,Result.Akt_Zustand.Zk.T.Teil_Anw_F.Anw),
                                                       Result.Akt_Zustand.Zk);
                              -- und lsche sie dort
                              if Result.Akt_Zustand.Zk.Rest.T.Teil_Anw_F /= null then
                                 Result.Akt_Zustand.Zk.Rest.T.Teil_Anw_F :=   --(***)
                                                      Result.Akt_Zustand.Zk.Rest.T.Teil_Anw_F.Rest;
                              end if;
                           end if;
            --Bed
            when Bed    => case Result.Akt_Zustand.Zk.T.Teil_Bed.Form is
                              --Bed: Wahr
                              when Wahr => Put_Line("-- (5 : Wahrheitswert lesen)--");

      --(***)
      -- kopiere Wahrheitswert von zk nach wk
      Result.Akt_Zustand.Wk := new Teile_Rec'(new Teil_Rec'
                                (Wwert,Result.Akt_Zustand.Zk.T.Teil_Bed.Wahr_Wert),
                                Result.Akt_Zustand.Wk);

                                           -- und lsche ihn in zk
                                           if Result.Akt_Zustand.Zk /= null then
                                              Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                                           end if;

                              --Bed: Vergleich
                              when Vergleich => Put_Line("-- (6 : Bedingung aufloesen)--");

      --(***)
      -- lse die Verknpfung auf und hnge sie an (oder vor) zk (ohne die Verknpfung)
      Result.Akt_Zustand.Zk := new Teile_Rec'(new Teil_Rec'
                                (Ausdr,Result.Akt_Zustand.Zk.T.Teil_Bed.Links_Ausdruck),
                                new Teile_Rec'(new Teil_Rec'
                                 (Ausdr,Result.Akt_Zustand.Zk.T.Teil_Bed.Rechts_Ausdruck),
                                 new Teile_Rec'(new Teil_Rec'
                                  (Rop,Result.Akt_Zustand.Zk.T.Teil_Bed.Vergleichs_Wert),
                                  Result.Akt_Zustand.Zk.Rest)));

                           end case;
            --Ausdr
            when Ausdr  => case Result.Akt_Zustand.Zk.T.Teil_Ausdr.Form is
                              --Ausdr: Zahl
                              when Zahl  => Put_Line("-- (1 : Zahl lesen) --");

      --(***)
      -- kopiere die Zahl von zk nach wk
      Result.Akt_Zustand.Wk := new Teile_Rec'(new Teil_Rec'
                                (Zwert,Result.Akt_Zustand.Zk.T.Teil_Ausdr.Zahl_Wert),
                                Result.Akt_Zustand.Wk);

                                            -- und lsche sie in zk
                                            if Result.Akt_Zustand.Zk /= null then
                                               Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                                            end if;

                              --Ausdr: Name
                              when Name => Put_Line("-- (2 : Variable lesen, Wert aus Umgebung)--");

      --(***)
      -- kopiere Belegung der Variable unter um nach wk
      Result.Akt_Zustand.Wk := new Teile_Rec'(new Teil_Rec'
                               (Zwert,--finde richtigen Wert fr Variable
                                Wert_Suche(Result.Akt_Zustand.Zk.T.Teil_Ausdr.Name_Wert,
                                           Result.Akt_Zustand.Um).Zahl_Wert),
                                           Result.Akt_Zustand.Wk);

                                            -- und lsche sie in zk
                                            if Result.Akt_Zustand.Zk /= null then
                                               Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                                            end if;
                              --Ausdr: Verkn
                              when Verkn => Put_Line("-- (3 : Verknuepfung aufloesen)--");

      --(***)
      -- lse die Verknpfung auf und hnge sie an (oder vor) zk (ohne die Verknpfung)
      Result.Akt_Zustand.Zk := new Teile_Rec'(new Teil_Rec'
                                (Ausdr,Result.Akt_Zustand.Zk.T.Teil_Ausdr.Links_Ausdruck),
                                new Teile_Rec'(new Teil_Rec'
                                 (Ausdr,Result.Akt_Zustand.Zk.T.Teil_Ausdr.Rechts_Ausdruck),
                                 new Teile_Rec'(new Teil_Rec'
                                  (Op,Result.Akt_Zustand.Zk.T.Teil_Ausdr.Operator_Wert),
                                  Result.Akt_Zustand.Zk.Rest)));

                           end case;
            when Bez    => null;     --hier auf string umsteigen
            when Zwert  => null;
            when Wwert  => null;
            --Op
            when Op     => Put_Line("-- (4 : Arithmetische Operation)--");
                           -- hnge das Ergebnis der Operation mit den letzten beiden
                           -- Werte von wk an wk statt der beiden Werte
                           Result.Akt_Zustand.Wk := new Teile_Rec'
                                                        (new Teil_Rec'(Zwert,Ada_Op
                                                           (Result.Akt_Zustand.Zk.T.Teil_Op,
                                                            Result.Akt_Zustand.Wk.Rest.T.Teil_Zwert,
                                                            Result.Akt_Zustand.Wk.T.Teil_Zwert)),
                                                        Result.Akt_Zustand.Wk.Rest.Rest);
                           -- und lsche den Operator in zk
                           if Result.Akt_Zustand.Zk /= null then
                              Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                           end if;
            -- Rop
            when Rop    => Put_Line("-- (7 : Vergleichsoperation)--");
                           -- hnge das Ergebnis der Operation mit den letzten beiden
                           -- Werte von wk an wk statt der beiden Werte
                           Result.Akt_Zustand.Wk := new Teile_Rec'
                                                        (new Teil_Rec'(Wwert,Ada_Rel_Op
                                                           (Result.Akt_Zustand.Zk.T.Teil_Rop,
                                                            Result.Akt_Zustand.Wk.Rest.T.Teil_Zwert,
                                                            Result.Akt_Zustand.Wk.T.Teil_Zwert)),
                                                        Result.Akt_Zustand.Wk.Rest.Rest);
                           -- und lsche den Operator in zk
                           Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
            --If_case
            when If_Case => -- Unterscheidung zwischen 13) und 14)
                            -- lsche if
                            Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                            if Result.Akt_Zustand.Wk.T.Teil_Wwert then
                               Put_Line("-- (13 : If-Anweisung True-Fall)--");
                               -- lsche Anw_f2
                               Result.Akt_Zustand.Zk.Rest := Result.Akt_Zustand.Zk.Rest.Rest;
                            else
                               Put_Line("-- (14 : If-Anweisung False-Fall)--");
                               -- lsche Anw_f1
                               Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                            end if;
                            -- lsche WWert in wk
                            Result.Akt_Zustand.Wk := Result.Akt_Zustand.Wk.Rest;
            --Do_case
            when Do_Case => -- Unterscheidung zwischen 16) und 17)
                            -- lsche Do_case
                            Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                            if Result.Akt_Zustand.Wk.T.Teil_Wwert then
                               Put_Line("-- (16 : Schleifenbedingung True)--");

      --(***)
      -- extrahiere Anw_f und erstelle neue Schleife
      Result.Akt_Zustand.Zk := new Teile_Rec'(new Teil_Rec'
                                (Anw_F,Result.Akt_Zustand.Zk.Rest.T.Teil_Anw_F),
                                new Teile_Rec'(new Teil_Rec'
                                 (Anw,new Anweisung_Rec'(Schleife,
                                                         Result.Akt_Zustand.Zk.T.Teil_Bed,
                                                         Result.Akt_Zustand.Zk.Rest.T.Teil_Anw_F)),
                                                         Result.Akt_Zustand.Zk.Rest.Rest));
                            else
                               Put_Line("-- (17 : Schleifenbedingung False)--");
                               -- lsche SchleifenRest
                               if Result.Akt_Zustand.Zk /= null then
                                  Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest.Rest;
                               end if;
                            end if;
                            -- lsche WWert
                            if Result.Akt_Zustand.Wk /= null then
                               Result.Akt_Zustand.Wk := Result.Akt_Zustand.Wk.Rest;
                            end if;
            --Zuw
            when Zuw     => Put_Line("-- (10 : Zuweisung ausfuehren)--");
                            -- verndere Umgebung
                            Result.Akt_Zustand.Um := Wert_Eintrag
                                                         (Result.Akt_Zustand.Zk.Rest.T.Teil_Bez,
                                                          Result.Akt_Zustand.Wk.T.Teil_Zwert,
                                                          Result.Akt_Zustand.Um);
                            -- lsche ZWert in wk
                            Result.Akt_Zustand.Wk := Result.Akt_Zustand.Wk.Rest;
                            -- und lsche Zuweisung und Bezeichner in zk
                            Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest.Rest;

         end case;
      else
         Put_Line("-- (Ende der Auswertung)--");
         EndOfResult.Ende_Zustand := Result.Akt_Zustand.Um;
         Result := EndOfResult;
      end if;

      return Result;
   end Delta2;

   function Delta2_FT (Z : Zustand; FT : File_Type) return Zustand is

      Result      : Zustand        := Z;
      EndOfResult : Zustand (Ende);

   begin
      New_Line;
      if (not (Result.Form = Ende)) and then (not (Result.Akt_Zustand.Zk = null)) then
         case Result.Akt_Zustand.Zk.T.Form is
            --Anw
            when Anw    => case Result.Akt_Zustand.Zk.T.Teil_Anw.Form is
                             --Anw: Fall
                             when Fall => Put_Line(ft, "-- (12 : If-Anweisung aufloesen)--");
      --(***)
      -- lse Fall auf und hnge die Komponenten vor zk (ohne Anw)
      Result.Akt_Zustand.Zk := new Teile_Rec'(new Teil_Rec'
                                (Bed,Result.Akt_Zustand.Zk.T.Teil_Anw.Bedingungs_Wert_Fall),
                                new Teile_Rec'(new Teil_Rec'(Form=>If_Case),
                                 new Teile_Rec'(new Teil_Rec'
                                  (Anw_F,Result.Akt_Zustand.Zk.T.Teil_Anw.Erst_Anweisung_Fall),
                                  new Teile_Rec'(new Teil_Rec'
                                   (Anw_F,Result.Akt_Zustand.Zk.T.Teil_Anw.Zweit_Anweisung_Fall),
                                    Result.Akt_Zustand.Zk.Rest))));

                             --Anw: Schleife
                             when Schleife => Put_Line(ft, "-- (15 : Schleife aufloesen)--");

      --(***)
      -- lse schleife auf und hnge die Teile vor zk (ohne die Schleife)
      Result.Akt_Zustand.Zk := new Teile_Rec'(new Teil_Rec'
                                (Bed,Result.Akt_Zustand.Zk.T.Teil_Anw.Bedingungs_Wert_Schleife),
                                new Teile_Rec'(new Teil_Rec'(Form=>Do_Case),
                                 new Teile_Rec'(new Teil_Rec'
                                  (Bed,Result.Akt_Zustand.Zk.T.Teil_Anw.Bedingungs_Wert_Schleife),
                                  new Teile_Rec'(new Teil_Rec'
                                   (Anw_F,Result.Akt_Zustand.Zk.T.Teil_Anw.Erst_Anweisung_Schleife),
                                   Result.Akt_Zustand.Zk.Rest))));

                           --               if result.akt_zustand.zk /= null then
                           --                 result.akt_zustand.zk := result.akt_zustand.zk.rest;
                           --               end if;

                             -- Anw: Skip
                             when Skip => Put_Line(ft, "-- (8 : Skipanweisung)--");
                                        -- mache nichts..
                                        if Result.Akt_Zustand.Zk /= null then
                                        Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                                        end if;
                             -- Anw: Zuweisung
                             when Zuweisung => Put_Line(ft, "-- (9 : Zuweisung aufloesen)--");

      --(***)
      -- Lse Zuweisung und ersetze sie durch ihren Inhalt
      Result.Akt_Zustand.Zk := new Teile_Rec'(new Teil_Rec'
                                (Ausdr,Result.Akt_Zustand.Zk.T.Teil_Anw.Links_Ausdruck),
                                new Teile_Rec'(new Teil_Rec'(Form=>Zuw),
                                 new Teile_Rec'(new Teil_Rec'
                                  (Bez,Result.Akt_Zustand.Zk.T.Teil_Anw.Zuweisungs_Name),
                                  Result.Akt_Zustand.Zk.Rest)));

                           end case;
            --Anw_F
            when Anw_F  => -- unterscheidung von 11) und 11)' (Teil_anw_f leer oder nicht)
                           if Result.Akt_Zustand.Zk.T.Teil_Anw_F = null then
                              Put_Line(ft, "-- (11' : leere Anweisungsfolge aufloesen)--");
                              -- leere Anweisungsfolge: wird gelscht
                              Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                           else
                              Put_Line(ft, "-- (11 : nichtleere Anweisungsfolge aufloesen)--");
                              -- hnge 1.Anweisung aus Anw_f vor Anw_f
                              Result.Akt_Zustand.Zk := new Teile_Rec'(new Teil_Rec'
                                                       (Anw,Result.Akt_Zustand.Zk.T.Teil_Anw_F.Anw),
                                                       Result.Akt_Zustand.Zk);
                              -- und lsche sie dort
                              if Result.Akt_Zustand.Zk.Rest.T.Teil_Anw_F /= null then
                                 Result.Akt_Zustand.Zk.Rest.T.Teil_Anw_F :=   --(***)
                                                      Result.Akt_Zustand.Zk.Rest.T.Teil_Anw_F.Rest;
                              end if;
                           end if;
            --Bed
            when Bed    => case Result.Akt_Zustand.Zk.T.Teil_Bed.Form is
                              --Bed: Wahr
                              when Wahr => Put_Line(ft, "-- (5 : Wahrheitswert lesen)--");

      --(***)
      -- kopiere Wahrheitswert von zk nach wk
      Result.Akt_Zustand.Wk := new Teile_Rec'(new Teil_Rec'
                                (Wwert,Result.Akt_Zustand.Zk.T.Teil_Bed.Wahr_Wert),
                                Result.Akt_Zustand.Wk);

                                           -- und lsche ihn in zk
                                           if Result.Akt_Zustand.Zk /= null then
                                              Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                                           end if;

                              --Bed: Vergleich
                              when Vergleich => Put_Line(ft, "-- (6 : Bedingung aufloesen)--");

      --(***)
      -- lse die Verknpfung auf und hnge sie an (oder vor) zk (ohne die Verknpfung)
      Result.Akt_Zustand.Zk := new Teile_Rec'(new Teil_Rec'
                                (Ausdr,Result.Akt_Zustand.Zk.T.Teil_Bed.Links_Ausdruck),
                                new Teile_Rec'(new Teil_Rec'
                                 (Ausdr,Result.Akt_Zustand.Zk.T.Teil_Bed.Rechts_Ausdruck),
                                 new Teile_Rec'(new Teil_Rec'
                                  (Rop,Result.Akt_Zustand.Zk.T.Teil_Bed.Vergleichs_Wert),
                                  Result.Akt_Zustand.Zk.Rest)));

                           end case;
            --Ausdr
            when Ausdr  => case Result.Akt_Zustand.Zk.T.Teil_Ausdr.Form is
                              --Ausdr: Zahl
                              when Zahl  => Put_Line(ft, "-- (1 : Zahl lesen) --");

      --(***)
      -- kopiere die Zahl von zk nach wk
      Result.Akt_Zustand.Wk := new Teile_Rec'(new Teil_Rec'
                                (Zwert,Result.Akt_Zustand.Zk.T.Teil_Ausdr.Zahl_Wert),
                                Result.Akt_Zustand.Wk);

                                            -- und lsche sie in zk
                                            if Result.Akt_Zustand.Zk /= null then
                                               Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                                            end if;

                              --Ausdr: Name
                              when Name => Put_Line(ft, "-- (2 : Variable lesen, Wert aus Umgebung)--");

      --(***)
      -- kopiere Belegung der Variable unter um nach wk
      Result.Akt_Zustand.Wk := new Teile_Rec'(new Teil_Rec'
                               (Zwert,--finde richtigen Wert fr Variable
                                Wert_Suche(Result.Akt_Zustand.Zk.T.Teil_Ausdr.Name_Wert,
                                           Result.Akt_Zustand.Um).Zahl_Wert),
                                           Result.Akt_Zustand.Wk);

                                            -- und lsche sie in zk
                                            if Result.Akt_Zustand.Zk /= null then
                                               Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                                            end if;
                              --Ausdr: Verkn
                              when Verkn => Put_Line(ft, "-- (3 : Verknuepfung aufloesen)--");

      --(***)
      -- lse die Verknpfung auf und hnge sie an (oder vor) zk (ohne die Verknpfung)
      Result.Akt_Zustand.Zk := new Teile_Rec'(new Teil_Rec'
                                (Ausdr,Result.Akt_Zustand.Zk.T.Teil_Ausdr.Links_Ausdruck),
                                new Teile_Rec'(new Teil_Rec'
                                 (Ausdr,Result.Akt_Zustand.Zk.T.Teil_Ausdr.Rechts_Ausdruck),
                                 new Teile_Rec'(new Teil_Rec'
                                  (Op,Result.Akt_Zustand.Zk.T.Teil_Ausdr.Operator_Wert),
                                  Result.Akt_Zustand.Zk.Rest)));

                           end case;
            when Bez    => null;     --hier auf string umsteigen
            when Zwert  => null;
            when Wwert  => null;
            --Op
            when Op     => Put_Line(ft, "-- (4 : Arithmetische Operation)--");
                           -- hnge das Ergebnis der Operation mit den letzten beiden
                           -- Werte von wk an wk statt der beiden Werte
                           Result.Akt_Zustand.Wk := new Teile_Rec'
                                                        (new Teil_Rec'(Zwert,Ada_Op
                                                           (Result.Akt_Zustand.Zk.T.Teil_Op,
                                                            Result.Akt_Zustand.Wk.Rest.T.Teil_Zwert,
                                                            Result.Akt_Zustand.Wk.T.Teil_Zwert)),
                                                        Result.Akt_Zustand.Wk.Rest.Rest);
                           -- und lsche den Operator in zk
                           if Result.Akt_Zustand.Zk /= null then
                              Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                           end if;
            -- Rop
            when Rop    => Put_Line(ft, "-- (7 : Vergleichsoperation)--");
                           -- hnge das Ergebnis der Operation mit den letzten beiden
                           -- Werte von wk an wk statt der beiden Werte
                           Result.Akt_Zustand.Wk := new Teile_Rec'
                                                        (new Teil_Rec'(Wwert,Ada_Rel_Op
                                                           (Result.Akt_Zustand.Zk.T.Teil_Rop,
                                                            Result.Akt_Zustand.Wk.Rest.T.Teil_Zwert,
                                                            Result.Akt_Zustand.Wk.T.Teil_Zwert)),
                                                        Result.Akt_Zustand.Wk.Rest.Rest);
                           -- und lsche den Operator in zk
                           Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
            --If_case
            when If_Case => -- Unterscheidung zwischen 13) und 14)
                            -- lsche if
                            Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                            if Result.Akt_Zustand.Wk.T.Teil_Wwert then
                               Put_Line(ft, "-- (13 : If-Anweisung True-Fall)--");
                               -- lsche Anw_f2
                               Result.Akt_Zustand.Zk.Rest := Result.Akt_Zustand.Zk.Rest.Rest;
                            else
                               Put_Line(ft, "-- (14 : If-Anweisung False-Fall)--");
                               -- lsche Anw_f1
                               Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                            end if;
                            -- lsche WWert in wk
                            Result.Akt_Zustand.Wk := Result.Akt_Zustand.Wk.Rest;
            --Do_case
            when Do_Case => -- Unterscheidung zwischen 16) und 17)
                            -- lsche Do_case
                            Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                            if Result.Akt_Zustand.Wk.T.Teil_Wwert then
                               Put_Line(ft, "-- (16 : Schleifenbedingung True)--");

      --(***)
      -- extrahiere Anw_f und erstelle neue Schleife
      Result.Akt_Zustand.Zk := new Teile_Rec'(new Teil_Rec'
                                (Anw_F,Result.Akt_Zustand.Zk.Rest.T.Teil_Anw_F),
                                new Teile_Rec'(new Teil_Rec'
                                 (Anw,new Anweisung_Rec'(Schleife,
                                                         Result.Akt_Zustand.Zk.T.Teil_Bed,
                                                         Result.Akt_Zustand.Zk.Rest.T.Teil_Anw_F)),
                                                         Result.Akt_Zustand.Zk.Rest.Rest));
                            else
                               Put_Line(ft, "-- (17 : Schleifenbedingung False)--");
                               -- lsche SchleifenRest
                               if Result.Akt_Zustand.Zk /= null then
                                  Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest.Rest;
                               end if;
                            end if;
                            -- lsche WWert
                            if Result.Akt_Zustand.Wk /= null then
                               Result.Akt_Zustand.Wk := Result.Akt_Zustand.Wk.Rest;
                            end if;
            --Zuw
            when Zuw     => Put_Line(ft, "-- (10 : Zuweisung ausfuehren)--");
                            -- verndere Umgebung
                            Result.Akt_Zustand.Um := Wert_Eintrag
                                                         (Result.Akt_Zustand.Zk.Rest.T.Teil_Bez,
                                                          Result.Akt_Zustand.Wk.T.Teil_Zwert,
                                                          Result.Akt_Zustand.Um);
                            -- lsche ZWert in wk
                            Result.Akt_Zustand.Wk := Result.Akt_Zustand.Wk.Rest;
                            -- und lsche Zuweisung und Bezeichner in zk
                            Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest.Rest;

         end case;
      else
         Put_Line(ft, "-- (Ende der Auswertung)--");
         EndOfResult.Ende_Zustand := Result.Akt_Zustand.Um;
         Result := EndOfResult;
      end if;

      return Result;
   end Delta2_FT;

end Baumwandler_Package;
