-- Die Prozedur "CompilerInterpreter" ist die "Haupt"-Prozedur unserer
-- Arbeitsgruppe. Sie empfaengt das zu analysierende While-Programm
-- und ruft die verschiedenen Funktionen und Prozeduren aus den 
-- Paketen auf.
-- Das While-Programm kann auf zwei verschiedene Arten uebergeben
-- werden, als Textdatei oder direkt als Eingabe auf der
-- Konsole.
-- Im Ergebnis werden zwei Text-Dateien erstellt, in denen der Syntax-
-- baum und der Trace erstellt wurde.
-- Die Ergebnisse werden ausserdem auf dem Bildschirm ausgegeben.

WITH Text_IO, Ada.Strings.Unbounded, Ada.Strings.Unbounded.Text_IO,
   Baumwandler_Package, PrintPak, Comtypes, ScannerPak;

USE Text_IO, Ada.Strings.Unbounded, Ada.Strings.Unbounded.Text_IO,
   Comtypes;

PROCEDURE CompilerInterpreter IS 


   Eingabe_Programm,
   Inp,             
   TraceD,
   BaumD            : Unbounded_String;              
   Datei_Eingabe,  
   Ausgabe_Baum,  
   Ausgabe_Trace,  
   Dummy            : File_Type;  
   GramBaum         : Comtypes.GrBaum;  
   Bcont            : Boolean          := True; 
   
   -- Resttoken
   pTksLeft : PTOKENLIST := new scannerpak.listOfToken;

   -- Fehlermeldung vom Parser
   pParserErr : PSTRING := new Unbounded_String;


   -- Die Prozedure "HeadLine" erzeugt die berschrift im Hauptprogramm
   PROCEDURE HeadLine IS 
   BEGIN
      Put_Line("   -----------------------------");
      Put_Line("   ---- CompilerInterpreter ----");
      Put_Line("   -----------------------------");
      New_Line;
   END HeadLine;


   -- In der Funktion "Eingabe" wird abgefragt, ob die Eingabe des
   -- While-Programmes durch eine vorhandene Text-Datei oder direkt 
   -- ueber die Konsole erfolgen soll.
   -- Die Funktion gibt True fr die Eingabe per Konsole und False fr die Eingabe
   -- durch eine Datei zurueck. Gross- und Kleinschreibung spielt keine Rolle.
   FUNCTION Eingabe RETURN Boolean IS Input : Unbounded_String;  
   BEGIN
      Put_Line("Wollen sie das Programm per Console oder Datei");
      put     ("einlesen (C/D)?: ");
      Input := Get_Line;
      IF Input="C" OR Input="c" THEN
         RETURN True;
      ELSIF Input="D" OR Input="d" THEN
         RETURN False;
      ELSE
         new_line;
         Put_Line("Unzulaessige Eingabe! Gueltige Eingaben: C, D.");
         New_Line;
         RETURN Eingabe;
      END IF;
   END Eingabe;

   -- Die Funktion "LeseVonConsole" wird aufgerufen, sofern der Anwender die 
   -- Eingabe per Konsole wuenscht. Die Eingabe des Benutzers wird als 
   -- Unbounded_String zurueckgegeben.
   FUNCTION LeseVonConsole (S : String) RETURN Unbounded_String IS 
   BEGIN
      New_Line;
      Put_Line(S);
      RETURN Get_Line;
   END LeseVonConsole;

   -- Die Funktion "LeseAusDatei" wird aufgerufen, wenn der Benutzer die Eingabe
   -- durch eine Datei wuenscht. Der Inhalt der Text-Datei wird als
   -- Unbounded_String zurueckgegeben.
   
     procedure concPF (fn, pf : in unbounded_string; rtn : out unbounded_string) is
      -- Zunchst in String umwandeln
      strFn : String := to_String(fn);
      
      -- Rckgabestring
      strRtn : unbounded_string;
      
      -- position des Punktes?
      PtPos : Natural := 0;
   begin
      -- Namen von hinten nach Punkt durchsuchen
      for pos in reverse strFn'range loop
         -- Punkt?
         if strFn(pos) = '.'then
            PtPos := Pos;
            exit;
         end if;         
      end loop;
      
      -- Punkt gefunden?
      if PtPos /= 0 then
         -- Bis zum Punkt auslesen
         strRtn := to_unbounded_String(strFn(1..PtPos - 1));
         
         -- Anhngen
         Append(strRtn, pf);
         
         -- Endung
         Append(strRtn, strFn(PtPos..strFn'last));
      else
         -- komplett auslesen
         strRtn := to_unbounded_String(strFn);
         
         -- Anhngen
         Append(strRtn, pf);
      end if;
      

      -- An Rckgabe weitergeben
      Rtn := strRtn; 
      
   end concPF;
   procedure LeseAusDatei (Eingabe: out Unbounded_String; Tr, BA : in out Unbounded_String) IS 
      Temp, Temp2 : Unbounded_String;  
   BEGIN
      New_Line;
      Put_Line(
         "Bitte geben Sie den Pfad der Datei im Format: C:\Ordner\Datei.txt an.");
      temp2 := get_line;
      Open(File => Datei_Eingabe, Name => To_String (temp2), Mode => In_File);
      
      WHILE NOT End_Of_File(File => Datei_Eingabe) LOOP         

         WHILE End_Of_Line(File => Datei_Eingabe) LOOP
            Skip_Line(File => Datei_Eingabe);
         END LOOP;

         Temp := Get_Line(Datei_Eingabe);
         Eingabe := Eingabe & ' ' & Temp;

      END LOOP;
      
      Close(File => Datei_Eingabe);
      Concpf(Temp2, To_Unbounded_String("_Baum"), Ba);
      concPF(temp2, to_unbounded_string("_Trace"), Tr);
   end Leseausdatei;
   
   -- Abfrage des Dateinamens der Trace-Datei 
   procedure TraceDatei(traceD : in out unbounded_String) is
   begin
      New_Line;
      Put_Line("Dateiname der zu erstellenden Trace-Datei:");
      Put_Line("(keine Eingabe, wenn keine Datei angelegt werden soll)");
      traceD := Get_Line;
   end Tracedatei;
   
   -- Abfrage des Dateinamens der Baum-Ausgabe-Datei 
   procedure BaumDatei(baumD : in out unbounded_String) is
   begin
      New_Line;
      Put_Line("Dateiname der zu erstellenden Baum-Datei:");
      Put_Line("(keine Eingabe, wenn keine Datei angelegt werden soll)");
      baumD := Get_Line;
   end Baumdatei;
   

BEGIN
   -- Ueberschrift
   Headline;
   New_Line;
   
   -- Die While-Schleife dient dazu, dass bei fehlerhafter Eingabe
   -- solange eine Eingabe gemacht werden kann, bis diese korrekt ist.
   WHILE BCont LOOP
      DECLARE
      BEGIN

         
         if Eingabe then
            Tracedatei(TraceD);
            BaumDatei(baumD);
            Eingabe_Programm := LeseVonConsole
               ("Bitte geben Sie jetzt das Programm als Fliesstext ein: ");
         ELSE
            Leseausdatei(Eingabe_Programm, traceD, BaumD);
            New_Line;
            put_line("Gelesenes Programm aus Datei:");
            put_line(Eingabe_Programm);
         END IF;
         BCont := False;

      EXCEPTION
         WHEN Name_Error =>
            Put_Line("Datei nicht gefunden.");
      END;
   END LOOP;


   -- Es wird eine Tokenliste mit Hilfe Scannerpak erzeugt und diese wird an
   -- die Funktion CreateGB (in Comtypes) uebergeben, um den Grammatik-Baum
   -- gemaess den Spezifikationen in Comtypes zu erzeugen.
   GramBaum := Comtypes.CreateGB(Scannerpak.Scanner(Eingabe_Programm), pTksLeft, pParserErr);

   -- Ist der Grammatikbaum "leer", so liegt das Eingegebene nicht in der While-Sprache.
   -- Dies wird ausgegeben und das Programm endet.
   IF GramBaum = NULL THEN
      New_Line;
      Put_Line("Das eingegebene Programm liegt nicht in der While-Sprache.");
      Put_Line(To_String(pParsererr.All));
      
      -------------------------------------------------------------------------
      -- Bei Bedarf kann hier die Restliste an Token ausgegeben werden
--      Put_Line("Rest-Token-Liste:");
--      
--      scannerpak.druckelistoftoken(pTksLeft.all);
   ELSE

      -- Hier wird der Grammatik-Baum als Baum auf dem Bildschirm ausgegeben
      New_Line;
      Put_Line("Der Grammatikbaum: ");
      PrintPak.PrintGB(GramBaum, Dummy, False);

      -- Im Folgenden werden die Dateien "Ausgabe_Trace.txt" und
      -- "Ausgabe_Baum.txt" erstellt und die jeweiligen File_Types
      -- zusammen mit dem Grammatik-Baum an die Prozeduren ueber-
      -- geben, die ihre Ausgaben dann in die Dateien schreiben.
      -- Zusaetzlich wird der Trace noch auf dem Bildschirm ausge-
      -- geben.
      Put_Line("Der Trace: ");

      -- Datei zum Trace erzeugen
      Create(File => Ausgabe_Trace, Name => to_string(TraceD));
      Baumwandler_Package.Machetrace(Grambaum, Ausgabe_Trace);
      Baumwandler_Package.Machetrace(Grambaum);
      Close(File => Ausgabe_Trace);

      -- Datei zum Baum erzeugen
      Create(File => Ausgabe_Baum, Name => to_string(BaumD));
      PrintPak.PrintGB(GramBaum, Ausgabe_Baum, True);
      Close(File => Ausgabe_Baum);
     
   END IF;
   New_Line(2);
   Put_Line("Moechten Sie ein weiteres Programm eingeben? (j/n)");
   
   Inp := Get_Line;
   if Inp ="J" or Inp ="j"
      then Compilerinterpreter;
      else New_Line(2);
           Put_Line("   -----------------------------");
           Put_Line("   ---- Ende des Programm   ----");
           Put_Line("   -----------------------------");
   end if;

   
-- Fehlerbehandlung: Wird im While-Programm eine Variable nicht
-- initialisiert, so wird dieser Fehler abgefangen und eine
-- entsprechende Fehlermeldung angezeigt.
EXCEPTION
   WHEN Baumwandler_Package.Whileinit_Error =>
      Put_Line("Fehlerhaftes While-Programm! (Initialisierung)");


END CompilerInterpreter;

