----------------------------------------------------------------------------------
----------------------------------------------------------------------------------
-- printpak.adb: Paketimplementierung zur Darstellung eines Programms aus der   --
--                  while-Sprache als Syntaxbaum                                --
-- Version 0.2                                                                  --
--  Mar 2008 by Ingo & Nick                                                    --
----------------------------------------------------------------------------------
----------------------------------------------------------------------------------

with comtypes; use comtypes;

with ada.Text_IO; use ada.Text_IO;

with ada.strings.unbounded; use ada.strings.unbounded;

with ada.Text_IO; use ada.Text_IO;

package body printpak is

   -------------------------------------------------------------------------------
   -- prozedur macheLeerzeichen -- wandelt eine strichliste in eine folge von 
   --                           -- Leerzeichen und ggf. senkrechten Strichen um
   -- parameter -- s : strichliste -- strichliste, die ausgegeben werden soll
   -- Beispiel -- s : strichliste;
   --   s := new strichknoten'(false, null);
   --   s := new strichknoten'(false, s);
   --   s := new strichknoten'(true, s);
   --   s := new strichknoten'(false, s);
   --   s := new strichknoten'(false, s);
   --   s := new strichknoten'(true, s);
   --   
   --   macheLeerzeichen(s); -> "        |     |  |" 
   --       also: 3xLeer & 3xLeer & 2xLeer & | & 3xLeer & 3xLeer & 2xLeer & |
   -------------------------------------------------------------------------------
   function macheLeerzeichen ( s : strichliste ) return unbounded_string is
      leerZ : unbounded_string;
      j     : strichliste      := s;
   begin
      while j /= null loop
         if j.b then
            leerz := leerz & "  |";
         else
            leerz := leerz & "   ";
         end if;
         j := j.next;
      end loop;
      return leerz;
   end macheLeerzeichen;

   -------------------------------------------------------------------------------
   -- prozedur get_Strichlisten_length -- ermittelt die Anzahl der Knoten in einer
   --                                     Strichliste
   -- parameter -- s : strichliste -- strichliste, deren Laenge ermittelt werden 
   --                                 soll
   --
   -- Beispiel -- s : strichliste;
   --   s := new strichknoten'(false, null);
   --   s := new strichknoten'(false, s);
   --   s := new strichknoten'(true, s);
   --   s := new strichknoten'(false, s);
   --   s := new strichknoten'(false, s);
   --   s := new strichknoten'(true, s);
   --   
   --   get_Strichlisten_length(s); -> 6
   -------------------------------------------------------------------------------
   function get_Strichlisten_length(s : strichliste) return integer is
      i : integer := 0;
      helpS : Strichliste := s;
   begin
      if s = null then
         return i;
      end if;
      i := 1;
      while helpS.next /= null loop
         i := i + 1;
         helpS := helpS.next;
      end loop;

      return i;
   end get_Strichlisten_length;

   -------------------------------------------------------------------------------
   -- prozedur reduceList  -- verkuerzt die strichliste bis zum naechsten Knoten 
   --                         mit b = true.
   -- parameter -- s : in out strichliste -- strichliste die verkuerzt werden soll
   -- Beispiel -- s : strichliste;
   --   s := new strichknoten'(false, null);
   --   s := new strichknoten'(false, s);
   --   s := new strichknoten'(true, s);
   --   s := new strichknoten'(false, s);
   --   s := new strichknoten'(false, s);
   --   s := new strichknoten'(true, s);
   --   
   --   reduceList(s); 
   --   s -> strichknoten(true, strichknoten(false, strichknoten(false, null)))
   --   also: die strichliste wird soweit verkuerzt, bis wieder ein b = true vorne
   --         steht
   -------------------------------------------------------------------------------
   procedure reduceList (s : in out strichliste) is
      newS, newS2 : strichliste;
      akt, index : integer := 1;
   begin
      newS := s;
      newS2 := s;
      if newS.next = null then
         newS.b := false;
      else
         while newS.next /= null loop
            if newS.b then
               index := akt;
            end if;
            akt := akt+1;
            newS := newS.next;
         end loop;
         akt := 1;
         while akt /= index loop
            akt := akt + 1;
            newS2 := newS2.next;
         end loop;
         newS2.next := null;
      end if;
   end reduceList;

   -------------------------------------------------------------------------------
   -- prozedur extendlist -- erweitert eine Strichliste um n Elemente, wobei das
   --                        letzte Element den Wert "True" bekommt
   -- parameter -- s : in out strichliste -- strichliste, die verlaengert werden 
   --                                        soll
   -- parameter -- n : integer -- Anzahl der Knoten, um welche s verlaengert wird
   -- Beispiel -- s : strichliste;
   --   s := strichknoten(true, strichknoten(false, strichknoten(false, null)))   
   --   extendList(s, 2);
   --   s -> strichknoten(true,
   --           strichknoten(false,
   --              strichknoten(true,
   --                 strichknoten(false,
   --                    strichknoten(false, null)))))
   --   also: die strichliste wird um einen knoten mit b = false und einen mit
   --         b = true erweitert
   --
   --   s := strichknoten(true, strichknoten(false, strichknoten(false, null)))
   --   extendList(s, 3);
   --   s -> strichknoten(true,
   --           strichknoten(false,
   --              strichknoten(false,
   --                 strichknoten(true,
   --                    strichknoten(false,
   --                       strichknoten(false, null))))))
   --   also: die strichliste wird um zwei knoten mit b = false und einen mit
   --         b = true erweitert

   -------------------------------------------------------------------------------
   procedure extendList (s : in out strichliste; n : integer ) is
      newS : strichliste;
      k    : integer;
   begin
      k := n;
      newS := s;
      while newS.next /= null loop
         newS := newS.next;
      end loop;
      for i in reverse 1..k loop
         if k = 1 then
            news.next := new strichKnoten'(true,null);
            -- der vorderste knoten -> vert. striche (b = true)
         else
            news.next := new strichKnoten'(false,null);
            -- noch nicht der vorderste knoten -> keine vert. striche (b = false)
         end if;
         k := k-1;
         newS := newS.next;
      end loop;
   end extendList;

   -------------------------------------------------------------------------------
   -- prozedur extendlist2 -- erweit./verkuerzt eine strichliste auf die Laenge n
   -- parameter -- s : in out strichliste -- strichliste, die verlaengert werden
   --                                        soll
   -- parameter -- n : integer -- neue Laenge der strichliste
   -- Beispiel -- s : strichliste;
   --   s := strichknoten(true, strichknoten(false, strichknoten(false, null)))      
   --   extendList2(s, 5); 
   --   s -> strichknoten(true,
   --           strichknoten(false,
   --              strichknoten(true,
   --                 strichknoten(false,
   --                    strichknoten(false, null)))))
   --   also: die strichliste wird um einen knoten mit b = false und einen mit
   --         b = true erweitert -> extendList(s, 5 - "laenge von s") wird
   --                               aufgerufen
   --
   --   extendList2(s, 3); 
   --   s -> strichknoten(true, strichknoten(false, strichknoten(false, null)))
   --   also: die eingegebene Liste ist laenger als die gefragte laenge. s muss
   --         erst solange verkuerzt werden(Rekursion), bis dies geaendert ist
   -------------------------------------------------------------------------------
   procedure extendList2 (s : in out strichliste; n : integer ) is
      newS : strichliste;
      length : integer;
   begin
      newS := s;
      length := get_Strichlisten_length(newS);
      if n > length then
         extendList(s, n-length);
      elsif n < length then
         reduceList(s);
         extendList2(s, n);
      end if;
   end extendList2;

   -------------------------------------------------------------------------------
   -- prozedur print -- gibt einen string/charakter in konsole oder txt-datei aus   
   -- parameter -- ausg : File_Type -- uebergebener verweis an eine geoeffnete
   --                                  datei
   -- parameter -- s : String -- ODER s : character -- auszugebener string/char
   -- parameter -- txt : boolean -- true : schreibe in datei
   --                            -- false: schreibe in konsole
   -- Beispiel -- trivial
   -------------------------------------------------------------------------------
   procedure print(ausg: File_Type; s : String; txt : boolean) is
   begin
      if txt then
         put(ausg, s);
      else
         put(s);
      end if;         
   end print;
      
   procedure print(ausg: File_Type; s : Character; txt : boolean) is
   begin
      if txt then
         put(ausg, s);
      else
         put(s);
      end if;  
   end print;


   -------------------------------------------------------------------------------
   -- prozedur AusgabeT -- gibt den eingegebenen GrBaum in baumstruktur aus mit
   --                      der Einschraenkung, dass GrBaum ein Terminal ist.
   -- parameter -- b : in out GrBaum -- Baum der ausgegeben werden soll
   -- parameter -- j : out strichliste -- merkt sich, wie weit eine Zeile
   --              eingerueckt werden muss und wo ggf vertikale Linien eingefgt
   --              werden muessen
   -- parameter -- ebene : in out integer -- gibt an wie weit eingerueckt werden
   --              muss und zwar unabhngig von vertikalen Linien
   -- parameter -- tokenN : String -- Name des Token
   -- parameter -- ausg : File_Type -- siehe procedure print(...)
   -- parameter -- txt : boolean -- siehe procedure print(...)
   --
   -- Beispiel
   -- b : GrBaum; j : strichliste;
   -- b := new GrKnoten'(ZahlT, ..., to_unbounded_string("5"));
   -- j := strichknoten(true,
   --         strichknoten(true,
   --            strichknoten(false,
   --               strichknoten(false, null))))
   -- AusgabeT (b, j, 4, "ZahlT", ausg, txt); -- ausg und txt werden uebergeben,
   --                                 ohne in dieser prozedur etwas zu bewirken
   --
   -- "        |  |_ ZahlT"   -- (zeile 1)
   -- "        |     |"       -- (zeile 2)
   -- "        |     |_ 5"    -- (zeile 3)
   -- "        |"             -- (zeile 4) 
   -------------------------------------------------------------------------------
   procedure AusgabeT (b : in out GrBaum; j : in out strichliste;
                       ebene : in out integer; tokenN : String; ausg : File_Type;
                       txt : boolean) is
      e : Integer := ebene;
      special : boolean;
   begin
      special := tokenN = "NameT" or tokenN = "ZahlT" or tokenN = "ropT" or
                 tokenN = "opT";
      if special then         
         print(ausg, "_ " & tokenN & character'val(10), txt);
         -- Bsp: rest von (zeile 1)
         if b.rest /= null then
            extendList(j,1);
         else
            reduceList(j);
            e := e + 1;
            extendList2(j,e);
         end if;
         print(ausg, to_string(macheleerzeichen(j)), txt); -- Bsp: (zeile 2)
         print(ausg, character'val(10) & to_string(macheleerzeichen(j)), txt);
         -- Bsp: anfang von (zeile 3)
         reduceList(j);        
      end if;
      print(ausg, "_ " & to_String(b.exp), txt); -- Bsp: rest von (zeile 3)
      if not special then
         if b.rest = null then
            reduceList(j);
         end if;
      end if;
      print(ausg, character'val(10) & to_string(macheleerzeichen(j)), txt);
      --Bsp: (zeile 4)
   end AusgabeT;

   -------------------------------------------------------------------------------
   -- prozedur AusgabeAnw -- gibt den eingegebenen GrBaum in baumstruktur aus
   --          mit der Einschraenkung, dass GrBaum eine bedingteAnweisung,
   --          Zuweisung, Schleife oder leere anweisung ist. Extra: Prozedur
   --          faengt ab, ob b.knoten = null ist
   -- parameter -- b : in out GrBaum -- Baum der ausgegeben werden soll
   -- parameter -- j : out strichliste -- merkt sich, wie weit eine Zeile
   --              eingeruekt werden muss und wo ggf vertikale Linien eingefuegt
   --              werden muessen
   -- parameter -- ebene : in out integer -- gibt an wie weit eingerueckt werden
   --              muss und zwar unabhngig von vertikalen Linien
   -- parameter -- tokenN : String -- Name des Token
   -- parameter -- ausg : File_Type -- siehe procedure print(...)
   -- parameter -- txt : boolean -- siehe procedure print(...)
   --
   -- Beispiele
   -- b : GrBaum; j : strichliste;
   -- b := new GrKnoten'(Zuweisung, ..., new GrKnoten'(...));
   -- j := strichknoten(true, strichknoten(true, strichknoten(false, null)))
   -- AusgabeT (b, j, 3, "Zuweisung", ausg, txt); -- ausg und txt werden
   --                     uebergeben, ohne in dieser prozedur etwas zu bewirken
   --
   -- "     |  |_ Zuweisung" -- (zeile 1)
   -- "     |     |"         -- (zeile 2.1)
   --
   --
   -- b : GrBaum; j : strichliste;
   -- b := new GrKnoten'(Zuweisung, ..., null);
   -- j := strichknoten(true, strichknoten(true, strichknoten(false, null)))
   -- AusgabeT (b, j, 3, "Zuweisung", ausg, txt); -- ausg und txt werden
   --                     uebergeben, ohne in dieser prozedur etwas zu bewirken
   --
   -- "     |  |_ Zuweisung"      -- (zeile 1)
   -- "     |     |"              -- (zeile 2.2)
   -- "     |     |_ HIER NULL!"  -- (zeile 3.2)

   -------------------------------------------------------------------------------
   procedure AusgabeAnw (b : in out GrBaum; j : in out strichliste;
                       ebene : in out integer; tokenN : String; ausg : File_Type;
                       txt : boolean) is
      e : integer;
      tmp_strichliste : Strichliste;
   begin
      e := ebene;
      print(ausg, "_ " & tokenN & character'val(10), txt); -- (zeile 1)
      reduceList(j);
      tmp_strichliste := j;
      e := ebene + 1;
      extendList2(j,e);
      if b.knoten = null then
         print(ausg, to_string(macheleerzeichen(j)) & character'val(10) &
               to_string(macheleerzeichen(j)), txt); -- (zeile 2.2)
         print(ausg, "_ HIER NULL!" & character'val(10), txt); -- (zeile 3.2)
         reduceList(j); 
      else
         print(ausg, to_string(macheleerzeichen(j)), txt); -- (zeile 2.1)
         ausgabeN (b.knoten, j, e, ausg, txt);
      end if;
      j := tmp_strichliste;
   end AusgabeAnw;
   

   -------------------------------------------------------------------------------
   -- prozedur AusgabeN -- gibt den eingegebenen GrBaum in baumstruktur aus
   -- parameter -- b : in out GrBaum -- Baum der ausgegeben werden soll
   -- parameter -- j : out strichliste -- merkt sich, wie weit eine Zeile
   --              eingerueckt werden muss und wo ggf vertikale Linien eingefgt
   --              werden muessen
   -- parameter -- ebene : in out integer -- gibt an wie weit eingerueckt werden
   --              muss und zwar unabhngig von vertikalen Linien 
   -- parameter -- ausg : File_Type -- siehe procedure print(...)
   -- parameter -- txt : boolean -- siehe procedure print(...)
   --
   -- Beispielausgaben  (Eingaben in den Beispielen in 'while-sprache')
   --
   -- if 5 = 6 then skip; skip fi
   --
   --Programm
   --  |
   --  |_ AnwF
   --     |
   --     |_ Anw
   --        |
   --        |_ bedingteAnweisung
   --           |
   --           |_ if
   --           |
   --           |_ Bedingung
   --           |  |
   --           |  |_ Ausdruck
   --           |  |  |
   --           |  |  |_ ZahlT
   --           |  |     |
   --           |  |     |_ 5
   --           |  |
   --           |  |_ ropT
   --           |  |  |
   --           |  |  |_ =
   --           |  |
   --           |  |_ Ausdruck
   --           |     |
   --           |     |_ ZahlT
   --           |        |
   --           |        |_ 6
   --           |
   --           |_ then
   --           |
   --           |_ AnwF
   --           |  |
   --           |  |_ Anw
   --           |     |
   --           |     |_ leereAnweisung
   --           |        |
   --           |        |_ skip
   --           |
   --           |_ else
   --           |
   --           |_ AnwF
   --           |  |
   --           |  |_ Anw
   --           |  |  |
   --           |  |  |_ leereAnweisung
   --           |  |     |
   --           |  |     |_ skip
   --           |  |
   --           |  |_ ;
   --           |  |
   --           |  |_ AnwF
   --           |     |
   --           |     |_ Anw
   --           |        |
   --           |        |_ leereAnweisung
   --           |           |
   --           |           |_ skip
   --           |
   --           |_ fi
   
   -------------------------------------------------------------------------------

   -- while a > b do true := 2; b := (a + (2 * b)) od

   --Programm
   --  |
   --  |_ AnwF
   --     |
   --     |_ Anw
   --        |
   --        |_ Schleife
   --           |
   --           |_ while
   --           |
   --           |_ Bedingung
   --           |  |
   --           |  |_ Ausdruck
   --           |  |  |
   --           |  |  |_ NameT
   --           |  |     |
   --           |  |     |_ a
   --           |  |
   --           |  |_ ropT
   --           |  |  |
   --           |  |  |_ >
   --           |  |
   --           |  |_ Ausdruck
   --           |     |
   --           |     |_ NameT
   --           |        |
   --           |        |_ b
   --           |
   --           |_ do
   --           |
   --           |_ AnwF
   --           |  |
   --           |  |_ Anw
   --           |  |  |
   --           |  |  |_ Zuweisung
   --           |  |     |
   --           |  |     |_ NameT
   --           |  |     |  |
   --           |  |     |  |_ true
   --           |  |     |
   --           |  |     |_ :=
   --           |  |     |
   --           |  |     |_ Ausdruck
   --           |  |        |
   --           |  |        |_ ZahlT
   --           |  |           |
   --           |  |           |_ 2
   --           |  |
   --           |  |_ ;
   --           |  |
   --           |  |_ AnwF
   --           |     |
   --           |     |_ Anw
   --           |        |
   --           |        |_ Zuweisung
   --           |           |
   --           |           |_ NameT
   --           |           |  |
   --           |           |  |_ b
   --           |           |
   --           |           |_ :=
   --           |           |
   --           |           |_ Ausdruck
   --           |              |
   --           |              |_ (
   --           |              |
   --           |              |_ Ausdruck
   --           |              |  |
   --           |              |  |_ NameT
   --           |              |     |
   --           |              |     |_ a
   --           |              |
   --           |              |_ opT
   --           |              |  |
   --           |              |  |_ +
   --           |              |
   --           |              |_ Ausdruck
   --           |              |  |
   --           |              |  |_ (
   --           |              |  |
   --           |              |  |_ Ausdruck
   --           |              |  |  |
   --           |              |  |  |_ ZahlT
   --           |              |  |     |
   --           |              |  |     |_ 2
   --           |              |  |
   --           |              |  |_ opT
   --           |              |  |  |
   --           |              |  |  |_ *
   --           |              |  |
   --           |              |  |_ Ausdruck
   --           |              |  |  |
   --           |              |  |  |_ NameT
   --           |              |  |     |
   --           |              |  |     |_ b
   --           |              |  |
   --           |              |  |_ )
   --           |              |
   --           |              |_ )
   --           |
   --           |_ od

   -------------------------------------------------------------------------------
   procedure AusgabeN (b : in out GrBaum; j : out strichliste;
                       ebene : in out integer; ausg : File_Type; txt : boolean) is
      e : integer; -- hilfsvariable zum merken der ebene
      tmp_strichliste : Strichliste;-- hilfsvariable zum merken der strichliste
   begin
      e := ebene; 
      if b /= null then
         print(ausg, character'val(10) & to_string(macheleerzeichen(j)), txt);
         -- neue zeile mit ggf vertikalen linien
         case b.al is

            --------------------
            -- Nicht-Terminal --
            --------------------
            when Programm =>
               print(ausg, "Programm" & character'val(10), txt);
               j := new strichknoten'(True, null);
               print(ausg, to_string(macheleerzeichen(j)), txt);
               ausgabeN(b.knoten, j, e, ausg, txt);


            when AnwF =>
               print(ausg, "_ AnwF" & character'val(10), txt);
               reduceList(j);
               extendList2(j,e);               
               tmp_strichliste := j;               
               if b.rest /= null then -- hat b einen rest auf der selben ebene
                                      -- zb: "AnwF ; Anw"
                  extendList(j,1);
                  e := e + 1;
                  if b.knoten /= null then
                     print(ausg, to_string(macheleerzeichen(j)), txt);
                     ausgabeN (b.knoten, j, e, ausg, txt);
                  end if;
               else
                  if b.knoten /= null then
                     reduceList(j);
                     e := e + 1;
                     extendList2(j,e);
                     print(ausg, to_string(macheleerzeichen(j)), txt);
                     ausgabeN (b.knoten, j, e, ausg, txt);
                  end if;
               end if;               
               e := get_Strichlisten_length(j);
               j := tmp_strichliste;
            

            when Ausdruck =>
               print(ausg, "_ Ausdruck" & character'val(10), txt);
               tmp_strichliste := j;
               if b.rest = null then                                 
                  reduceList(j);                  
               end if;
               e := ebene + 1;
                  extendList2(j,e);
                  print(ausg, to_string(macheleerzeichen(j)), txt);
                  ausgabeN (b.knoten, j, e, ausg, txt);
               e := ebene; -- ebene wird zurueckgesetzt
               j := tmp_strichliste;


            when Bedingung =>
               print(ausg, "_ Bedingung" & character'val(10), txt);
               e:= ebene + 1;
               extendList2(j,e);
               tmp_strichliste := j;
               print(ausg, to_string(macheleerzeichen(j)), txt);
               ausgabeN (b.knoten, j, e, ausg, txt);
               e := ebene; -- ebene wird zurueckgesetzt
               j := tmp_strichliste;           
            

            when Anw => 
               print(ausg, "_ Anw" & character'val(10), txt);
               tmp_strichliste := j;               
               if b.rest /= null then
                  extendList(j,1);
                  e := e + 1;
                  if b.knoten /= null then
                     print(ausg, to_string(macheleerzeichen(j)), txt);
                     ausgabeN (b.knoten, j, e, ausg, txt);
                  end if;
               else
                  if b.knoten /= null then
                     reduceList(j);
                     e := e + 1;
                     extendList2(j,e);                     
                     print(ausg, to_string(macheleerzeichen(j)), txt);
                     ausgabeN (b.knoten, j, e, ausg, txt);
                  end if;
               end if;               
               e := get_Strichlisten_length(j);
               j := tmp_strichliste;
               
               
            when leereAnweisung    =>
               AusgabeAnw (b, j, e, "leereAnweisung", ausg, txt);   
            when Zuweisung         =>
               AusgabeAnw (b, j, e, "Zuweisung", ausg, txt);               
            when bedingteAnweisung =>
               AusgabeAnw (b, j, e, "bedingteAnweisung", ausg, txt);
            when Schleife          =>
               AusgabeAnw (b, j, e, "Schleife", ausg, txt);


            --------------
            -- Terminal --
            --------------
            when ifT        => AusgabeT (b, j, e, "ifT", ausg, txt);
            when nameT      => AusgabeT (b, j, e, "NameT", ausg, txt);
            when skipT      => AusgabeT (b, j, e, "skipT", ausg, txt);
            when whilet     => AusgabeT (b, j, e, "whileT", ausg, txt);
            when zahlT      => AusgabeT (b, j, e, "ZahlT", ausg, txt);
            when ropT       => AusgabeT (b, j, e, "ropT", ausg, txt);
            when opT        => AusgabeT (b, j, e, "opT", ausg, txt);
            when zuweisungT => AusgabeT (b, j, e, "ZuweisungsT", ausg, txt);
            when thenT      => AusgabeT (b, j, e, "thenT", ausg, txt);
            when elseT      => AusgabeT (b, j, e, "elseT", ausg, txt);
            when fiT        => AusgabeT (b, j, e, "fiT", ausg, txt);
            when doT        => AusgabeT (b, j, e, "doT", ausg, txt);
            when odT        => AusgabeT (b, j, e, "odT", ausg, txt);
            when trueT      => AusgabeT (b, j, e, "TrueT", ausg, txt);
            when falseT     => AusgabeT (b, j, e, "FalseT", ausg, txt);
            when semikolonT => AusgabeT (b, j, e, "semikolonT", ausg, txt);
            when aufT       => AusgabeT (b, j, e, "aufT", ausg, txt);
            when zuT        => AusgabeT (b, j, e, "zuT", ausg, txt);
            when others     => AusgabeT (b, j, e, "skipT", ausg, txt);
                           
         end case;
         ausgabeN(b.rest, j, e, ausg, txt); -- naechster knoten
      end if;
   end AusgabeN;
   

   -------------------------------------------------------------------------------
   -- prozedur printGB -- Schnittstelle des Pakets nach aussen
   -- parameter -- gb : GrBaum -- Baum der ausgegeben werden soll
   -- parameter -- ausg : File_Type -- siehe procedure print(...)
   -- parameter -- txt : boolean -- siehe procedure print(...)
   --
   -- Beispiel - trivial: siehe ausgabeN(...)
   -------------------------------------------------------------------------------
   procedure printGB (gb : GrBaum; ausg : File_Type; txt : boolean) is
      strichzeiger : strichliste := null;
      ebene_anf : integer := 1;         
      -- Zeiger auF Baum
      pgb : GrBaum := gb;
   begin
      ausgabeN(pgb, strichzeiger, ebene_anf, ausg, txt);
   end printGB;

end printpak;