with Text_IO; use Text_IO;

procedure EndlicheAutomaten is

   type detueber is
     array (Integer range <>,
            Character range <>) of Integer;
   -- ordnet einem Zustand und einem Zeichen
   -- einen Nachfolgezustand zu

   type det_ueber_type (n : Positive) is record
      ueber : detueber (0..n,' ' .. '~');
   end record;
   -- Einschraenkung auf Zustaende 0..n-1 und Fangzustand n
   -- (damit die Uebergangsfunktion total ist);
   -- T ist die Menge der druckbaren ASCII-Zeichen

   type charFktn is
     array (Integer range <>) of Boolean;
   -- Charakteristische Funktion fuer Mengen
   -- ganzer Zahlen

   type charFktn_type (n : Positive) is record
      inh : charFktn (0..n) := (others => False);
   end record;
   -- beschraenkt die Obermenge auf die Zustaende
   -- 0 bis n;
   -- Initialisierung beschreibt die leere Menge

   type DEA_type (n : Positive) is record
      det : det_ueber_type (n);
      F   : charFktn_type (n);
   end record;
   -- beschreibt einen deterministischen endlichen
   -- Automaten mit
   --   Q = {0,1,...,n-1,n} (n ist Fangzustand)
   --   q0 = 0
   --   T = Menge der druckbaren ASCII-Zeichen
   --   delta = det
   --   F = F

   DEA1 : DEA_type (4);
   
   type DEA_ref is access DEA_Type;
   -- nuetzlich, wenn man z.B. mehrere DEAs in einem
   -- Array speichern moechte

   procedure pruefe (DEA : DEA_type; w : String) is
      akt : Integer := 0; -- aktueller Zustand
   begin
      for i in 1 .. w'last loop
         akt := DEA.det.ueber (akt, w(i));
      end loop;
      put ("""" &w& """" & " gehoert");
      if not DEA.F.inh(akt) then
         put (" nicht");
      end if;
      put (" zum Sprachschatz");
      new_line(2);
   end pruefe;

   ------------------------------------------------

   type zustandsMenge is access charFktn_type;
   -- zeigt auf eine Menge von Zustaenden

   type ndetueber is
     array (Integer range <>,
            Character range <>) of zustandsMenge;
   -- ordnet einem Zustand und einem Zeichen
   -- eine Menge von Nachfolgezustaenden zu

   type ndet_ueber_type (n : Positive) is record
      ueber : ndetueber (0..n,'!' .. Standard.ASCII.del);
   end record;
   -- Einschraenkung auf Zustaende 0..n ;
   -- T ist die Menge der druckbaren ASCII-Zeichen
   -- plus Standard.ASCII.del als "epsilon"

   type NDEA_type (n : Positive) is record
      ndet : ndet_ueber_type (n);
      F    : charFktn_type (n);
   end record;
   -- beschreibt einen deterministischen endlichen
   -- Automaten mit
   --   Q = {0,1,...,n-1,n}
   --   q0 = 0
   --   T = Menge der druckbaren ASCII-Zeichen
   --   delta = ndet
   --   F = F

   eps : constant Character := Standard.ASCII.del;
   -- Repraesentiert das leere Wort (!!!)

   NDEA1 : NDEA_type (5);
   
   type NDEA_ref is access NDEA_Type;
   -- nuetzlich, wenn man z.B. mehrere NDEAs in einem
   -- Array speichern moechte


--     function myhill (NDEA : NDEA_type) return DEA_type is
--
--     begin
--
--     end myhill;

begin

   DEA1.det.ueber :=
     (0 => ('a' => 1, 'b' => 2, others => 4),
      1 => ('a' => 0, 'b' => 3, others => 4),
      2 => ('a' => 3, 'b' => 0, others => 4),
      3 => ('a' => 2, 'b' => 1, others => 4),
      4 => (others => 4));

   DEA1.F.inh := (3 => True, others => False);

   pruefe (DEA1, "a");
   pruefe (DEA1, "abb");
   pruefe (DEA1, "abaa");
   pruefe (DEA1, "abbb");
   pruefe (DEA1, "ab");
   pruefe (DEA1, "ab ");
   
   -- Ergebnis des Programmlaufs:

   -- "a" gehoert nicht zum Sprachschatz

   -- "abb" gehoert nicht zum Sprachschatz

   -- "abaa" gehoert zum Sprachschatz

   -- "abbb" gehoert zum Sprachschatz

   -- "ab" gehoert zum Sprachschatz

   -- "ab " gehoert nicht zum Sprachschatz

   ---------------------------------------------------

   NDEA1.ndet.ueber :=
     (0 => (eps =>    new charFktn_type'(5, (1 | 3 => True, others => False)),
            -- von Zustand 0 aus mit eps nach {1,3}
            others => new charFktn_type'(5, (others => False))),
            -- von Zustand 0 aus keine weiteren Uebergaenge
      1 => ('a' =>    new charFktn_type'(5, (2 => True, others => False)),
            -- von Zustand 1 aus mit 'a' nach {2}
            others => new charFktn_type'(5, (others => False))),
            -- von Zustand 1 aus keine weiteren Uebergaenge
      2 => (eps =>    new charFktn_type'(5, (5 => True, others => False)),
            -- von Zustand 2 aus mit eps nach {5}
            others => new charFktn_type'(5, (others => False))),
            -- von Zustand 2 aus keine weiteren Uebergaenge
      3 => ('b' =>    new charFktn_type'(5, (4 => True, others => False)),
            -- von Zustand 3 aus mit 'b' nach {4}
            others => new charFktn_type'(5, (others => False))),
            -- von Zustand 3 aus keine weiteren Uebergaenge
      4 => (eps =>    new charFktn_type'(5, (5 => True, others => False)),
            -- von Zustand 4 aus mit eps nach {5}
            others => new charFktn_type'(5, (others => False))),
            -- von Zustand 4 aus keine weiteren Uebergaenge
      5 => (others => new charFktn_type'(5, (others => False))));
            -- von Zustand 5 aus keine Uebergaenge

   NDEA1.F.inh := (5 => True, others => False);
   -- der einzige Endzustand ist 5, d.h. F = {5}


end EndlicheAutomaten;
