with Text_Io; use Text_Io;

procedure Aufgabe2 is
   
   -------------------------------------------------
   -- Vorgaben
   -------------------------------------------------

type detueber is
     array (Integer range <>,
            Character range <>) of Integer;
   -- ordnet einem Zustand und einem Zeichen
   -- einen Nachfolgezustand zu

   type det_ueber_type (n : Positive) is record
      ueber : detueber (0..n,' ' .. '~');
   end record;
   -- Einschraenkung auf Zustaende 0..n-1 und Fangzustand n
   -- (damit die Uebergangsfunktion total ist);
   -- T ist die Menge der druckbaren ASCII-Zeichen

   type charFktn is
     array (Integer range <>) of Boolean;
   -- Charakteristische Funktion fuer Mengen
   -- ganzer Zahlen

   type charFktn_type (n : Positive) is record
      inh : charFktn (0..n) := (others => False);
   end record;
   -- beschraenkt die Obermenge auf die Zustaende
   -- 0 bis n;
   -- Initialisierung beschreibt die leere Menge

   type DEA_type (n : Positive) is record
      det : det_ueber_type (n);
      F   : charFktn_type (n);
   end record;
   -- beschreibt einen deterministischen endlichen
   -- Automaten mit
   --   Q = {0,1,...,n-1,n} (n ist Fangzustand)
   --   q0 = 0
   --   T = Menge der druckbaren ASCII-Zeichen
   --   delta = det
   --   F = F

   DEA1 : DEA_type (4);
   
   type DEA_ref is access DEA_Type;
   -- nuetzlich, wenn man z.B. mehrere DEAs in einem
   -- Array speichern moechte

   procedure pruefe (DEA : DEA_type; w : String) is
      akt : Integer := 0; -- aktueller Zustand
   begin
      for i in 1 .. w'last loop
         akt := DEA.det.ueber (akt, w(i));
      end loop;
      put ("""" &w& """" & " gehoert");
      if not DEA.F.inh(akt) then
         put (" nicht");
      end if;
      put (" zum Sprachschatz");
      new_line(2);
   end pruefe;

   ------------------------------------------------

   type zustandsMenge is access charFktn_type;
   -- zeigt auf eine Menge von Zustaenden

   type ndetueber is
     array (Integer range <>,
            Character range <>) of zustandsMenge;
   -- ordnet einem Zustand und einem Zeichen
   -- eine Menge von Nachfolgezustaenden zu

   type ndet_ueber_type (n : Positive) is record
      ueber : ndetueber (0..n,'!' .. Standard.ASCII.del);
   end record;
   -- Einschraenkung auf Zustaende 0..n ;
   -- T ist die Menge der druckbaren ASCII-Zeichen
   -- plus Standard.ASCII.del als "epsilon"

   type NDEA_type (n : Positive) is record
      ndet : ndet_ueber_type (n);
      F    : charFktn_type (n);
   end record;
   -- beschreibt einen deterministischen endlichen
   -- Automaten mit
   --   Q = {0,1,...,n-1,n}
   --   q0 = 0
   --   T = Menge der druckbaren ASCII-Zeichen
   --   delta = ndet
   --   F = F
   
   type NDEA_ref is access NDEA_Type;

   eps : constant Character := Standard.ASCII.del;
   -- Repraesentiert das leere Wort (!!!)
   
   -------------------------------------------------
   -- Ende Vorgaben
   -------------------------------------------------
   
   -------------------------------------------------
   -- Eigene Deklarationen
   -------------------------------------------------

   -------------------------------------------------
   -- Ausnahmebehandlung
   --
   -------------------------------------------------
   Grundmengen_Verschieden : exception; 
   Index_zu_gross : exception;  

   -------------------------------------------------
   -- LeereMenge
   --
   -- erzeugt eine neue charakteristische Fkt
   -- auf der Halde die der charakteristischen Fkt 
   -- der leeren Menge auf der
   -- Grundmenge 0..n entspricht.
   -------------------------------------------------
   
   function LeereMenge(N:Positive) return ZustandsMenge is
   begin
      return new CharFktn_Type'(N=>N, inh=>(others=>False));
   end LeereMenge;

   -------------------------------------------------
   -- istLeer
   --
   -- gibt genau dass TRUE zurck, wenn die Menge
   -- ausschlielich den Wert FALSE enthlt.
   -------------------------------------------------

   function istLeer(Z:ZustandsMenge) return boolean is
   begin
        for i in Z.inh'range loop
            if Z.inh(i) then return false; end if;
        end loop;
        return true;
   end istLeer;


   -------------------------------------------------
   -- hinzu
   --
   -- fgt den Zustand i zur Zustandsmenge z hinzu.
   -------------------------------------------------
   
   procedure hinzu(i: Integer; z: in out Zustandsmenge) is
   begin
      if not (i in z.inh'range) then raise Index_zu_gross; end if;
      z.inh(i) := true;
   end;

   -------------------------------------------------
   -- +
   --
   -- gibt die Vereinigung zweier Zustandsmengen 
   -- zurck.
   -- Es wird vorausgesetzt, dass z1 und z2 als
   -- charakteristische Fkt auf der gleichen Grund-
   -- menge initialisiert sind.
   -- Sonst wird Grundmengen_Verschieden geworfen.
   -------------------------------------------------

   function "+"(Z1,Z2: ZustandsMenge) return Zustandsmenge is
         z3 : Zustandsmenge := LeereMenge(z1.inh'length-1);
   begin
      if (Z1.Inh'Length /= Z2.Inh'Length) then
         raise Grundmengen_Verschieden;
      end if;
      
        for i in z2.inh'range loop
            z3.Inh(I):= z1.Inh(I) or z2.Inh(I);
        end loop;
        return z3;
   end;


   -------------------------------------------------
   -- =
   --
   -- gibt genau dann TRUE zurck, wenn z1 und z2
   -- identisch sind.
   -- Es wird vorausgesetzt, dass z1 und z2 als
   -- charakteristische Fkt auf der gleichen Grund-
   -- menge initialisiert sind.
   -- Sonst wird Grundmengen_Verschieden geworfen.
   -------------------------------------------------   

   function "="(z1,z2: ZustandsMenge) return boolean is
   begin
      if (Z1.Inh'Length /= Z2.Inh'Length) then
         raise Grundmengen_Verschieden;
      end if;      
        for i in z1.inh'range loop
            if z1.inh(i) /= z2.inh(i) then return false; end if;
            end loop;
        return true;
   end "=";

    -------------------------------------------------
   -- Drucke
   --
   -- Druckt die Teilmenge, die durch die charakt.
   -- Fkt auf die Z zeigt definiert ist in
   -- Mengendarstellung. 
   -------------------------------------------------

   procedure Drucke(Z:ZustandsMenge) is
      mengeGroesserEins:boolean:=false;
   begin
      Put("{");
      for I in Z.Inh'range loop
         if Z.Inh(I) then
            if MengeGroesserEins then
            Put(",");
            end if;
            Put(Positive'Image(I));
            if not MengeGroesserEins then
               MengeGroesserEins := True;
               end if;
         end if;
      end loop;
      Put_Line("}");
   end Drucke;
   
   
   -------------------------------------------------
   -- eHuelleVon
   --
   -- Gibt zu einem Zustand die Epsilonhlle in 
     -- Bezug auf den bergebenen Automaten zurck.
   -------------------------------------------------
   

   function eHuelleVon(q: Integer; automat: NDEA_ref) return zustandsMenge is
       eHuelle: zustandsMenge := leereMenge (automat.F.inh'length-1);
       geaendert: boolean := true;
       aktneu: zustandsMenge;
   begin
       eHuelle := new Charfktn_type'(Automat.NDet.Ueber(Q,Eps).All);
       hinzu(q,eHuelle);
       while geaendert loop
             geaendert := false ;
	     for p in eHuelle.inh'range loop
	         if not eHuelle.inh(p) then null;
	            else aktneu := automat.nDet.ueber(p,eps);
    		       for z in aktneu.inh'range loop
			     if aktneu.inh(z) and then not eHuelle.inh(z) then
				  hinzu(z,eHuelle);
				  geaendert  := true; 
                   end if;
                end loop;
	         end if;
             end loop;
       end loop;
       return eHuelle;
   end eHuelleVon;
   
   -------------------------------------------------
   -- Eigene Variablen
   --
   -------------------------------------------------

   Ndea_2oder3 : Ndea_Type(5);
   Ndea_2o3 :    Ndea_Type(5);
   Ndea_eps :    Ndea_Type(5);
   ref2oder3:    Ndea_ref;
   ref2o3 :      Ndea_ref;
   refeps:       Ndea_ref;
   Z1,Z2 : zustandsMenge;


begin

   -------------------------------------------------
   -- NDEA_2oder3 erzeugen
   --
   -- belegt NDEA_2oder3 so, dass es dem Automaten
   -- A Index 2 oder 3 (S. 115) entspricht.
   -------------------------------------------------

   NDEA_2oder3.nDet.ueber :=
      (
      0 => ('a' => new CharFktn_Type'(N=>5, Inh=>(1|3=>True, others=>False)),others => LeereMenge(5)),
      1 => ('a' => new CharFktn_Type'(N=>5, Inh=>(2=>True, others=>False)),others=>LeereMenge(5)),
      2 => ('a' => new CharFktn_Type'(N=>5, Inh=>(1=>True, others=>False)),others=>LeereMenge(5)),
      3 => ('a' => new CharFktn_Type'(N=>5, Inh=>(4=>True, others=>False)),others=>LeereMenge(5)),
      4 => ('a' => new CharFktn_Type'(N=>5, Inh=>(5=>True, others=>False)),others=>LeereMenge(5)),
      5 => ('a' => new CharFktn_Type'(N=>5, Inh=>(3=>True, others=>False)),others=>LeereMenge(5))
      );
   NDEA_2oder3.F.inh:=(0|2|5 => true, others=>false);

   Drucke(Ndea_2oder3.NDet.Ueber(0,'a'));
     -- druckt die Menge der Zustaende d(0,a) von A2oder3            
     -- Ausgabe: { 1, 3}

   Ref2oder3 := new Ndea_Type(5);
   Ref2oder3.Ndet := (Ndea_2oder3.Ndet);
     -- Zeiger zum Automaten erzeugen   

   -------------------------------------------------
   -- NDEA_2/3 erzeugen
   --
   -- belegt NDEA_2o3 so, dass es dem Automaten
   -- A Index 2/3 (S. 120) entspricht.
   -------------------------------------------------

   NDEA_2o3.nDet.ueber :=
      (
      0 => (eps => new CharFktn_Type'(N=>5, Inh=>(1|3=>True, others=>False)),others => LeereMenge(5)),
      1 => ('a' => new CharFktn_Type'(N=>5, Inh=>(2=>True, others=>False)),others=>LeereMenge(5)),
      2 => ('a' => new CharFktn_Type'(N=>5, Inh=>(1=>True, others=>False)),others=>LeereMenge(5)),
      3 => ('a' => new CharFktn_Type'(N=>5, Inh=>(4=>True, others=>False)),others=>LeereMenge(5)),
      4 => ('a' => new CharFktn_Type'(N=>5, Inh=>(5=>True, others=>False)),others=>LeereMenge(5)),
      5 => ('a' => new CharFktn_Type'(N=>5, Inh=>(3=>True, others=>False)),others=>LeereMenge(5))
      );
   NDEA_2o3.F.inh:=(0|1|3 => true, others=>false);
      
   Drucke(Ndea_2o3.NDet.Ueber(0,Eps));
     -- druckt die Menge der Zustaende d(0,eps) von A2/3    
     -- Ausgabe: { 1, 3}
     
   Ref2o3 := new Ndea_Type(5);
   Ref2o3.Ndet := (Ndea_2o3.Ndet);   
     -- Zeiger zum Automaten erzeugen
 
   -------------------------------------------------
   -- NDEA_eps erzeugen
   --
   -- belegt NDEA_eps so, dass es den vorhergehenden
   -- Automaten entspricht, wobei nur der  Terminal
   -- eps belegt wird. Dieser NDEA dient zum Test
   -- der Funktion eHuelleVon.
   -------------------------------------------------

   NDEA_eps.nDet.ueber :=
      (
      0 => (eps => new CharFktn_Type'(N=>5, Inh=>(1|3=>True, others=>False)),others => LeereMenge(5)),
      1 => (eps => new CharFktn_Type'(N=>5, Inh=>(2=>True, others=>False)),others=>LeereMenge(5)),
      2 => (eps => new CharFktn_Type'(N=>5, Inh=>(1=>True, others=>False)),others=>LeereMenge(5)),
      3 => (eps => new CharFktn_Type'(N=>5, Inh=>(4=>True, others=>False)),others=>LeereMenge(5)),
      4 => (eps => new CharFktn_Type'(N=>5, Inh=>(5=>True, others=>False)),others=>LeereMenge(5)),
      5 => (eps => new CharFktn_Type'(N=>5, Inh=>(3=>True, others=>False)),others=>LeereMenge(5))
      );
   NDEA_eps.F.inh:=(0|1|3 => true, others=>false);

   Refeps := new Ndea_Type(5);
   Refeps.Ndet := (Ndea_Eps.Ndet);
     -- Zeiger zum Automaten erzeugen   

   -------------------------------------------------
   -- Aufrufe der Hilfsprogramme mit Ergebnissen
   --
   -------------------------------------------------
      
   Z1 := LeereMenge(3);
   Drucke(Z1);
     -- Erzeugt eine leere Menge und druckt sie.
     -- Ausgabe: {}
   Put_Line(Boolean'Image(Istleer(Z1)));  
     -- Prft ob die Menge leer ist.
     -- Ausgabe: TRUE
   Hinzu(3,Z1);
   Drucke(Z1);
     -- Fgt ein Element zur Menge hinzu und gibt
     -- die genderte Menge aus.
     -- Ausgabe: { 3}   
   Z2 := new CharFktn_Type'(N=>3, Inh=>(3=>True,others=>False));
   Put_Line(Boolean'Image(Z1=Z2));
     -- Erzeugt eine aequivalente Menge durch Aggregate.
     -- Prft die beiden Mengen auf Gleichheit.
     -- Ausgabe: TRUE    
   Hinzu(1,Z2);
   Put_Line(Boolean'Image(Z1=Z2));
     -- ndert eine Menge und vergleicht erneut.
     -- Ausgabe: FALSE
   Drucke(Z1+Z2);
     -- Bildet die Vereinigung beider Mengen.
     -- Ausgabe: { 1, 3} 
   
   Drucke(EHuelleVon(0,Ref2oder3));
     -- Zeigt die Eps-Huelle vom Zustand 0 des 2oder3 Automaten.
     -- Diese besteht nur aus dem Zustand 0.
     -- Ausgabe: { 0}
   Drucke(EHuelleVon(0,Ref2o3));
     -- Zeigt die Eps-Huelle vom Zustand 0 des 2/3 Automaten.
     -- Diese besteht aus den Zustaenden 0, 1 und 3.
     -- Ausgabe: { 0, 1, 3}
   Drucke(eHuelleVon(3,refeps));
     -- Zeigt die Eps-Huelle vom Zustand 3 des eps-Test Automaten.
     -- Diese besteht aus den Zustaenden 3, 4 und 5.
     -- Ausgabe: { 3, 4, 5}

end Aufgabe2;

