with Text_Io;
use Text_Io;

procedure Aufgabe3 is 

   -------------------------------------------------
   -- Vorgaben
   -------------------------------------------------

   type Detueber is array (Integer range <>, Character range <>) of Integer; 
   -- ordnet einem Zustand und einem Zeichen
   -- einen Nachfolgezustand zu

   type Det_Ueber_Type 
         (N : Positive) is 
      record 
         Ueber : Detueber (0 .. N, '!' .. '~');  
      end record; 
   -- Einschraenkung auf Zustaende 0..n-1 und Fangzustand n
   -- (damit die Uebergangsfunktion total ist);
   -- T ist die Menge der druckbaren ASCII-Zeichen

   type Charfktn is array (Integer range <>) of Boolean; 
   -- Charakteristische Funktion fuer Mengen
   -- ganzer Zahlen

   type Charfktn_Type 
         (N : Positive) is 
      record 
         Inh : Charfktn (0 .. N) := (others => False);  
      end record; 
   -- beschraenkt die Obermenge auf die Zustaende
   -- 0 bis n;
   -- Initialisierung beschreibt die leere Menge

   type Dea_Type 
         (N : Positive) is 
      record 
         Det : Det_Ueber_Type (N);  
         F   : Charfktn_Type (N);  
      end record; 
   -- beschreibt einen deterministischen endlichen
   -- Automaten mit
   --   Q = {0,1,...,n-1,n} (n ist Fangzustand)
   --   q0 = 0
   --   T = Menge der druckbaren ASCII-Zeichen
   --   delta = det
   --   F = F

   -- DEA1 : DEA_Type (4);  

   type DEA_Ref is access DEA_Type; 
   -- ntzlich, wenn man z.B. mehrere DEAs in einem
   -- Array speichern moechte (siehe Scanner!)

   procedure Pruefe (
         Dea : Dea_Type; 
         W   : String    ) is 
      Akt : Integer := 0; -- aktueller Zustand                                               
   begin
      for I in 1 .. W'Last loop
         Akt := Dea.Det.Ueber (Akt, W(I));
      end loop;
      Put ("""" &W& """" & " gehoert");
      if not Dea.F.Inh(Akt) then
         Put (" nicht");
      end if;
      Put (" zum Sprachschatz");
      New_Line(2);
   end Pruefe;

   -------------------------------------------------

   Eps : constant Character := Standard.Ascii.Del;  
   -- Repraesentiert das leere Wort (!!!)
   
   type Zustandsmenge is access Charfktn_Type; 
   -- zeigt auf eine Menge von Zustaenden

   type Ndetueber is array (Integer range <>, Character range <>) of Zustandsmenge; 
   -- ordnet einem Zustand und einem Zeichen
   -- eine Menge von Nachfolgezustaenden zu

   type Ndet_Ueber_Type 
         (N : Positive) is 
      record 
         Ueber : Ndetueber (0 .. N, '!' .. Eps);  
      end record; 
   -- Einschraenkung auf Zustaende 0..n ;
   -- T ist die Menge der druckbaren ASCII-Zeichen
   -- plus Standard.ASCII.del als "epsilon"

   type Ndea_Type 
         (N : Positive) is 
      record 
         Ndet : Ndet_Ueber_Type (N);  
         F    : Charfktn_Type (N);  
      end record; 
   -- beschreibt einen deterministischen endlichen
   -- Automaten mit
   --   Q = {0,1,...,n-1,n}
   --   q0 = 0
   --   T = Menge der druckbaren ASCII-Zeichen
   --   delta = ndet
   --   F = F

   type Ndea_Ref is access Ndea_Type; 
   -- nuetzlich, wenn man z.B. mehrere NDEAs in einem
   -- Array speichern moechte

   -------------------------------------------------
   -- Ende Vorgaben
   -------------------------------------------------


   -------------------------------------------------
   -- Import aus Aufgabe2
   -------------------------------------------------

   -------------------------------------------------
   -- Ausnahmebehandlung
   --
   -------------------------------------------------
   Grundmengen_Verschieden : exception;
   Ungueltige_Menge : exception; 
   Index_zu_gross : exception;  

   -------------------------------------------------
   -- LeereMenge
   --
   -- erzeugt eine neue charakteristische Fkt
   -- auf der Halde die der charakteristischen Fkt 
   -- der leeren Menge auf der
   -- Grundmenge 0..n entspricht.
   -------------------------------------------------
   
   function LeereMenge(N:Positive) return ZustandsMenge is
   begin
      return new CharFktn_Type'(N=>N, inh=>(others=>False));
   end LeereMenge;

   -------------------------------------------------
   -- hinzu
   --
   -- fgt den Zustand i zur Zustandsmenge z hinzu.
   -------------------------------------------------
   
   procedure hinzu(i: Integer; z: in out Zustandsmenge) is
   begin
      if Z = null then raise Ungueltige_Menge; end if;
      if not (i in z.inh'range) then raise Index_zu_gross; end if;
      z.inh(i) := true;
   end;

   -------------------------------------------------
   -- +
   --
   -- gibt die Vereinigung zweier Zustandsmengen 
   -- zurck.
   -- Es wird vorausgesetzt, dass z1 und z2 als
   -- charakteristische Fkt auf der gleichen Grund-
   -- menge initialisiert sind.
   -- Sonst wird Grundmengen_Verschieden geworfen.
   -------------------------------------------------

   function "+"(z1,z2: ZustandsMenge) return Zustandsmenge is
      z3 : Zustandsmenge := LeereMenge(z1.inh'length-1);
   begin
      if (z1.Inh'Length /= z2.Inh'Length) then
         raise Grundmengen_Verschieden;
      end if;
      for i in z1.inh'range loop
         z3.Inh(I):= z1.Inh(I) or z2.Inh(I);
      end loop;
      return z3;
   end;

   -------------------------------------------------
   -- =
   --
   -- gibt genau dann TRUE zurck, wenn z1 und z2
   -- identisch sind.
   -- Es wird vorausgesetzt, dass z1 und z2 als
   -- charakteristische Fkt auf der gleichen Grund-
   -- menge initialisiert sind.
   -- Sonst wird Grundmengen_Verschieden geworfen.
   -------------------------------------------------   

   function "="(z1,z2: ZustandsMenge) return boolean is
   begin
      --if (z1 = null) then raise Ungueltige_Menge; end if;
      if (Z1.Inh'Length /= Z2.Inh'Length) then
         raise Grundmengen_Verschieden;
      end if;      
      for i in z1.inh'range loop
         if z1.inh(i) /= z2.inh(i) then return false; end if;
      end loop;
      return true;
   end "=";

   -------------------------------------------------
   -- Epsilonhlle
   --
   -- Gibt zu einem Zustand die Epsilonhlle in 
   -- Bezug auf den bergebenen Automaten zurck.
   -------------------------------------------------

   function EHuelleVon (
         Q       : Integer; 
         Automat : NDEA_Ref ) 
     return ZustandsMenge is 
      EHuelle   : ZustandsMenge := LeereMenge (Automat.F.Inh'Length-1);  
      Geaendert : Boolean       := True;  
      Aktneu    : ZustandsMenge;  
   begin
      EHuelle := new Charfktn_type'(Automat.NDet.Ueber(Q,Eps).All);
      Hinzu(Q,EHuelle);
      while Geaendert loop
         Geaendert := False ;
         for P in EHuelle.Inh'range loop
            if not EHuelle.Inh(P) then
               null;
            else
               Aktneu := Automat.NDet.Ueber(P,Eps);
               for Z in Aktneu.Inh'range loop
                  if Aktneu.Inh(Z) and then not EHuelle.Inh(Z) then
                     Hinzu(Z,EHuelle);
                     Geaendert  := True;
                  end if;
               end loop;
            end if;
         end loop;
      end loop;
      return EHuelle;
   end EHuelleVon;

   -------------------------------------------------
   -- Ende Import aus Aufgabe2
   -------------------------------------------------
   

   -------------------------------------------------
   -- Eigene Deklarationen
   -------------------------------------------------

   subtype PrintCharEps is Character range '!'..Eps;
   -- Gibt dem Bereich druckbarer Zeichen einen Namen
   -- (einschl. eps)

   subtype PrintChar is Character range '!'..'~';
   -- Gibt dem Bereich druckbarer Zeichen einen Namen
   -- (ohne eps)

   DEA1,  
   DEA2,
   DEA3 : DEA_Ref;    -- Zeiger auf einen beliebigen DEA   
   NDEA1,  
   NDEA2,
   NDEA3 : Ndea_Ref;  -- Zeiger auf einen beliebigen NDEA 
   A2oder3 : NDEA_Type (5); -- Variable fr den NDEA A2oder3  
   Azahl : NDEA_Type (6);   -- Variable fr den NDEA fr Float-Zahlen
   A2o3 : NDEA_Type (5);    -- Variable fr den NDEA A2/3
      

   -- Aufgabe 3 a) + c)
   -------------------------------------------------
   -- Myhill
   --
   -- Wandelt einen NDEA in einen DEA um
   -- Es wird gem der myhillschen Teilmengen-
   -- konstruktion aus den verschiedenen entstehenden
   -- Mengen eine Liste aufgebaut, aus der dann der
   -- neue DEA ausgelesen werden kann.
   -------------------------------------------------
   
   function Myhill (
         Ndea : Ndea_Ref ) 
     return Dea_Ref is 

      type Tupel; 
      type TListe is access Tupel; 
      type Tupel is 
         record 
            Nr      : Natural;   -- Zustand im DEA
            Zeichen : Character;  
            Nach    : TListe;  
         end record; 
      -- Liste, in der die mglichen Zeichen fr einen bergang in 
      -- einen anderen Zustand gespeichert werden.

      type MHelem; 
      type MHListe is access MHelem; 
      type MHelem is 
         record 
            Nr   : Natural;       -- Nummer des Zustandes   
            Cf   : Zustandsmenge; -- Charakteristischer Vektor fr die erreichbaren Zustnde   
            Rest : Mhliste;       -- Zeiger auf nchstes Element dieser Liste   
            Nfl  : TListe;        -- Zeiger auf Liste mit bergangszeichen   
         end record; 
      -- Liste, mit der der neue Automat (v.A. die neuen Zustnde) aufgebaut wird

      Beenden,                           -- Dient zum Abbruch des Aufbaus der MHListe 
      Ueberspringen : BOOLEAN  := FALSE; -- Gibt spter an, ob das Erstellen eines MHelem 
                                         -- bersprungen wird
      geaendert : BOOLEAN;               -- Bestimmt Vektoraddition beim TListe-Erstellen
      Kopf,  
      Akt1,
      Akt2,  
      Ende     : Mhliste       := null;  -- Zeiger auf Elemente der MHListe 
      AktTLst,
      AktZeichenUeber : TListe := null;  -- Hilfszeiger auf TListe     
      ZustdMge : Zustandsmenge;          -- Zeiger auf eine Zustandsmenge    
      N : Natural := NDEA.F.Inh'Length-1;-- N+1 (Dimension der char. Vektoren)
      RekNr : Natural;                   -- Speichert die Stelle auf die ein
                                         -- Zustand des DEAs zurckverweisen wird


      ----------------------------------------------
      -- EndZstdDEA
      --
      -- Alle Zustnde der MHListe, in deren char.
      -- Vektoren mind. einer der Endzustnde des
      -- NDEAs vorkommt, werden zu Endzustnden
      -- des DEAs.
      ----------------------------------------------

      function EndZstdDEA (
            F_Alt  : CharFktn_Type; 
            Liste  : MHListe;       
            Anzahl : Natural        ) 
        return CharFktn_Type is 

         F_Neu : CharFktn_Type (Anzahl + 1);  
         Akt   : MHListe;  

         -------------------------------------------
         -- Gemeinsam
         --
         -- Prfung ob char. Vektoren mind. einen
         -- gemeinsamen Zustand haben.
         -------------------------------------------
         function Gemeinsam (
               V1,                
               V2 : CharFktn_Type ) 
           return Boolean is 
         begin
            for I in V1.Inh'range loop
               if V1.Inh(I) and V2.Inh(I) then
                  return True;
               end if;
            end loop;
            return False;
         end Gemeinsam;

      begin -- EndZstdDEA
         Akt:=Liste;
         for I in 0..Anzahl loop
            F_Neu.Inh(I):=Gemeinsam(F_Alt, Akt.Cf.All);
            Akt:=Akt.Rest;
         end loop;
         F_Neu.Inh(Anzahl+1):=False;
         return F_Neu;
      end EndZstdDEA;


   begin -- MyHill()
      -- In der Schleife wird die MHListe aufgebaut
      while not(Beenden) loop
         if Kopf = null then -- Falls Listenanfang
            -- Erstellen des neuen char. Vektors
            ZustdMge:=EHuelleVon(0, NDEA);
            -- Erstellen des neuen MHListen-Elements
            Kopf:=new MHelem'(0, ZustdMge, null, null);
            ZustdMge:=LeereMenge(N);
            Ende:=Kopf;
            Akt1:=Ende;
         else -- Falls bereits Liste existiert
            AktZeichenUeber:= null;
            -- In der Schleife werden fr jedes Zeichen alle mgl. berg. zusammengestellt
            for J in PrintChar'range loop
               -- Berechnung Zustandsmenge von einem Zeichen aus
               for I in 0..N loop
                  if Akt1.Cf.Inh(I) then
                     if not(NDEA.Ndet.Ueber(I, J) = LeereMenge(N)) then
                        if ZustdMge = LeereMenge(N) then
                           ZustdMge:=new CharFktn_Type'(NDEA.Ndet.Ueber(I, J).All);
                        else
                           ZustdMge:=ZustdMge + NDEA.Ndet.Ueber(I, J);
                        end if;
                     end if;
                  end if;
               end loop;
               -- in nchster Schleife folgt die Epsilon-Hllen-Anwendung
               geaendert:=True;
               while Geaendert and not(ZustdMge = LeereMenge(N)) loop
                  geaendert:=False;
                  for K in 0..N loop
                     if ZustdMge.Inh(K) and not(ZustdMge = ZustdMge + EhuelleVon(K, NDEA)) then
                        ZustdMge:=ZustdMge + EhuelleVon(K, NDEA);
                        Geaendert:=True;
                     end if;
                  end loop;
               end loop;
               -- in nchster Schleife wird geprft ob neue ZustdMge bereits existiert               
               Akt2:=Kopf;
               while Akt2 /= null loop
                  if Akt2.Cf = ZustdMge then
                     RekNr:=Akt2.Nr; -- speichert den Zustand auf den der letzte
                                     -- Zustand des DEAs zeigen wird
                     Ueberspringen:=True;
                  end if;
                  Akt2:=Akt2.Rest;
               end loop;
               if not(Ueberspringen) then -- RekNr kein Rckverweis, sondern weiter
                  RekNr:=Ende.Nr+1;
               end if;
               if ZustdMge = LeereMenge(N) then -- Leere ZustdMge werden nicht aufgenommen
                  Ueberspringen:=True;
               else -- Aufbau TListe
                  if AktZeichenUeber = null then
                     Akt1.Nfl:= new Tupel'(Zeichen => J, Nr => RekNr, Nach => null);
                     AktZeichenUeber:=Akt1.Nfl;
                  else
                     AktZeichenUeber.Nach:= new Tupel'(Zeichen => J, Nr => RekNr, Nach => null);
                     AktZeichenUeber:=AktZeichenUeber.Nach;
                  end if;
               end if;
               if not(Ueberspringen) then -- MHListe erweitern
                  Ende.Rest:= new MHelem'(Ende.Nr+1, ZustdMge, null, null);
                  Ende:=Ende.Rest;
               else
                  Ueberspringen:=False;
               end if;
               if not(ZustdMge = LeereMenge(N)) then
                  ZustdMge:= LeereMenge(N);
               end if;
            end loop;
            Akt1:=Akt1.Rest;
            if Akt1 = null then
               Beenden:=True;
            end if;
         end if;
      end loop;
      -- Jetzt steht die Lnge des DEAs fest und er kann erzeugt werden
      declare
         DEA_Out : Dea_Type (Ende.Nr + 1); -- "+1" fr Fangzustand   
      begin
         -- Alle bergnge werden zunchst auf den Fangzustand gelegt
         DEA_Out.Det.Ueber:=(others => (others => Ende.Nr+1));
         Akt1:=Kopf;
         -- Alle tatschlichen bergnge werden jetzt ermittelt und eingefgt
         for I in 0..Ende.Nr loop
            AktTLst:=Akt1.Nfl;
            while AktTLst /= null loop
               DEA_Out.Det.Ueber(I, AktTlst.Zeichen):=AktTLst.Nr;
               AktTLst:=AktTLst.Nach;
            end loop;
            Akt1:=Akt1.Rest;
         end loop;
         -- Zu guter Letzt werden die neuen Endzustnde ermittelt
         DEA_Out.F:=EndZstdDEA(NDEA.F, Kopf, Ende.Nr);
         return new DEA_Type'(DEA_Out);
      end;
   end Myhill;

   -------------------------------------------------
   -- Drucke_NDEA
   --
   -- Prozedur zur grafischen Darstellung eines NDEA.
   -- Der erste Zustand jeder Zeile ist ein Ausgangs-
   -- zustand. Es folgen die bergnge zu den anderen
   -- Zustnden durch Pfeile dargestellt. Im Pfeil
   -- steht das jeweilige bergangszeichen.
   -- Grobuchstaben markieren einen Endzustand.
   -------------------------------------------------

   procedure Drucke_NDEA (
         NDEA : NDEA_Ref ) is 

   begin
      for I in NDEA.F.Inh'range loop
         New_Line;
         if NDEA.F.Inh(I) then
            Put("Q");
         else
            Put("q");
         end if;
         Put(Integer'Image(I) & " ");
         for J in PrintCharEps loop
            for K in NDEA.F.Inh'range loop
               if NDEA.Ndet.Ueber(I,J).Inh(K) then
                  Put("-" & J & "-> ");
                  if NDEA.F.Inh(K) then
                     Put("Q");
                  else
                     Put("q");
                  end if;
                  Put(Integer'Image(K) & " | ");
               end if;
            end loop;
         end loop;
      end loop;
      New_Line;
   end Drucke_NDEA;

   -------------------------------------------------
   -- Drucke_DEA
   --
   -- Prozedur zur grafischen Darstellung eines DEA.
   -- Der erste Zustand jeder Zeile ist ein Ausgangs-
   -- zustand. Es folgen die bergnge zu den anderen
   -- Zustnden durch Pfeile dargestellt. Im Pfeil
   -- steht das jeweilige bergangszeichen.
   -- Grobuchstaben markieren einen Endzustand.
   -------------------------------------------------

   procedure Drucke_DEA (
         DEA : DEA_Ref ) is 
   begin
      for I in DEA.F.Inh'range loop
         New_Line;
         if DEA.F.Inh(I) then
            Put("Q");
         else
            Put("q");
         end if;
         Put(Integer'Image(I) & " ");
         for J in PrintChar loop
            if DEA.Det.Ueber(I,J) /= DEA.F.Inh'Length-1 then
               Put("-" & J & "-> ");
               if DEA.F.Inh(DEA.Det.Ueber(I,J)) then
                  Put("Q");
               else
                  Put("q");
               end if;
               Put(Integer'Image(DEA.Det.Ueber(I,J)) & " | ");
            end if;
         end loop;
         Put("-others-> q" & Integer'Image(DEA.F.Inh'Length-1));
      end loop;
      New_Line;
   end Drucke_DEA;


begin -- Aufgabe3
   
   -- Aufgabe 3 b)
   -- Es wird zunchst der A2oder3-NDEA definiert:
   A2oder3.Ndet.Ueber :=
     (0 => ('a' =>    new CharFktn_Type'(5, (1 | 3 => True, others => False)),
         -- von Zustand 0 aus mit 'a' nach {1,3}
         others => new CharFktn_Type'(5, (others => False))),
         -- von Zustand 0 aus keine weiteren Uebergaenge
      1 => ('a' =>    new CharFktn_Type'(5, (2 => True, others => False)),
         -- von Zustand 1 aus mit 'a' nach {2}
         others => new CharFktn_Type'(5, (others => False))),
         -- von Zustand 1 aus keine weiteren Uebergaenge
      2 => ('a' =>    new CharFktn_Type'(5, (1 => True, others => False)),
         -- von Zustand 2 aus mit 'a' nach {1}
         others => new CharFktn_Type'(5, (others => False))),
         -- von Zustand 2 aus keine weiteren Uebergaenge
      3 => ('a' =>    new CharFktn_Type'(5, (4 => True, others => False)),
         -- von Zustand 3 aus mit 'a' nach {4}
         others => new CharFktn_Type'(5, (others => False))),
         -- von Zustand 3 aus keine weiteren Uebergaenge
      4 => ('a' =>    new CharFktn_Type'(5, (5 => True, others => False)),
         -- von Zustand 4 aus mit 'a' nach {5}
         others => new CharFktn_Type'(5, (others => False))),
         -- von Zustand 4 aus keine weiteren Uebergaenge
      5 => ('a' =>    new CharFktn_Type'(5, (3 => True, others => False)),
         -- von Zustand 5 aus mit 'a' nach {3}
         others => new CharFktn_Type'(5, (others => False)))
         -- von Zustand 5 aus keine weiteren Uebergaenge
      );
   A2oder3.F.Inh := (0|2|5 => True, others => False);
   -- Endzustaende sind 0, 2 und 5, d.h. F = {0,2,5}

   NDEA1:=new NDEA_Type'(A2oder3);
   Drucke_NDEA(NDEA1);
   -- Der Aufruf von Drucke_NDEA ergibt:

   -- Q 0 -a-> q 1 | -a-> q 3 | 
   -- q 1 -a-> Q 2 | 
   -- Q 2 -a-> q 1 | 
   -- q 3 -a-> q 4 | 
   -- q 4 -a-> Q 5 | 
   -- Q 5 -a-> q 3 | 

   -- Nun wird die Myhillsche Teilmengenkonstruktion auf A2oder3 angewendet:
   DEA1:=Myhill(NDEA1);
   Drucke_DEA(DEA1);
   -- Der Aufruf von Drucke_DEA ergibt:

   -- Q 0 -a-> q 1 | -others-> q 7
   -- q 1 -a-> Q 2 | -others-> q 7
   -- Q 2 -a-> Q 3 | -others-> q 7
   -- Q 3 -a-> Q 4 | -others-> q 7
   -- Q 4 -a-> q 5 | -others-> q 7
   -- q 5 -a-> Q 6 | -others-> q 7
   -- Q 6 -a-> q 1 | -others-> q 7
   -- q 7 -others-> q 7

   -- Es folgt die Definition des NDEAs "Azahl" fr Float-Konstanten:
   Azahl.Ndet.Ueber :=
     (0 => ('+' | '-' => new CharFktn_Type'(6, (1 | 2 => True, others => False)),
         -- von Zustand 0 aus mit '+' oder '-' nach {1,2}
         '0'..'9' => new CharFktn_Type'(6, (3 | 4 => True, others => False)),
         -- von Zusatand 0 aus mit '0'..'9' nach {3,4}
         others => new CharFktn_Type'(6, (others => False))),
         -- von Zustand 0 aus keine weiteren Uebergaenge
      1 => ('0'..'9' => new CharFktn_Type'(6, (3 => True, others => False)),
         --von Zustand 1 aus mit '0'..'9' nach {3}
         others => new CharFktn_Type'(6, (others => False))),
         -- von Zustand 1 aus keine weiteren Uebergaenge
      2 => ('0'..'9' => new CharFktn_Type'(6, (4 => True, others => False)),
         -- von Zustand 2 aus mit '0'..'9' nach {4}
         others => new CharFktn_Type'(6, (others => False))),
         --von Zustand 2 aus keine weiteren Uebergaenge
      3 => ('0'..'9' => new CharFktn_Type'(6, (3 => True, others => False)),
         -- von Zustand 3 aus mit '0'..'9' nach {3}
         '^' => new CharFktn_Type'(6, (0 => True, others => False)),
         -- von Zustand 3 aus mit '^' nach {0}
         others => new CharFktn_Type'(6, (others => False))),
         --von Zustand 3 aus keine weiteren Uebergaenge
      4 => ('0'..'9' => new CharFktn_Type'(6, (4 => True, others => False)),
         -- von Zustand 4 aus mit '0'..'9' nach {4}
         '.' => new CharFktn_Type'(6, (5 => True, others => False)),
         -- von Zustand 3 aus mit '.' nach {5}
         others => new CharFktn_Type'(6, (others => False))),
         -- von Zustand 4 aus keine weiteren Uebergaenge
      5 => ('0'..'9' => new CharFktn_Type'(6, (6 => True, others => False)),
         --von Zustand 5 aus mit '0'..'9' nach {6}
         others => new CharFktn_Type'(6, (others => False))),
         -- von Zustand 5 aus keine weiteren Uebergaenge
      6 => ('0'..'9' => new CharFktn_Type'(6, (6 => True, others => False)),
         --von Zustand 6 aus mit '0'..'9' nach {6}
         '^' => new CharFktn_Type'(6, (0 => True, others => False)),
         -- von Zustand 6 aus mit '^' nach {0}
         others => new CharFktn_Type'(6, (others => False)))
         -- von Zusatnd 5 aus keine weiteren Uebergaenge
      );

   Azahl.F.Inh := (3 | 4 | 6 => True, others => False);
   -- die Endzustaende sind 3, 6 und 8, d.h. F= {3,6,8}

   NDEA2:=new NDEA_Type'(Azahl);
   Drucke_NDEA(NDEA2);
   -- Der Aufruf von Drucke_NDEA ergibt:

   -- q 0 -+-> q 1 | -+-> q 2 | ---> q 1 | ---> q 2 | -0-> Q 3 | -0-> Q 4 | -1-> Q 3 | -1-> Q 4 |
   --     -2-> Q 3 | -2-> Q 4 | -3-> Q 3 | -3-> Q 4 | -4-> Q 3 | -4-> Q 4 | -5-> Q 3 | -5-> Q 4 |
   --     -6-> Q 3 | -6-> Q 4 | -7-> Q 3 | -7-> Q 4 | -8-> Q 3 | -8-> Q 4 | -9-> Q 3 | -9-> Q 4 | 
   -- q 1 -0-> Q 3 | -1-> Q 3 | -2-> Q 3 | -3-> Q 3 | -4-> Q 3 | -5-> Q 3 | -6-> Q 3 | -7-> Q 3 |
   --     -8-> Q 3 | -9-> Q 3 | 
   -- q 2 -0-> Q 4 | -1-> Q 4 | -2-> Q 4 | -3-> Q 4 | -4-> Q 4 | -5-> Q 4 | -6-> Q 4 | -7-> Q 4 |
   --     -8-> Q 4 | -9-> Q 4 | 
   -- Q 3 -0-> Q 3 | -1-> Q 3 | -2-> Q 3 | -3-> Q 3 | -4-> Q 3 | -5-> Q 3 | -6-> Q 3 | -7-> Q 3 |
   --     -8-> Q 3 | -9-> Q 3 | -^-> q 0 | 
   -- Q 4 -.-> q 5 | -0-> Q 4 | -1-> Q 4 | -2-> Q 4 | -3-> Q 4 | -4-> Q 4 | -5-> Q 4 | -6-> Q 4 |
   --     -7-> Q 4 | -8-> Q 4 | -9-> Q 4 | 
   -- q 5 -0-> Q 6 | -1-> Q 6 | -2-> Q 6 | -3-> Q 6 | -4-> Q 6 | -5-> Q 6 | -6-> Q 6 | -7-> Q 6 |
   --     -8-> Q 6 | -9-> Q 6 | 
   -- Q 6 -0-> Q 6 | -1-> Q 6 | -2-> Q 6 | -3-> Q 6 | -4-> Q 6 | -5-> Q 6 | -6-> Q 6 | -7-> Q 6 |
   --     -8-> Q 6 | -9-> Q 6 | -^-> q 0 | 

   -- Nun wird die Myhillsche Teilmengenkonstruktion auf Azahl angewendet:
   DEA2:=Myhill(NDEA2);
   Drucke_DEA(DEA2);
   -- Der Aufruf von Drucke_DEA ergibt:

   -- q 0 -+-> q 1 | ---> q 1 | -0-> Q 2 | -1-> Q 2 | -2-> Q 2 | -3-> Q 2 | -4-> Q 2 | -5-> Q 2 |
   --     -6-> Q 2 | -7-> Q 2 | -8-> Q 2 | -9-> Q 2 | -others-> q 5
   -- q 1 -0-> Q 2 | -1-> Q 2 | -2-> Q 2 | -3-> Q 2 | -4-> Q 2 | -5-> Q 2 | -6-> Q 2 | -7-> Q 2 |
   --     -8-> Q 2 | -9-> Q 2 | -others-> q 5
   -- Q 2 -.-> q 3 | -0-> Q 2 | -1-> Q 2 | -2-> Q 2 | -3-> Q 2 | -4-> Q 2 | -5-> Q 2 | -6-> Q 2 |
   --     -7-> Q 2 | -8-> Q 2 | -9-> Q 2 | -^-> q 0 | -others-> q 5
   -- q 3 -0-> Q 4 | -1-> Q 4 | -2-> Q 4 | -3-> Q 4 | -4-> Q 4 | -5-> Q 4 | -6-> Q 4 | -7-> Q 4 |
   --     -8-> Q 4 | -9-> Q 4 | -others-> q 5
   -- Q 4 -0-> Q 4 | -1-> Q 4 | -2-> Q 4 | -3-> Q 4 | -4-> Q 4 | -5-> Q 4 | -6-> Q 4 | -7-> Q 4 |
   --     -8-> Q 4 | -9-> Q 4 | -^-> q 0 | -others-> q 5
   -- q 5 -others-> q 5

   -- Es folgt die Definition des NDEAs "A2o3":
   A2o3.Ndet.Ueber :=
     (0 => (eps =>    new CharFktn_Type'(5, (1 | 3 => True, others => False)),
         -- von Zustand 0 aus mit 'a' nach {1,3}
         others => new CharFktn_Type'(5, (others => False))),
         -- von Zustand 0 aus keine weiteren Uebergaenge
      1 => ('a' =>    new CharFktn_Type'(5, (2 => True, others => False)),
         -- von Zustand 1 aus mit 'a' nach {2}
         others => new CharFktn_Type'(5, (others => False))),
         -- von Zustand 1 aus keine weiteren Uebergaenge
      2 => ('a' =>    new CharFktn_Type'(5, (1 => True, others => False)),
         -- von Zustand 2 aus mit 'a' nach {1}
         others => new CharFktn_Type'(5, (others => False))),
         -- von Zustand 2 aus keine weiteren Uebergaenge
      3 => ('a' =>    new CharFktn_Type'(5, (4 => True, others => False)),
         -- von Zustand 3 aus mit 'a' nach {4}
         others => new CharFktn_Type'(5, (others => False))),
         -- von Zustand 3 aus keine weiteren Uebergaenge
      4 => ('a' =>    new CharFktn_Type'(5, (5 => True, others => False)),
         -- von Zustand 4 aus mit 'a' nach {5}
         others => new CharFktn_Type'(5, (others => False))),
         -- von Zustand 4 aus keine weiteren Uebergaenge
      5 => ('a' =>    new CharFktn_Type'(5, (3 => True, others => False)),
         -- von Zustand 5 aus mit 'a' nach {3}
         others => new CharFktn_Type'(5, (others => False)))
         -- von Zustand 5 aus keine weiteren Uebergaenge
      );
   
   A2o3.F.Inh := (0 | 1 | 3 => True, others => False);
   -- Endzustaende sind 0, 1 und 3, d.h. F = {0,1,3}

   NDEA3:=new NDEA_Type'(A2o3);
   Drucke_NDEA(NDEA3);
   -- Der Aufruf von Drucke_NDEA ergibt:
   
   -- Q 0 -eps-> Q 1 | -eps-> Q 3 | 
   -- Q 1 -a-> q 2 | 
   -- q 2 -a-> Q 1 | 
   -- Q 3 -a-> q 4 | 
   -- q 4 -a-> q 5 | 
   -- q 5 -a-> Q 3 | 

   -- Nun wird die Myhillsche Teilmengenkonstruktion auf A2o3 angewendet:
   DEA3:=Myhill(NDEA3);
   Drucke_DEA(DEA3);
   -- Der Aufruf von Drucke_DEA ergibt:
   
   -- Q 0 -a-> q 1 | -others-> q 7
   -- q 1 -a-> Q 2 | -others-> q 7
   -- Q 2 -a-> Q 3 | -others-> q 7
   -- Q 3 -a-> Q 4 | -others-> q 7
   -- Q 4 -a-> q 5 | -others-> q 7
   -- q 5 -a-> Q 6 | -others-> q 7
   -- Q 6 -a-> q 1 | -others-> q 7
   -- q 7 -others-> q 7

   -- Im Folgenden werden die drei konstruierten DEAs getestet:
   -- Zunchst A2oder3:

   Pruefe (DEA1.All, "a");
   Pruefe (DEA1.All, "aa");
   Pruefe (DEA1.All, "aaa");
   Pruefe (DEA1.All, "aaaa");
   Pruefe (DEA1.All, "aaaaa");
   Pruefe (DEA1.All, "aaaaaa");
   Pruefe (DEA1.All, "aaaaaaa");

   -- Das Ergebnis nach Ablauf sieht so aus:

   -- "a" gehoert nicht zum Sprachschatz

   -- "aa" gehoert zum Sprachschatz

   -- "aaa" gehoert zum Sprachschatz

   -- "aaaa" gehoert zum Sprachschatz

   -- "aaaaa" gehoert nicht zum Sprachschatz

   -- "aaaaaa" gehoert zum Sprachschatz

   -- "aaaaaaa" gehoert nicht zum Sprachschatz


   -- Test fr Azahl:

   Pruefe (DEA2.All, "2");
   Pruefe (DEA2.All, "2.1");
   Pruefe (DEA2.All, "-2.1^10");
   Pruefe (DEA2.All, "+2.123^10^2.1");
   Pruefe (DEA2.All, "-2.123^10^-2.1");
   Pruefe (DEA2.All, "2.2.2");
   Pruefe (DEA2.All, "2^0");
   Pruefe (DEA2.All, "^10");
   Pruefe (DEA2.All, "-^2.3");
   Pruefe (DEA2.All, "2.3^3.4^-+3");
   Pruefe (DEA2.All, "e");

   -- Das Ergebnis nach Ablauf sieht so aus:

   -- "2" gehoert zum Sprachschatz

   -- "2.1" gehoert zum Sprachschatz

   -- "-2.1^10" gehoert zum Sprachschatz

   -- "+2.123^10^2.1" gehoert zum Sprachschatz

   -- "-2.123^10^-2.1" gehoert zum Sprachschatz

   -- "2.2.2" gehoert nicht zum Sprachschatz

   -- "2^0" gehoert zum Sprachschatz

   -- "^10" gehoert nicht zum Sprachschatz

   -- "-^2.3" gehoert nicht zum Sprachschatz

   -- "2.3^3.4^-+3" gehoert nicht zum Sprachschatz

   -- "e" gehoert nicht zum Sprachschatz
   

   -- Test fr A2o3:

   Pruefe (DEA3.All, "a");
   Pruefe (DEA3.All, "aa");
   Pruefe (DEA3.All, "aaa");
   Pruefe (DEA3.All, "aaaa");
   Pruefe (DEA3.All, "aaaaa");
   Pruefe (DEA3.All, "aaaaaa");
   Pruefe (DEA3.All, "aaaaaaa");

   -- Das Ergebnis nach Ablauf sieht so aus:

   -- "a" gehoert nicht zum Sprachschatz

   -- "aa" gehoert zum Sprachschatz

   -- "aaa" gehoert zum Sprachschatz

   -- "aaaa" gehoert zum Sprachschatz

   -- "aaaaa" gehoert nicht zum Sprachschatz

   -- "aaaaaa" gehoert zum Sprachschatz

   -- "aaaaaaa" gehoert nicht zum Sprachschatz

end Aufgabe3;