with Text_IO; use Text_IO; 

procedure Warshall is

   package Int_IO  is new Integer_IO(Integer); use Int_IO; 
--   package Bool_IO is new Enumeration_IO(Boolean); use Bool_IO;
   
   n : constant Integer := 6;
   
   -- (n,n)-Adjazenzmatrizen beschreiben Relationen AUF einer Menge
   type Adjazenzmatrix is 
      array (Integer range <>, Integer range <>) of Boolean;
   
   R : Adjazenzmatrix (1..n,1..n)  := 
        ((false,true, false,false,false,false),
         (false,false,true ,false,false,false),
         (false,false,false,true, false,false),
         (false,false,false,false,true ,false),
         (false,false,false,false,false,true),
         (true, false,false,false,false,false));
         
   H : Adjazenzmatrix (1..n,1..n)  := R;
   
   procedure drucke (A    : in Adjazenzmatrix; 
                     Text : in String := "Adjazenzmatrix:") is
   begin
      new_line(2);   
      put_line(Text);
      new_line; 
      put("     ");
      for i in 1..n loop put(i,3); end loop;
      new_line(2);
      for i in 1..n loop
         put(i,3); put(" :");
         for j in 1..n loop
            if A(i,j) then put("  1"); else put("  0"); end if;
         end loop;
         new_line;
      end loop;
      new_line; 
   end drucke;  
   
   -- Spiegelung an der Hauptdiagonalen
   function transponiere(A : Adjazenzmatrix) return Adjazenzmatrix is
      C : Adjazenzmatrix := A;
      h : Boolean;
   begin
      for i in 1..n-1 loop
         for j in i+1..n loop
             h := C(i,j); C(i,j) := C(j,i); C(j,i) := h;
         end loop;
      end loop; 
      return C;
   end transponiere;

   -- Relationenprodukt
   function "*" (A,B : Adjazenzmatrix) return Adjazenzmatrix is
      C : Adjazenzmatrix(1..n,1..n);
   begin
      for i in 1..n loop
         for j in 1..n loop
             C(i,j) := false;
             for k in 1..n loop
                C(i,j) := C(i,j) or (A(i,k) and B(k,j));
             end loop;
         end loop;
      end loop; 
      return C; 
   end "*";  

   -- Vereinigung zweier Relationen (= Mengen von Paaren)
   function "+" (A,B : Adjazenzmatrix) return Adjazenzmatrix is
      C : Adjazenzmatrix(1..n,1..n);
   begin
      for i in 1..n loop
         for j in 1..n loop
             C(i,j) := A(i,j) or B(i,j);
         end loop;
      end loop; 
      return C;
   end "+";  
   
   -- Transitive Huelle berechnen gemaess
   -- A+ = A + A*A + A*A*A + ... 
   function THmult (A : Adjazenzmatrix) return Adjazenzmatrix is
      C : Adjazenzmatrix := A;
   begin
      for i in 1..n loop
         C := A + (C*A);
      end loop; 
      return C; 
   end THmult;
   
   function FloydWarshall(A : Adjazenzmatrix) return Adjazenzmatrix is
      C : Adjazenzmatrix := A;
   begin
      for k in 1..n loop
         for i in 1..n loop
            if C(i,k) then
               for j in 1..n loop
                  C(i,j) := C(i,j) or C(k,j);
               end loop;
            end if;
         end loop;
      end loop;
      return C; 
   end FloydWarshall;
   
   kv : constant Integer := Integer'last;
   
   type Entfernungsmatrix is 
      array (Integer range <>, Integer range <>) of Integer;
   
   
   E : Entfernungsmatrix (1..n,1..n)  := 
        ((  0, 17, kv, kv, kv, 19),
         ( 17,  0, 22, kv, kv, 38),
         ( kv, 22,  0, 25, 34, kv),
         ( kv, kv, 25,  0,  2, kv),
         ( kv, kv, 34,  2,  0,  6),  
         ( 19, 38, kv, kv,  6,  0));
         
   
   W : Entfernungsmatrix (1..n,1..n) := (others =>(others => 0));
   
   
   procedure drucke (A    : in Entfernungsmatrix; 
                     Text : in String := "Entfernungsmatrix:") is
   begin
      new_line(2);   
      put_line(Text);
      new_line; 
      put("     ");
      for i in 1..n loop put(i,5); end loop;
      new_line(2);
      for i in 1..n loop
         put(i,3); put(" :");
         for j in 1..n loop
            if A(i,j) = Integer'last then put("     "); else put(A(i,j),5); end if;
         end loop;
         new_line;
      end loop;
      new_line; 
   end drucke;  

   function FloydWarshall(A : Entfernungsmatrix) return Entfernungsmatrix is
      C : Entfernungsmatrix := A;
   begin
      for k in 1..n loop
         for i in 1..n loop
           if C(i,k)  /= Integer'last then
            for j in 1..n loop
               if C(k,j) /= Integer'last then 
                  if C(i,j) = Integer'last or else
                     C(i,k) + C(k,j) < C(i,j) then
                     W(i,j) := k;  -- kuerzester Weg geht ueber k
                     C(i,j) := C(i,k) + C(k,j); -- Laenge von kuerzestem Weg
                  end if;
               end if;
            end loop;
           end if;
         end loop;
      end loop;
      return C;
   end FloydWarshall;

   procedure druckeWeg (A : in Entfernungsmatrix; q,z : Integer) is
   
      procedure zwischen(von,nach :Integer) is
         h : Integer := A(von,nach);    
      begin    
         if h /= 0 then
            zwischen(von,h);
            put(h,2);
            put(", ");
            zwischen(h,nach);
         end if;
      end zwischen;
      
   begin
   
      new_line; put("Weg: ");  
      put("("); put(q,2); put(", ");
      zwischen(q,z);
      put(z,2); put(" )");
      
   end druckeWeg;  
   
begin 
   
   drucke(R,"Adjazenzmatrix R:");
   
   drucke(transponiere(R),"Adjazenzmatrix R transponiert:");
   
   drucke(THmult(R),"Adjazenzmatrix H der transitiven Huelle von R:");
      
   for i in 1..n loop 
      drucke(H,"Wege der Lnge"&Integer'image(i));
      H := H*R;
   end loop;

   drucke(FloydWarshall(R),"Adjazenzmatrix H der transitiven Huelle von R:");

   put_line(Boolean'image(THmult(R)=FloydWarshall(R)));
   
   -----------------------------------------------------------------------
   
   drucke(E);
   
   drucke(FloydWarshall(E),"Matrix der kuerzesten Wege:");
   
   drucke(W,"Matrix der Zwischenknoten:");
   
   if FloydWarshall(E)(6,3) < 0 then
      new_line;
      put_line("Keine Verbindung zwischen 6 und 3");
   else
      druckeWeg(W,6,3);
      put_line(" :"&Integer'image(FloydWarshall(E)(6,3))&" km");
   end if;
   
   if FloydWarshall(E)(3,6) < 0 then
      new_line;
      put_line("Keine Verbindung zwischen 3 und 6");
   else
      druckeWeg(W,3,6);
      put_line(" :"&Integer'image(FloydWarshall(E)(3,6))&" km");
   end if;
   
   if FloydWarshall(E)(5,5) < 0 then
      new_line;
      put_line("Keine Verbindung zwischen 5 und 5");
   else
      druckeWeg(W,5,5);
      put_line(" :"&Integer'image(FloydWarshall(E)(5,5))&" km");
   end if;
   
   if FloydWarshall(E)(2,6) < 0 then
      new_line;
      put_line("Keine Verbindung zwischen 2 und 6");
   else
      druckeWeg(W,2,6);
      put_line(" :"&Integer'image(FloydWarshall(E)(2,6))&" km");
   end if;
      

end Warshall;