-- Grammatik der Arithmetischen Terme

-- In diesem Beispiel werden die arithmetischen Terme mit ihren Regeln
-- herangezogen. In der Typdefinition "Grammatiksymbole" werden zuerst
-- die Nicht-Terminale und Terminale festgelegt. Danach werden die
-- notwendigen Pakete geladen, instanziiert - hier nur das
-- "Relationenpaket" - und eingebunden. Die Pakete "Gg", "Paar" und
-- "Menge" werden vom "Relationenpaket" geladen und ber die "use"
-- Aufrufe hierher weitergegeben. Nun werden alle mglichen
-- "rechten Regelseiten" in "rechteSeiten" aufgelistet.
-- Mit jedem Aufruf "neueAlternativeHinzu" wird dem ersten Parameter,
-- dem Nichtterminal, entweder keine - Zuweisung aus dem leeren
-- Array 1..0 - oder eine rechte Seite aus dem Zeichenarray
-- "rechte Seiten" zugewiesen. Durch mehrere Aufrufe mit demselben
-- Nicht-Terminal knnen selbstverstndlich mehrere unterschiedliche
-- Regeln erstellt werden. Hierdruch werden die Regeln der Grammatik
-- definiert. Zuletzt folgt die Ausgabe der kompletten Grammatik
-- mit allen Symbolen, Regeln, einer Aussage ber ihre Einfachheit,
-- sowie - falls vorhanden - der Epsilonsymbole, und den Anfang-,
-- Neben-, Ende, First- und Followmengen.

with Text_IO; use Text_IO;
with Relationenpaket;

procedure ArithTermGramm is 

   type Grammatiksymbole is
      (E, ERest, T, TRest, F, addop, mulop, auf, zu, id, zahl, '#');

   package Klammpak is new Relationenpaket(Grammatiksymbole,5);
   use Klammpak;
   use Gg;
                         
-- in "Relationenpaket" mit "Gg" instanz. Paket "GrammatikGenerator2"
   
   use Paar;
                       
-- in "Relationenpaket" mit "Paar" instanz. Paket "ToAdi"
   
   use Menge; 
                     
-- in "Relationenpaket" mit "Menge" instanz. Paket "Mengen"

   Rechteseiten : Zeichenarray (1 .. 15) :=
      (T, Erest, Addop, T, Erest, F, Trest, Mulop, F, Trest, Auf, E,
          zu, id, zahl);  


begin


   Neuealternativehinzu (E, Rechteseiten(1..2));         
   
   -- E -> T ERest

   Neuealternativehinzu (Erest, Rechteseiten(1..0));     
   
   -- Erest -> Epsilon
   
   Neuealternativehinzu (Erest, Rechteseiten(3..5));     
   
   -- ERest ->  addop T Erest

   Neuealternativehinzu (T, Rechteseiten(6..7));         
   
   -- T -> F TRest

   Neuealternativehinzu (Trest, Rechteseiten(1..0));     
   
   -- TRest -> Epsilon
   
   Neuealternativehinzu (Trest, Rechteseiten(8..10));    
   
   -- TRest -> mulop F Trest

   Neuealternativehinzu (F, Rechteseiten(11..13));       
   
   -- F -> auf E zu
   
   Neuealternativehinzu (F, Rechteseiten(14..14));       
   
   -- F -> id
   
   Neuealternativehinzu (F, Rechteseiten(15..15));      
   
    -- F -> zahl



   Druckegrammatik;
   new_line(2);
   
   put("Die Grammatik der arithmetischen Terme ist ");
   if not istGrammatikEinfach
         then
      put("nicht ");
   end if;
   put("einfach.");
   new_line(2);
   
   druckeEps(Epszaehler);
   new_line(2);
   
   put_line("Anfang:");
   druckeMenge(Anfang);
   new_line(2);
   
   put_line("Neben:");
   druckeMenge(Neben);
   new_line(2);
   
   put_line("Ende:");
   druckeMenge(Ende);
   new_Line(2);
   
   put_line("First:");
   druckeMenge (first);
   new_Line(2);
   
   put_line("Follow:");
   druckeMenge(follow);


-- Testaufrufe
--------------

-- druckeEps(Epszaehler): { EREST, TREST } sind Epsilon-Symbole.

-- Anfang:
                   
--          M:={(ZAHL,ZAHL);(ID,ID);(ZU,ZU);(AUF,AUF);(MULOP,MULOP);
--              (ADDOP,ADDOP);(F,ZAHL);(F,ID);(F,AUF);(T,F);(E,T);}

-- Neben:                 
   
--          M:={(E,ZU);(AUF,E);(F,TREST);(MULOP,F);(F,TREST);
--              (T,EREST);(ADDOP,T);(T,EREST);}

-- Ende:                  
   
--          M:={(ZAHL,ZAHL);(ID,ID);(ZU,ZU);(AUF,AUF);(MULOP,MULOP);
--              (ADDOP,ADDOP);(F,ZAHL);(F,ID);(F,ZU);(T,TREST);(T,F);
--              (E,EREST);(E,T);}

-- First:                 
--          M:={(F,ZAHL);(F,ID);(F,AUF);(T,ZAHL);(T,ID);(T,AUF);
--              (E,ZAHL);(E,ID);(E,AUF);}

-- Follow:                
--          M:={(F,ZU);(TREST,ZU);(T,ZU);(EREST,ZU);(E,ZU);}
   

end ArithTermGramm;