-- Grammatik der Klammergebirge

-- In diesem Beispiel werden die Klammergebirge mit ihren Regeln
-- herangezogen. In der Typdefinition "Grammatiksymbole" werden
-- zuerst die Nicht-Terminale und Terminale festgelegt. Danach werden
-- die notwendigen Pakete geladen, instanziiert - hier nur das
-- "Relationenpaket" - und eingebunden. Die Pakete "Gg", "Paar" und
-- "Menge" werden vom "Relationenpaket" geladen und ber die "use"
-- Aufrufe hierher weitergegeben. Nun werden alle mglichen "rechten
-- Regelseiten" in "rechteSeiten" aufgelistet. "test1" beschreibt ein
-- Beispielwort zur berprfung, ob es im Sprachschatz liegt. Mit
-- jedem Aufruf "neueAlternativeHinzu" wird dem ersten Parameter,
-- dem Nicht-Terminal, entweder keine - Zuweisung aus dem leeren
-- Array 1..0 - oder eine rechte Seite aus dem Zeichenarray
-- "rechte Seiten" zugewiesen. Durch mehrere Aufrufe mit demselben
-- Nicht-Terminal knnen selbstverstndlich mehrere unterschiedliche
-- Regeln erstellt werden. Hierdurch werden die Regeln der Grammatik
-- definiert. Zuletzt folgt die Ausgabe der kompletten Grammatik
-- mit allen Symbolen, Regeln, einer Aussage ber ihre Einfachheit,
-- der Auswertung des Testwortes "test1", sowie - falls vorhanden -
-- der Epsilonsymbole, und den Anfang-, Neben-, Ende, First- und
-- Followmengen.

with Text_IO; use Text_IO;
with Relationenpaket;

procedure KlammergebirgeTest is 

   type Grammatiksymbole is (S, auf, zu, '#');

   package Klammpak is new Relationenpaket(Grammatiksymbole,1);
   use Klammpak;
   use Gg;
                         
   -- in "Relationenpaket" mit "Gg" instanz. Paket "GrammatikGenerator2"
   
   use Paar;
                       
   -- in "Relationenpaket" mit "Paar" instanz. Paket "ToAdi"
   
   use Menge; 
                     
   -- in "Relationenpaket" mit "Menge" instanz. Paket "Mengen"

   rechteSeiten : Zeichenarray (1 .. 4) := (auf, S, zu, S);  

   test1 : Wort (1 .. 4) := (auf, zu, auf, zu);  



begin

   neueAlternativeHinzu (S, rechteSeiten(1..4));    -- S -> auf S zu S
   neueAlternativeHinzu (S, rechteSeiten(1..0));    -- S -> Epsilon


   Druckegrammatik;
   new_line(2);

   put("Die Grammatik der Klammergebirge ist ");
   if not istGrammatikEinfach
         then
      put("nicht ");
   end if;
   put("einfach.");
   new_line(2);
   
   put_line("()() liegt wegen");
   if not wortImSprachschatz(test1) then
      put ("nicht ");
   end if;
   put("im Sprachschatz ");
   new_line(2);

   druckeEps(Epszaehler);
   new_line(2);
   
   put_line("Anfang:");
   druckeMenge(Anfang);
   new_line(2);
   
   put_line("Neben:");
   druckeMenge(Neben);
   new_line(2);
   
   put_line("Ende:");
   druckeMenge(Ende);
   new_Line(2);
   
   put_line("First:");
   druckeMenge (first);
   new_Line(2);
   
   put_line("Follow:");
   druckeMenge(follow);
   

-- Testaufrufe:
---------------

-- wortImSprachschatz(test1): ()() liegt wegen

--                            ( AUF ZU AUF ZU , S  )
--                            => ( AUF ZU AUF ZU , AUF S ZU S  )
--                            => ( ZU AUF ZU , S ZU S  )
--                            => ( ZU AUF ZU , ZU S  )
--                            => ( AUF ZU , S  )
--                            => ( AUF ZU , AUF S ZU S  )
--                            => ( ZU , S ZU S  )
--                            => ( ZU , ZU S  )
--                            => ( , S  )
--                            => ( ,  )

--                          im Sprachschatz

-- druckeEps(Epszaehler):   { S } sind Epsilon-Symbole.
   
-- Anfang:                  M:={(ZU,ZU);(AUF,AUF);(S,AUF);}

-- Neben:                   M:={(ZU,S);(S,ZU);(AUF,ZU);(AUF,S);}

-- Ende:                    M:={(ZU,ZU);(AUF,AUF);(S,S);(S,ZU);(S,AUF);}

-- First:                   M:={(S,AUF);}

-- Follow:                  M:={(S,ZU);}


end KlammergebirgeTest;