-- Test-Grammatik2

-- In diesem Beispiel wird eine fiktive Grammatik mit ihren Regeln
-- herangezogen. In der Typdefinition "Grammatiksymbole" werden zuerst
-- die Nicht-Terminale und Terminale festgelegt. Danach werden die
-- notwendigen Pakete geladen, instanziiert - hier nur das
-- "Relationenpaket" - und eingebunden. Die Pakete "Gg", "Paar" und
-- "Menge" werden vom "Relationenpaket" geladen und ber die "use"
-- Aufrufe hierher weitergegeben. Nun werden alle mglichen
-- "rechten Regelseiten" in "rechteSeiten" aufgelistet.
-- "p" wird zur berprfung der Funktion "Loesche" angelegt.
-- Mit jedem Aufruf "neueAlternativeHinzu" wird dem ersten Parameter,
-- dem Nicht-Terminal, entweder keine - Zuweisung aus dem leeren
-- Array 1..0 - oder eine rechte Seite aus dem Zeichenarray
-- "rechte Seiten" zugewiesen. Durch mehrere Aufrufe mit demselben
-- Nicht-Terminal knnen selbstverstndlich mehrere unterschiedliche
-- Regeln erstellt werden. Hierdurch werden die Regeln der Grammatik
-- definiert. Zuletzt folgt die Ausgabe der kompletten Grammatik
-- mit allen Symbolen, Regeln, einer Aussage ber ihre Einfachheit,
-- sowie - falls vorhanden - der Epsilonsymbole, und den Anfang-,
-- Neben-, Ende, First- und Followmengen.
-- Aus "p" wurde das Symbol "S" entfernt, um die Funktion "Loesche"
-- zu testen.

with Text_IO; use Text_IO;
with Relationenpaket;

procedure TestGrammatik2 is 

   type Grammatiksymbole is (S, A, B, C, D, E, aa, bb, dd, '#');

   package Relpak is new Relationenpaket(Grammatiksymbole,6);
   use Relpak;
   use Gg;
                         
   -- in "Relationenpaket" mit "Gg" instanz. Paket "GrammatikGenerator2"
   
   use Paar;
                       
   -- in "Relationenpaket" mit "Paar" instanz. Paket "ToAdi"
   
   use Menge; 
                     
   -- in "Relationenpaket" mit "Menge" instanz. Paket "Mengen"
   
   Rechteseiten : Zeichenarray (1 .. 19) :=
      (C, C, A, E, B, D, S, C, aa, C, C, S, A, dd, D, S, bb, E, bb);

   p : Listz; 

begin

   neueAlternativeHinzu (S, rechteSeiten(1..2));         -- S -> C C
   neueAlternativeHinzu (S, rechteSeiten(3..4));         -- S -> A E

   neueAlternativeHinzu (A, rechteSeiten(5..6));         -- A -> B D
   neueAlternativeHinzu (A, rechteSeiten(7..8));         -- A -> S C

   neueAlternativeHinzu (B, rechteSeiten(9..10));        -- B -> aa C
   neueAlternativeHinzu (B, rechteSeiten(11..13));       -- B -> C S A


   neueAlternativeHinzu (C, rechteSeiten(1..0));         -- C ->

   neueAlternativeHinzu (D, rechteSeiten(14..14));       -- D -> dd
   neueAlternativeHinzu (D, rechteSeiten(15..16));       -- D -> D S 

   neueAlternativeHinzu (E, rechteSeiten(17..19));       -- E -> bb E bb


   Druckegrammatik;
   new_line(2);

   put("Die Testgrammatik2 ist ");
   if not istGrammatikEinfach
         then
      put("nicht ");
   end if;
   put("einfach.");

   p := Epszaehler;

   Druckeeps(Epszaehler);
   New_Line(2);

   Loesche(S,p);   -- Probeaufruf, ob Loesche funktioniert

   Druckeeps(p);
   new_line(2);

   druckeEps(Epszaehler);
   new_line(2);
   
   put_line("Anfang:");
   druckeMenge(Anfang);
   new_line(2);
   
   put_line("Neben:");
   druckeMenge(Neben);
   new_line(2);
   
   put_line("Ende:");
   druckeMenge(Ende);
   new_Line(2);
   
   put_line("First:");
   druckeMenge (first);
   new_Line(2);
   
   put_line("Follow:");
   druckeMenge(follow);
   

-- Testaufrufe:
---------------

-- druckeEps(Epszaehler): { S A B C } sind Epsilon-Symbole.
   
-- druckeEps(p):          { A B C } sind Epsilon-Symbole.

-- Anfang:       M:={(DD,DD);(BB,BB);(AA,AA);(E,BB);(D,D);(D,DD);(B,A);
--                   (B,S);(B,C);(B,AA);(A,C);(A,S);(A,D);(A,B);(S,E);
--                   (S,A);(S,C);(S,C);}

-- Neben:        M:={(E,BB);(BB,E);(D,S);(S,A);(C,A);(C,S);(AA,C);
--                   (S,C);(B,D);(A,E);(C,C);}

-- Ende:         M:={(DD,DD);(BB,BB);(AA,AA);(E,BB);(D,S);(D,D);(D,DD);
--                   (B,A);(B,S);(B,C);(B,C);(B,AA);(A,C);(A,S);(A,D);
--                   (S,E);(S,C);(S,C);}

-- First:        M:={(E,BB);(D,DD);(B,DD);(B,BB);(B,AA);(A,DD);(A,BB);
--                   (A,AA);(S,DD);(S,BB);(S,AA);}

-- Follow:       M:={(E,DD);(E,BB);(E,AA);(D,DD);(D,BB);(D,AA);(C,DD);
--                   (C,BB);(C,AA);(B,DD);(A,DD);(A,BB);(S,DD);(S,BB);
--                   (S,AA);}
   

end TestGrammatik2;
