-- While-Grammatik

-- In diesem Beispiel wird die "While-Grammatik" mit ihren Regeln
-- herangezogen. In der Typdefinition "Grammatiksymbole" werden zuerst
-- die Nicht-Terminale und Terminale festgelegt. Danach werden die
-- notwendigen Pakete geladen, instanziiert - hier nur das
-- "Relationenpaket" - und eingebunden. Die Pakete "Gg", "Paar" und
-- "Menge" werden vom "Relationenpaket" geladen und ber die
-- "use" Aufrufe hierher weitergegeben. Nun werden alle mglichen
-- "rechten Regelseiten" in "rechteSeiten" aufgelistet. Mit jedem
-- Aufruf "neueAlternativeHinzu" wird dem ersten Parameter, dem 
-- Nicht-Terminal, entweder keine - Zuweisung aus dem leeren
-- Array 1..0 - oder eine rechte Seite aus dem Zeichenarray
-- "rechte Seiten" zugewiesen. Durch mehrere Aufrufe mit demselben
-- Nicht-Terminal knnen selbstverstndlich mehrere unterschiedliche
-- Regeln erstellt werden. Hierdurch werden die Regeln der Grammatik
-- definiert. Zuletzt folgt die Ausgabe der kompletten Grammatik
-- mit allen Symbolen, Regeln, einer Aussage ber ihre Einfachheit,
-- sowie - falls vorhanden - der Epsilonsymbole, und den Anfang-, 
-- Neben-, Ende, First- und Followmengen.

with Text_IO; use Text_IO;
with Relationenpaket;

procedure WhileGrammatik2 is 

   -- S : Programm
   -- NT: AnwF, Anw, bedAnw, Zuweisung, leereAnwisung, Schleife
   -- T : name, zahl, rop, op, true, rop, op, false, if, then, else,
   --     fi, while, do, od, #, semikolon, skip

   type Grammatiksymbole is
      (Programm, Anwf, Anw, Bedingteanweisung, Bedingung, Zuweisung,
       Leereanweisung, Schleife, Ausdruck, Name, Zahl, Rop, Op,True,
       False, If1, Then1, Else1, Fi, While1, Do1, Od, Semikolon,
       Skip, Auf, Zu, zuweisung1, '#');

   package Whilepak is new Relationenpaket(Grammatiksymbole,9);
   use Whilepak;
   use Gg;
                         
   -- in "Relationenpaket" mit "Gg" instanz. Paket "GrammatikGenerator2"
   
   use Paar;
                       
   -- in "Relationenpaket" mit "Paar" instanz. Paket "ToAdi"
   
   use Menge; 
                     
   -- in "Relationenpaket" mit "Menge" instanz. Paket "Mengen"

   Rechteseiten : Zeichenarray (1 .. 36) :=
      (Anwf, Anw, Semikolon, Anwf, Bedingteanweisung, Zuweisung,
       Leereanweisung, Schleife,Skip, Name, Zuweisung1, Ausdruck, Zahl,
       Name, Auf, Ausdruck, Op, Ausdruck, Zu, If1, Bedingung, Then1,
       Anwf, Else1,Anwf, Fi, While1, Bedingung, Do1, Anwf, Od, True,
       Ausdruck, rop, Ausdruck, false); 


begin

   Neuealternativehinzu(Programm, Rechteseiten(1..1));             
   
   -- Programm -> AnwF

   Neuealternativehinzu(Anwf, Rechteseiten(2..4));                 
   
   -- AnwF -> Anw Semikolon AnwF      
   
   Neuealternativehinzu(Anwf, Rechteseiten(2..2));                 
   
   -- AnwF -> Anw

   Neuealternativehinzu(Anw, Rechteseiten(5..5));                  
   
   -- Anw -> bedingteAnweisung
   
   Neuealternativehinzu(Anw, Rechteseiten(6..6));                  
   
   -- Anw -> Zuweisung
   
   Neuealternativehinzu(Anw, Rechteseiten(7..7));                  
   
   -- Anw -> leereAnweisung
   
   Neuealternativehinzu(Anw, Rechteseiten(8..8));                  
   
   -- Anw -> Schleife

   Neuealternativehinzu(Leereanweisung, Rechteseiten(9..9));       
   
   -- leereAnweisung -> skip

   Neuealternativehinzu(Zuweisung, Rechteseiten(10..12));          
   
   -- Zuweisung -> name zuweisung1 Ausdruck

   Neuealternativehinzu(Ausdruck, Rechteseiten(13..13));           
   
   -- Ausdruck -> zahl
   
   Neuealternativehinzu(Ausdruck, Rechteseiten(14..14));           
   
   -- Ausdruck -> name
   
   Neuealternativehinzu(Ausdruck, Rechteseiten(15..19));           
   
   -- Ausdruck -> auf Ausdruck op Ausdruck zu

   Neuealternativehinzu(Bedingteanweisung, Rechteseiten(20..26));  
   
   -- bedingteAnweisung -> if Bedingung then AnwF else AnwF fi

   Neuealternativehinzu(Schleife, Rechteseiten(27..31));           
   
   -- Schleife -> while Bedingung do AnwF od

   Neuealternativehinzu(Bedingung, Rechteseiten(32..32));          
   
   -- Bedingung -> true
   
   Neuealternativehinzu(Bedingung, Rechteseiten(33..35));          
   
   -- Bedingung -> Ausdruck rop Ausdruck
   
   Neuealternativehinzu(Bedingung, Rechteseiten(36..36));          
   
   -- Bedingung -> false



   Druckegrammatik;
   new_line(2);


   put("Die While-Grammtik ist ");
   if not istGrammatikEinfach
         then
      put("nicht ");
   end if;
   put("einfach.");
   new_line(2);
   

   druckeEps(Epszaehler);
   new_line(2);
   
   put_line("Anfang:");
   druckeMenge(Anfang);
   new_line(2);
   
   put_line("Neben:");
   druckeMenge(Neben);
   new_line(2);
   
   put_line("Ende:");
   druckeMenge(Ende);
   new_Line(2);
   
   put_line("First:");
   druckeMenge (first);
   new_Line(2);
   
   put_line("Follow:");
   druckeMenge(follow);


-- Testaufrufe:
---------------

-- druckeEps(Epszaehler): { } sind Epsilon-Symbole.

-- Anfang:                
--          M:={(ZUWEISUNG1,ZUWEISUNG1);(ZU,ZU);(AUF,AUF);(SKIP,SKIP);
--              (SEMIKOLON,SEMIKOLON);('#','#');(OD,OD);(DO1,DO1);
--              (WHILE1,WHILE1);(FI,FI);(ELSE1,ELSE1);(THEN1,THEN1);
--              (IF1,IF1);(FALSE,FALSE);(TRUE,TRUE);(OP,OP);(ROP,ROP);
--              (ZAHL,ZAHL);(NAME,NAME);(AUSDRUCK,AUF);
--              (AUSDRUCK,NAME);(AUSDRUCK,ZAHL);(SCHLEIFE,WHILE1);
--              (LEEREANWEISUNG,SKIP);(ZUWEISUNG,NAME);
--              (BEDINGUNG,FALSE);(BEDINGUNG,AUSDRUCK);
--              (BEDINGUNG,TRUE);(BEDINGTEANWEISUNG,IF1);
--              (ANW,SCHLEIFE);(ANW,LEEREANWEISUNG);(ANW,ZUWEISUNG);
--              (ANW,BEDINGTEANWEISUNG);(ANWF,ANW);(ANWF,ANW);
--              (PROGRAMM,ANWF);}

-- Neben:                 
--          M:={(AUSDRUCK,ZU);(OP,AUSDRUCK);(AUSDRUCK,OP);
--              (AUF,AUSDRUCK);(ANWF,OD);(DO1,ANWF);(BEDINGUNG,DO1);
--              (WHILE1,BEDINGUNG);(ZUWEISUNG1,AUSDRUCK);
--              (NAME,ZUWEISUNG1);(ROP,AUSDRUCK);(AUSDRUCK,ROP);
--              (ANWF,FI);(ELSE1,ANWF);(ANWF,ELSE1);(THEN1,ANWF);
--              (BEDINGUNG,THEN1);(IF1,BEDINGUNG);(SEMIKOLON,ANWF);
--              (ANW,SEMIKOLON);}

-- Ende:                  
--          M:={(AUSDRUCK,ZU);(OP,AUSDRUCK);(AUSDRUCK,OP);
--              (AUF,AUSDRUCK);(ANWF,OD);(DO1,ANWF);(BEDINGUNG,DO1);
--              (WHILE1,BEDINGUNG);(ZUWEISUNG1,AUSDRUCK);
--              (NAME,ZUWEISUNG1);(ROP,AUSDRUCK);(AUSDRUCK,ROP);
--              (ANWF,FI);(ELSE1,ANWF);(ANWF,ELSE1);(THEN1,ANWF);
--              (BEDINGUNG,THEN1);(IF1,BEDINGUNG);(SEMIKOLON,ANWF);
--              (ANW,SEMIKOLON);}

-- First:                 
--          M:={(AUSDRUCK,AUF);(AUSDRUCK,ZAHL);(AUSDRUCK,NAME);
--              (SCHLEIFE,WHILE1);(LEEREANWEISUNG,SKIP);
--              (ZUWEISUNG,NAME);(BEDINGUNG,AUF);(BEDINGUNG,FALSE);
--              (BEDINGUNG,TRUE);(BEDINGUNG,ZAHL);(BEDINGUNG,NAME);
--              (BEDINGTEANWEISUNG,IF1);(ANW,SKIP);(ANW,WHILE1);
--              (ANW,IF1);(ANW,NAME);(ANWF,SKIP);(ANWF,WHILE1);
--              (ANWF,IF1);(ANWF,NAME);(PROGRAMM,SKIP);
--              (PROGRAMM,WHILE1);(PROGRAMM,IF1);(PROGRAMM,NAME);}

-- Follow:                
--          '#'
--          M:={(AUSDRUCK,'#');(AUSDRUCK,ZU);(AUSDRUCK,SEMIKOLON);
--              (AUSDRUCK,OD);(AUSDRUCK,DO1);(AUSDRUCK,FI);
--              (AUSDRUCK,ELSE1);(AUSDRUCK,THEN1);(AUSDRUCK,OP);
--              (AUSDRUCK,ROP);(SCHLEIFE,'#');(SCHLEIFE,SEMIKOLON);
--              (SCHLEIFE,OD);(SCHLEIFE,FI);(SCHLEIFE,ELSE1);
--              (LEEREANWEISUNG,'#');(LEEREANWEISUNG,SEMIKOLON);
--              (LEEREANWEISUNG,OD);(LEEREANWEISUNG,FI);
--              (LEEREANWEISUNG,ELSE1);(ZUWEISUNG,'#');
--              (ZUWEISUNG,SEMIKOLON);(ZUWEISUNG,OD);(ZUWEISUNG,FI);
--              (ZUWEISUNG,ELSE1);(BEDINGUNG,DO1);(BEDINGUNG,THEN1);
--              (BEDINGTEANWEISUNG,'#');(BEDINGTEANWEISUNG,SEMIKOLON);
--              (BEDINGTEANWEISUNG,OD);(BEDINGTEANWEISUNG,FI);
--              (BEDINGTEANWEISUNG,ELSE1);(ANW,'#');(ANW,SEMIKOLON);
--              (ANW,OD);(ANW,FI);(ANW,ELSE1);(ANWF,'#');(ANWF,OD);
--              (ANWF,FI);(ANWF,ELSE1);(PROGRAMM,'#');}


end WhileGrammatik2;