with Ada.Integer_Text_Io, Ada.Text_Io, Ada.Strings.Unbounded;
use  Ada.Integer_Text_Io, Ada.Text_Io, Ada.Strings.Unbounded;

procedure While_Interpreter is 
-----------------------------------------------------------------------------------------------
--------- Typdeklarationen --------------------------------------------------------------------
-- Konstruktion der Syntax der while-Sprache --------------------------------------------------
-- Einfhrung aller notwendigen Datentypen ----------------------------------------------------
-----------------------------------------------------------------------------------------------
   type Operator is (Plus,Minus,Mal,Div,Modulo);
-----------------------------------------------------------------------------------------------
   type Vergleicher is (Kl,Gl,Gr,Kg,Ug,Gg);
-----------------------------------------------------------------------------------------------
   type Ausdrucks_Art is (Zahl,Name,Verkn);
   type Ausdruck_Rec;
   type Ausdruck is access Ausdruck_Rec;
   type Ausdruck_Rec (Form : Ausdrucks_Art := Zahl) is record
      case Form is
         when Zahl  => Zahl_Wert       : Integer;

         when Name  => Name_Wert       : Unbounded_String;

         when Verkn => Operator_Wert   : Operator;
                       Links_Ausdruck,
                       Rechts_Ausdruck : Ausdruck;
      end case;
   end record;
-----------------------------------------------------------------------------------------------
   type Bedingungs_Art is (Wahr,Vergleich);
   type Bedingung_Rec;
   type Bedingung is access Bedingung_Rec;
   type Bedingung_Rec (Form : Bedingungs_Art := Wahr) is record
      case Form is
         when Wahr      => Wahr_Wert       : Boolean;

         when Vergleich => Vergleichs_Wert : Vergleicher;
                           Links_Ausdruck,
                           Rechts_Ausdruck : Ausdruck;
      end case;
   end record;
-----------------------------------------------------------------------------------------------
   type Anweisungs_Art is (Fall,Schleife,Skip,Zuweisung);
   type Anweisung_Rec;
   type Anweisung_F_Rec;
   type Anweisung is access Anweisung_Rec;
   type Anweisung_F is access Anweisung_F_Rec;
   type Anweisung_F_Rec is record
         Anw  : Anweisung;
         Rest : Anweisung_F;
   end record;
   type Anweisung_Rec (Form : Anweisungs_Art := Fall) is record
      case Form is
         when Fall      => Bedingungs_Wert_Fall     : Bedingung;
                           Erst_Anweisung_Fall,
                           Zweit_Anweisung_Fall     : Anweisung_F;

         when Schleife  => Bedingungs_Wert_Schleife : Bedingung;
                           Erst_Anweisung_Schleife  : Anweisung_F;

         when Skip      => null;

         when Zuweisung => Zuweisungs_Name          : Unbounded_String;
                           Links_Ausdruck           : Ausdruck;
      end case;
   end record;

-- Da Anweisungen mehrfach auch Listen von Anweisungen enthalten knnen,
-- muss der Typ der Anweisungsfolge extra deklariert werden. Die Typen
-- Anweisung und Anweisung_F werden letztlich auch gegenseitig zur Dekla-
-- ration verwendet. Dies ist ein Beispiel von vielen, in der der Mehr-
-- aufwand in Ada gegenber der Haskellversion sehr deutlich wird.
-----------------------------------------------------------------------------------------------
   type Wert_Art is (Undef,Def_Wert);
   type Wert (Form : Wert_Art := Undef) is record
      case Form is
         when Undef    => null;
         when Def_Wert => Zahl_Wert : Integer;
      end case;
   end record;

-- Speicherbelgung einer Variable in der Umgebung
-----------------------------------------------------------------------------------------------
   type Variable is record
         Variable_Name : Unbounded_String;
         Variable_Wert : Wert;
   end record;

-- Variable als Tupel des Variablennamens und der Speicherbelegung
-----------------------------------------------------------------------------------------------
   type Variable_Rec;
   type Variablen is access Variable_Rec;
   type Variable_Rec is record
      Variable_Name : Unbounded_String;
      Variable_Wert : Wert;
      Variable_Next : Variablen;
   end record;
-----------------------------------------------------------------------------------------------
   type Umgebung_Rec;
   type Umgebung is access Umgebung_Rec;
   type Umgebung_Rec is record
      Elem : Variable;
      Next : Umgebung;
   end record;

-- Die Umgebung wird aus einer Liste von Variablen erzeugt. Jedes
-- Listenelement enthlt dabei Variablennamen und Speicherbelegung.
-----------------------------------------------------------------------------------------------
   type Teil_Arten is (Anw,Anw_F,Bed,Ausdr,Bez,Zwert,Wwert,Op,Rop,If_Case,Do_Case,Zuw);
   type Teil_Rec;
   type Teil is access Teil_Rec;
   type Teil_Rec (Form : Teil_Arten := Anw) is record
      case Form is
         when Anw     => Teil_Anw   : Anweisung;

         when Anw_F   => Teil_Anw_F : Anweisung_F;

         when Bed     => Teil_Bed   : Bedingung;

         when Ausdr   => Teil_Ausdr : Ausdruck;

         when Bez     => Teil_Bez   : Unbounded_String;

         when Zwert   => Teil_Zwert : Integer;

         when Wwert   => Teil_Wwert : Boolean;

         when Op      => Teil_Op    : Operator;

         when Rop     => Teil_Rop   : Vergleicher;

         when others  => null;
      end case;
   end record;

-- Der Datentyp Teil definiert den Aufbau des Programmkellers und
-- des Wertekellers. Zwar wird nicht jedes mgliche Programmteil
-- bei der Auswertung auch im Wertekeller gespeichert, jedoch ist
-- es bersichtlicher nur einen Typen fr beide Keller zu nutzen.
-----------------------------------------------------------------------------------------------
   type Teile_Rec;
   type Teile is access Teile_Rec;
   type Teile_Rec is record
      T    : Teil;
      Rest : Teile;
   end record;
-----------------------------------------------------------------------------------------------
   type Konfiguration is record
      Um  : Umgebung;
      Wk,
      Zk  : Teile;
   end record;

-- Der Zustand der abstrakten Maschine als Tripple der Speicher-
-- umgebung, dem Wertekeller und dem Programmkeller
-----------------------------------------------------------------------------------------------
   type Zustand_Arten is (Ende, Akt);
   type Zustand (Form : Zustand_Arten := Ende) is record
      case Form is
         when Ende => Ende_Zustand : Umgebung;

         when Akt  => Akt_Zustand  : Konfiguration;
      end case;
   end record;

-- Der Datentyp Zustand, um vom Anfangszustand sowie den
-- den Zwischenzustnden whrend der Auswertung und von
-- der Endspeicherbelegung, fr die die Kellerinhalte nicht
-- mehr von Bedeutung sind, unterscheiden zu knnen.
-----------------------------------------------------------------------------------------------
--------- Funktionen --------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
   function Ada_Op (Op     : Operator;
                    Z1, Z2 : Integer  ) return Integer is

      Erg : Integer;

   begin
      case Op is
         when Plus   => Erg:= Z1 + Z2;

         when Minus  => Erg:= Z1 - Z2;

         when Mal    => Erg:= Z1 * Z2;

         when Div    => Erg:= Z1 / Z2;

         when Modulo => Erg:= Z1 mod Z2;
      end case;
      return Erg;
   end Ada_Op;
-----------------------------------------------------------------------------------------------
   function Ada_Rel_Op (Rop    : Vergleicher;
                        Z1,Z2  : Integer     ) return Boolean is

      Erg : Boolean;

   begin
      case Rop is
         when Kl => Erg := Z1 < Z2;

         when Gl => Erg := Z1 = Z2;

         when Gr => Erg := Z1 > Z2;

         when Kg => Erg := Z1 <= Z2;

         when Ug => Erg := Z1 /= Z2;

         when Gg => Erg := Z1 >= Z2;
      end case;
      return Erg;
   end Ada_Rel_Op;

-- Die Funktionen ada_Op und ada_Rel_Op werden in der 4. und 7. Regel
-- der Zustandsbergangsfunktion genutzt, um aus zwei auf Integerwerte
-- ausgewertete Ausdrcke und dem entsprechenden Operatorer ein Ergebnis
-- (Integerwert bei arithmetischen Operatoren, Boolean bei Relationsoper-
-- atoren) zu erzeugen. Anders als bei der Haskellversion war es in Ada
-- hierzu notwendig, die Parameter der Verknpfung bzw. Bedingung der
-- Funktion mit zu bergeben, da in Ada Funktionen keine Funktionen
-- als Ausgabewert wiedergeben knnen.

-- Aufruf:   I:=(ada_Op (Plus,1,2);Put(Integer'Image(T));
-- Ergebnis: 3
-- Aufruf:   B:=(ada_rel_Op(Kl,2,2);Put(Boolean'Image(B));
-- Ergebnis: False

-- Anders zu den Enumerationstypen Operator und Vergleicher werden nahezu
-- smtliche folgende Typen als Verbunde mit Diskriminanten deklariert.
-- Nur auf diese Weise ist es mglich die algebraischen Datentypen aus
-- Haskell in Ada abzubilden.
-----------------------------------------------------------------------------------------------
   function Ist_Undef (W : Wert) return Boolean is

      Wert_Def : Wert := W;

   begin
      if Wert_Def.Form = Undef then
         return True;
      else
         return False;
      end if;
   end Ist_Undef;
-----------------------------------------------------------------------------------------------
   function VarD return Variable is

      Var : Variable;
      Zw  : Wert (Def_Wert);

   begin
      Var.Variable_Name := To_Unbounded_String("d");
      Zw.Zahl_Wert      := 0;
      Var.Variable_Wert := Zw;
      return Var;
   end VarD;
-----------------------------------------------------------------------------------------------
   function Wert_Suche (Var : Unbounded_String;
                        Umg : Umgebung         ) return Wert is

      C  : Unbounded_String := Var;
      U  : Umgebung         := Umg;
      Zw : Wert (Undef);

   begin
      if U = null then
         return Zw;
      elsif U.Elem.Variable_Name = C then
         return U.Elem.Variable_Wert;
      else
         return Wert_Suche (C,U.Next);
      end if;
   end Wert_Suche;

-- Mit wertSuche kann aus einer Umgebung im aktuellen Zustand die
-- Speicherbelegung einer bergegeben Variable ausgegeben werden.

-- Aufruf:   umgeb:=um;                                        -> (d:= undef, x:=17, y:=4) 
--           h:=wert_suche(To_Unbounded_String("y"),umgeb);
--           if h.form = def_wert then
--             put(wert_Art'image(h.form)); put(" mit"); put(integer'image(h.zahl_wert));
--           else put(wert_Art'image(h.form));
--           end if;
-- Ergebnis: DEF_WERT mit 4
-- Aufruf:   umgeb:=um;                                        -> (d:= undef, x:=17, y:=4) 
--           h:=wert_suche(To_Unbounded_String("d"),umgeb);
--           if h.form = def_wert then
--             put(wert_Art'image(h.form)); put(" mit"); put(integer'image(h.zahl_wert));
--           else put(wert_Art'image(h.form));
--           end if;
-- Ergebnis: UNDEF
-----------------------------------------------------------------------------------------------
   function Wert_Eintrag (Var      : Unbounded_String;
                          Int_Wert : Integer;
                          Umg      : Umgebung         ) return Umgebung is

      C   : Unbounded_String := Var;
      U   : Umgebung         := Umg;
      Int : Integer          := Int_Wert; -- copyfunction da alte umgebung zerstrt wird
      Zw  : Wert (Def_Wert);

   begin
      if U = null then
         Put("error: Variable " & To_String(C) & " nicht in der Umgebung");
      elsif U.Elem.Variable_Name = C then
         case U.Elem.Variable_Wert.Form is
            when Undef    => U.Elem.Variable_Wert           := Zw ;
                             U.Elem.Variable_Name           := C;
                             U.Elem.Variable_Wert.Zahl_Wert := Int;
            when Def_Wert => U.Elem.Variable_Wert.Zahl_Wert := Int;
         end case;
      else
         U.Next := Wert_Eintrag (C,Int,U.Next);
         return U;
      end if;
      return U;
   end Wert_Eintrag;

-- wertEintrag wird fr die Regel 10 der Zustandsbergangs-
-- funktion verwendet, welche den Integerwert einer Variable
-- in der aktuellen Umgebung verndert, bzw. im Falle Undef
-- erstmalig belegt. Es kann nur auf bereits in der aktuellen
-- Umgebung vorhandene Variablen zugegriffen werden.

-- Aufruf:      umgeb:=um                                               -> (d:= undef, x:=17, y:=4)
--              umgeb:=wert_Eintrag(To_Unbounded_String("d"),5,umgeb);
--              h:=wert_suche(To_Unbounded_String("d"),umgeb);
--              if h.form = def_wert then
--                put(wert_Art'image(h.form)); put(" mit"); put(integer'image(h.zahl_wert));
--              else put(wert_Art'image(h.form));
--              end if;
-- Ergebnis:    DEF_WERT mit 5
-- Aufruf:      umgeb:=um                                               -> (d:= undef, x:=17, y:=4)
--              umgeb:=wert_Eintrag(To_Unbounded_String("a"),5,umgeb);
--              h:=wert_suche(To_Unbounded_String("d"),umgeb);
--              if h.form = def_wert then
--                put(wert_Art'image(h.form)); put(" mit"); put(integer'image(h.zahl_wert));
--              else put(wert_Art'image(h.form));
--              end if;
-- Ergebnis:    error: Variable a nicht in der Umgebung
--              UNDEF
-- Aufruf:      b:=Ist_Undef(w1);               -> w1:Wert(Undef);
--              put(boolean'image(b));
-- Ergebnis:    TRUE
-- Aufruf:      w2.zahl_wert:=10;               -> w2:Wert(Def_Wert);
--              b:=Ist_Undef(w2);
--              put(boolean'image(b));
-- Ergebnis:    FALSE   
-----------------------------------------------------------------------------------------------
   function Um return Umgebung is

      Var1,Var2,Var3 : Variable;
      W1             : Wert (Undef);
      W2,W3          : Wert (Def_Wert);
      U              : Umgebung;

   begin
      W2.Zahl_Wert       := 17;
      W3.Zahl_Wert       := 4;
      Var1.Variable_Name := To_Unbounded_String("d");
      Var2.Variable_Name := To_Unbounded_String("x");
      Var3.Variable_Name := To_Unbounded_String("y");
      Var1.Variable_Wert := W1;
      Var2.Variable_Wert := W2;
      Var3.Variable_Wert := W3;
      U                  := new Umgebung_Rec'(Var3,null);
      U                  := new Umgebung_Rec'(Var2,U);
      U                  := new Umgebung_Rec'(Var1,U);
      return U;
   end Um;
-----------------------------------------------------------------------------------------------
---------- 5 Beispielkonfigurationen ----------------------------------------------------------
-----------------------------------------------------------------------------------------------
   function DrStart1 return Konfiguration is

      Var1,Var2,Var3 : Variable;
      W1             : Wert (Undef);
      W2,W3          : Wert (Def_Wert);
      T1,T2          : Teile;
      U              : Umgebung;
    
   begin
      W2.Zahl_Wert       := 17;
      W3.Zahl_Wert       := 4;
      Var1.Variable_Name := To_Unbounded_String("d");
      Var2.Variable_Name := To_Unbounded_String("x");
      Var3.Variable_Name := To_Unbounded_String("y");
      Var1.Variable_Wert := W1;
      Var2.Variable_Wert := W2;
      Var3.Variable_Wert := W3;

      U  := new Umgebung_Rec'(Var1,
                              new Umgebung_Rec'(Var2,
                                                new Umgebung_Rec'(Var3,
                                                                  null)));
      T1 := null;
      T2 := new Teile_Rec'(
                new Teil_Rec'(Anw_F,
                              new Anweisung_F_Rec'(
                                  (new Anweisung_Rec'(Zuweisung,
                                                      To_Unbounded_String("d"),
                                                      new Ausdruck_Rec'(Zahl,0))
                                  ),null)
                              ),null);
      return (U,T1,T2);
   end DrStart1;

-- d := 0;

-- Einfache Zuweisung

   function Z1 return Zustand is

   begin
      return (Akt,DrStart1);
   end Z1;
-----------------------------------------------------------------------------------------------
   function DrStart2 return Konfiguration is

      Var1,Var2,Var3,Var4 : Variable;
      W4                  : Wert (Undef);
      W1,W2,W3            : Wert (Def_Wert);
      T1,T2               : Teile;
      U                   : Umgebung;

   begin
      W1.Zahl_Wert       := 1;
      W2.Zahl_Wert       := 17;
      W3.Zahl_Wert       := 4;
      Var1.Variable_Name := To_Unbounded_String("d");
      Var2.Variable_Name := To_Unbounded_String("x");
      Var3.Variable_Name := To_Unbounded_String("y");
      Var4.Variable_Name := To_Unbounded_String("z");
      Var1.Variable_Wert := W1;
      Var2.Variable_Wert := W2;
      Var3.Variable_Wert := W3;
      Var4.Variable_Wert := W4;

      U  := new Umgebung_Rec'
                (Var1,new Umgebung_Rec'
                          (Var2,new Umgebung_Rec'
                                    (Var3,new Umgebung_Rec'
                                              (Var4,null))));
      T1 := null;
      T2 := new Teile_Rec'
                (new Teil_Rec'
                     (Anw_F,
                     new Anweisung_F_Rec'
                         (new Anweisung_Rec'
                              (Zuweisung,
                               To_Unbounded_String("d"),
                               new Ausdruck_Rec'
                                   (Verkn,
                                    Plus,
                                    new Ausdruck_Rec'(Name,To_Unbounded_String("d")),
                                    new Ausdruck_Rec'(Zahl,1))
                                    ),
                         (new Anweisung_F_Rec'
                              (new Anweisung_Rec'
                                   (Zuweisung,
                                    To_Unbounded_String("x"),
                                    new Ausdruck_Rec'
                                        (Verkn,
                                         Minus,
                                         new Ausdruck_Rec'(Name,To_Unbounded_String("x")),
                                         new Ausdruck_Rec'(Name,To_Unbounded_String("y"))))
                         ,null))))
                ,null);
   return (U,T1,T2);
   end DrStart2;

-- d := (d+1);
-- x := (x-y);

-- Einfache arithmetische Ausdrcke

   function Z2 return Zustand is

   begin
      return (Akt,DrStart2);
   end Z2;
-----------------------------------------------------------------------------------------------
   function DrStart3 return Konfiguration is

      Var1,Var2, Var3 : Variable;
      W1              : Wert (Undef);
      W2,W3           : Wert (Def_Wert);
      T1,T2           : Teile;
      U               : Umgebung;

   begin
      W2.Zahl_Wert       := 17;
      W3.Zahl_Wert       := 4;
      Var1.Variable_Name := To_Unbounded_String("d");
      Var2.Variable_Name := To_Unbounded_String("x");
      Var3.Variable_Name := To_Unbounded_String("y");
      Var1.Variable_Wert := W1;
      Var2.Variable_Wert := W2;
      Var3.Variable_Wert := W3;

      U  := new Umgebung_Rec'
                (Var1,new Umgebung_Rec'
                          (Var2,new Umgebung_Rec'
                                    (Var3,null)));
      T1 := null;
      T2 := new Teile_Rec'
                (new Teil_Rec'
                (Anw_F,new Anweisung_F_Rec'
                           (new Anweisung_Rec'
                                (Zuweisung,
                                 To_Unbounded_String("d"),
                                 new Ausdruck_Rec'(Zahl,0)),
                           new Anweisung_F_Rec'
                               (new Anweisung_Rec'
                               (Schleife,
                                new Bedingung_Rec'
                                    (Vergleich,
                                     Gg,
                                     new Ausdruck_Rec'
                                         (Name,To_Unbounded_String("x")),
                                     new Ausdruck_Rec'
                                         (Name,To_Unbounded_String("y"))),
                                new Anweisung_F_Rec'
                                   (new Anweisung_Rec'
                                        (Zuweisung,
                                         To_Unbounded_String("d"),
                                         new Ausdruck_Rec'
                                             (Verkn,
                                              Plus,
                                              new Ausdruck_Rec'
                                                  (Name,To_Unbounded_String("d")),
                                              new Ausdruck_Rec'(Zahl,1))),
                                   new Anweisung_F_Rec'
                                       (new Anweisung_Rec'
                                            (Zuweisung,
                                             To_Unbounded_String("x"),
                                             new Ausdruck_Rec'
                                                 (Verkn,
                                                  Minus,
                                                  new Ausdruck_Rec'
                                                      (Name,To_Unbounded_String("x")),
                                                  new Ausdruck_Rec'
                                                      (Name,To_Unbounded_String("y"))))
                                   ,null)))
                               ,null)))
                ,null);
      return (U,T1,T2);
   end DrStart3;

-- d := 0;
-- while (x>=y) do
--   d := (d+1);
--   x := (x-y);
-- od

-- Division mit Rest

   function Z3 return Zustand is

   begin
      return (Akt,DrStart3);
   end Z3;
-----------------------------------------------------------------------------------------------
   function DrStart4 return Konfiguration is

      Var1,Var2 : Variable;
      W1        : Wert (Undef);
      W2        : Wert (Def_Wert);
      T1,T2     : Teile;
      U         : Umgebung;

   begin
      W2.Zahl_Wert       := 6;
      Var1.Variable_Name := To_Unbounded_String("f");
      Var2.Variable_Name := To_Unbounded_String("n");
      Var1.Variable_Wert := W1;
      Var2.Variable_Wert := W2;

      U  := new Umgebung_Rec'
                (Var1,new Umgebung_Rec'
                          (Var2,null));
      T1 := null;
      T2 := new Teile_Rec'
                (new Teil_Rec'
                     (Anw_F,new Anweisung_F_Rec'
                                (new Anweisung_Rec'
                                     (Zuweisung,
                                      To_Unbounded_String("f"),
                                      new Ausdruck_Rec'(Zahl,1)),
                                new Anweisung_F_Rec'
                                    (new Anweisung_Rec'
                                         (Schleife,
                                          new Bedingung_Rec'
                                              (Vergleich,
                                              Gr,
                                              new Ausdruck_Rec'(Name,To_Unbounded_String("n")),
                                              new Ausdruck_Rec'(Zahl,1)),
                                          new Anweisung_F_Rec'
                                              (new Anweisung_Rec'
                                                   (Zuweisung,
                                                    To_Unbounded_String("f"),
                                                    new Ausdruck_Rec'
                                                        (Verkn,
                                                         Mal,
                                                         new Ausdruck_Rec'
                                                             (Name,To_Unbounded_String("f")),
                                                         new Ausdruck_Rec'
                                                             (Name,To_Unbounded_String("n")))),
                                              new Anweisung_F_Rec'
                                                  (new Anweisung_Rec'
                                                       (Zuweisung,
                                                        To_Unbounded_String("n"),
                                                        new Ausdruck_Rec'
                                                            (Verkn,
                                                             Minus,
                                                             new Ausdruck_Rec'
                                                                (Name,To_Unbounded_String("n")),
                                                             new Ausdruck_Rec'(Zahl,1)))
                                                  ,null)))
                                 ,null)))
                ,null);
      return (U,T1,T2);
   end DrStart4;

-- f := 1;
-- while (n>1) do
--   f:=(f*n);
--   n:=(n-1);
-- od

-- Fakulttsfunktion

   function Z4 return Zustand is

   begin
      return (Akt,DrStart4);
   end Z4;
-----------------------------------------------------------------------------------------------
   function DrStart5 return Konfiguration is

      Var1,Var2 : Variable;
      W1        : Wert (Undef);
      W2        : Wert (Def_Wert);
      T1,T2     : Teile;
      U         : Umgebung;

   begin
      W2.Zahl_Wert       := 0;
      Var1.Variable_Name := To_Unbounded_String("f");
      Var2.Variable_Name := To_Unbounded_String("n");
      Var1.Variable_Wert := W1;
      Var2.Variable_Wert := W2;

      U  := new Umgebung_Rec'
                (Var1,new Umgebung_Rec'
                          (Var2,null));
      T1 := null;
      T2 := new Teile_Rec'
                (new Teil_Rec'
                     (Anw_F,
                      new Anweisung_F_Rec'
                          (new Anweisung_Rec'
                               (Zuweisung,
                                To_Unbounded_String("f"),
                                new Ausdruck_Rec'(Zahl,1)),
                          new Anweisung_F_Rec'
                              (new Anweisung_Rec'
                                   (Schleife,
                                    new Bedingung_Rec'
                                        (Vergleich,
                                         Gr,
                                         new Ausdruck_Rec'(Name,To_Unbounded_String("n")),
                                         new Ausdruck_Rec'(Zahl,1)),
                                    new Anweisung_F_Rec'
                                        (new Anweisung_Rec'
                                             (Zuweisung,
                                              To_Unbounded_String("f"),
                                              new Ausdruck_Rec'
                                                  (Verkn,
                                                   Mal,
                                                   new Ausdruck_Rec'
                                                       (Name,To_Unbounded_String("f")),
                                                   new Ausdruck_Rec'
                                                       (Name,To_Unbounded_String("n")))),
                                        new Anweisung_F_Rec'
                                            (new Anweisung_Rec'
                                                 (Zuweisung,
                                                  To_Unbounded_String("n"),
                                                  new Ausdruck_Rec'
                                                      (Verkn,
                                                       Minus,
                                                       new Ausdruck_Rec'
                                                           (Name,To_Unbounded_String("n")),
                                                       new Ausdruck_Rec'(Zahl,1)))
                                            ,null)))
                              ,null)))
                ,null);
      return (U,T1,T2);
   end DrStart5;

   function Z5 return Zustand is

-- f := 1;
-- while (n>1) do
--   f:=(f*n);
--   n:=(n-1);
-- od

-- Fakulttsfunktion, ohne Schleifendurchlauf (0!)

   begin
      return (Akt,DrStart5);
   end Z5;
-----------------------------------------------------------------------------------------------
---------- Zeige-Funktionen -------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
   function Zeige_Op (A : Operator) return String is

      Erg : Unbounded_String := Null_Unbounded_String;

   begin
      case A is
         when Plus   => Erg := To_Unbounded_String("+");

         when Minus  => Erg := To_Unbounded_String("-");

         when Mal    => Erg := To_Unbounded_String("*");

         when Div    => Erg := To_Unbounded_String("`div`");

         when Modulo => Erg := To_Unbounded_String("`mod`");
      end case;
      return To_String(Erg);
   end Zeige_Op;
-----------------------------------------------------------------------------------------------
   function Zeige_Vergl (A : Vergleicher ) return String is

      Erg : Unbounded_String := Null_Unbounded_String;

   begin
      case A is
         when Kl => Erg := To_Unbounded_String("<");

         when Gl => Erg := To_Unbounded_String("=");

         when Gr => Erg := To_Unbounded_String(">");

         when Kg => Erg := To_Unbounded_String("<=");

         when Ug => Erg := To_Unbounded_String("/=");

         when Gg => Erg := To_Unbounded_String(">=");
      end case;
      return To_String(Erg);
   end Zeige_Vergl;
-----------------------------------------------------------------------------------------------
   function Zeige_Ausdr (A : Ausdruck) return String is

      Erg      : Unbounded_String := Null_Unbounded_String;
      Li,Mi,Re : Unbounded_String := Null_Unbounded_String;

   begin
      case A.Form is
         when Zahl  => Erg := To_Unbounded_String(Integer'Image(A.Zahl_Wert));

         when Name  => Erg := A.Name_Wert;

         when Verkn => Li  := To_Unbounded_String(Zeige_Ausdr(A.Links_Ausdruck));
                       Mi  := To_Unbounded_String(Zeige_Op(A.Operator_Wert));
                       Re  := To_Unbounded_String(Zeige_Ausdr(A.Rechts_Ausdruck));
                       Erg := "(" & Li & Mi & Re & ")";
      end case;
      return To_String(Erg);
   end Zeige_Ausdr;
-----------------------------------------------------------------------------------------------
   function Zeige_Bed (A : Bedingung) return String is

      Erg       : Unbounded_String := Null_Unbounded_String;
      Li,Mi,Re  : Unbounded_String := Null_Unbounded_String;

   begin
      case A.Form is
         when Wahr      => if A.Wahr_Wert then
                              Erg := To_Unbounded_String("true");
                           else
                              Erg := To_Unbounded_String("false");
                           end if;

         when Vergleich => Li  := To_Unbounded_String(Zeige_Ausdr(A.Links_Ausdruck));
                           Mi  := To_Unbounded_String(Zeige_Vergl(A.Vergleichs_Wert));
                           Re  := To_Unbounded_String(Zeige_Ausdr(A.Rechts_Ausdruck));
                           Erg := "(" & Li & Mi & Re & ")";
      end case;
      return To_String(Erg);
   end Zeige_Bed;
-----------------------------------------------------------------------------------------------
   function Zeige_Anw_F (A : Anweisung_F) return String is

      Erg : Unbounded_String := Null_Unbounded_String;

      function Zeige_Anw (A : Anweisung) return String is

         Erglok      : Unbounded_String := Null_Unbounded_String;
         Bed,Afa,Afb : Unbounded_String := Null_Unbounded_String;

      begin
         case A.Form is
            when Fall     => Bed    := To_Unbounded_String(Zeige_Bed(A.Bedingungs_Wert_Fall));
                             Afa    := To_Unbounded_String(Zeige_Anw_F(A.Erst_Anweisung_Fall));
                             Afb    := To_Unbounded_String(Zeige_Anw_F(A.Zweit_Anweisung_Fall));
                             Erglok := " if " & Bed & " then " & Afa & " else " & Afb & " fi";

            when Schleife => Bed := To_Unbounded_String(Zeige_Bed(A.Bedingungs_Wert_Schleife));
                             Afa := To_Unbounded_String(Zeige_Anw_F(A.Erst_Anweisung_Schleife));
                             Erglok := " while " & Bed & " do " & Afa & " od ";

            when Skip      => Erglok := To_Unbounded_String("skip");

            when Zuweisung => Erglok := A.Zuweisungs_Name & 
                                        ":=" &
                                        To_Unbounded_String(Zeige_Ausdr(A.Links_Ausdruck));
         end case;
         return To_String(Erglok);
      end Zeige_Anw;

   begin
      if A /= null then
         if A.Rest = null then
            Erg := To_Unbounded_String(Zeige_Anw(A.Anw));
         else
            Erg := To_Unbounded_String(Zeige_Anw(A.Anw)) & 
                   ";" & 
                   To_Unbounded_String(Zeige_Anw_F(A.Rest));
         end if;
      else
         Erg := To_Unbounded_String("[]");
      end if;
      return To_String(Erg);
   end Zeige_Anw_F;
-----------------------------------------------------------------------------------------------
   function Zeige_Teile (A : Teile) return String is

      Erg : Unbounded_String := Null_Unbounded_String;
      -----------------------------------------------------------------------------------------
      function Zeige_Teil (A : Teil) return String is

         Erglok : Unbounded_String := Null_Unbounded_String;

      begin
         case A.Form is
            when Anw     => Erglok := To_Unbounded_String
                                      (Zeige_Anw_F(new Anweisung_F_Rec'(A.Teil_Anw,null)));

            when Anw_F   => Erglok := To_Unbounded_String(Zeige_Anw_F(A.Teil_Anw_F));

            when Bed     => Erglok := To_Unbounded_String(Zeige_Bed(A.Teil_Bed));

            when Ausdr   => Erglok := To_Unbounded_String(Zeige_Ausdr(A.Teil_Ausdr));

            when Bez     => Erglok := (A.Teil_Bez);

            when Zwert   => Erglok := To_Unbounded_String(Integer'Image(A.Teil_Zwert));

            when Wwert   => if A.Teil_Wwert then
                               Erglok := To_Unbounded_String("true");
                            else
                               Erglok := To_Unbounded_String("false");
                            end if;

            when Op      => Erglok := To_Unbounded_String(Zeige_Op(A.Teil_Op));

            when Rop     => Erglok := To_Unbounded_String(Zeige_Vergl(A.Teil_Rop));

            when If_Case => Erglok := To_Unbounded_String("if");

            when Do_Case => Erglok := To_Unbounded_String("do");

            when Zuw     => Erglok := To_Unbounded_String(":=");
         end case;
         return To_String(Erglok);
      end Zeige_Teil;
      -----------------------------------------------------------------------------------------
   begin
      if A /= null then
         if A.Rest = null then
            Erg := To_Unbounded_String(Zeige_Teil(A.T));
         else
            Erg := To_Unbounded_String(Zeige_Teil(A.T)) & 
                   "&" & 
                   To_Unbounded_String(Zeige_Teile(A.Rest));
         end if;
      end if;
      return To_String(Erg);
   end Zeige_Teile;
-----------------------------------------------------------------------------------------------
   function Zeige_Wert (A : Wert) return String is

      Erg : Unbounded_String := Null_Unbounded_String;

   begin
      case A.Form is
         when Undef    => Erg := To_Unbounded_String("_");

         when Def_Wert => Erg := To_Unbounded_String(Integer'Image(A.Zahl_Wert));
      end case;
      return To_String(Erg);
   end Zeige_Wert;
-----------------------------------------------------------------------------------------------
   function Zeige_Var (A : Variable) return String is

      Erg : Unbounded_String := Null_Unbounded_String;

   begin
      Erg := To_Unbounded_String("(" & To_String(A.Variable_Name) & 
                                 "," & Zeige_Wert(A.Variable_Wert) & 
                                 ")");
      return To_String(Erg);
   end Zeige_Var;
-----------------------------------------------------------------------------------------------
   function Zeige_Vars (A : Umgebung) return String is

      Erg : Unbounded_String := Null_Unbounded_String;

   begin
      if A /= null then
         Erg := To_Unbounded_String(Zeige_Var (A.Elem) & Zeige_Vars (A.Next));
      end if;
      return To_String(Erg);
   end Zeige_Vars;
-----------------------------------------------------------------------------------------------
   function Rev (T : Teile) return Teile is

      Tx     : Teile := T;
      Result : Teile;

   begin
      while Tx /= null loop
         Result := new Teile_Rec'(Tx.T, Result);
         Tx     := Tx.Rest;
      end loop;
      return Result;
   end Rev;
-----------------------------------------------------------------------------------------------
   function Zeige_Konf (A : Konfiguration) return String is

      Erg : Unbounded_String := Null_Unbounded_String;

   begin
      Erg := To_Unbounded_String("<< " & Zeige_Vars (A.Um) &
                                 " , " & Zeige_Teile (Rev(A.Wk)) &
                                 " , " & Zeige_Teile (A.Zk) &
                                 " >>");
      return To_String(Erg);
   end Zeige_Konf;

-- Die Funktion zeigeKonf wird von der Tracerfunktion genutzt.
-- zeigeKonf setzt aus den einzelnen Bestandteilen des ber-
-- gebenen Zustands, die aktuelle Umgebung und die beiden
-- Kellerspeicher, einen einzigen String - in spitze Klammern
-- gesetzt - zusammen. Smtliche Zeigefunktionen sind an die
-- vorher eingefhrte Syntax mit den entsprechenden Datentypen
-- des Interpreters angelehnt. Dazu werden die jeweiligen Be-
-- standteile der Konfiguration in den entsprechenden zeige-
-- Funktionen zunchst vom bergebenen Wert in einen Unbounded_
-- String mit den entsprechenden Zeichen umgewandelt, um dann
-- aber erst nach der Umwandlung in einer normalen String den
-- Funktionswert auszugeben. So werden die mehreren mglichen
-- Lngen des tatschlichen Rckgabestring bercktsichtigt.

-- Aufruf:   Put_Line(Zeige_Konf(Drstart2));
-- Ergebnis: << (d, 1)(x, 17)(y, 4)(z,_) ,  , d:=(d +  1);x:=(x - y) >>
-- Aufruf:   Put_Line(Zeige_Konf(Drstart4));
-- Ergebnis: << (f,_)(n, 6) ,  , f:= 1; while (n >  1) do f:=(f * n);n:=(n -  1) od  >>

-----------------------------------------------------------------------------------------------
---------- DELTA-Funktion (Zustandsbergangsfunktion) -----------------------------------------
--------------------------------------------------------------- teilweise eingerckt (***) ----
-----------------------------------------------------------------------------------------------
   function Delta2 (Z : Zustand) return Zustand is

      Result      : Zustand        := Z;
      EndOfResult : Zustand (Ende);

   begin
      New_Line;
      if (not (Result.Form = Ende)) and then (not (Result.Akt_Zustand.Zk = null)) then
         case Result.Akt_Zustand.Zk.T.Form is
            --Anw
            when Anw    => case Result.Akt_Zustand.Zk.T.Teil_Anw.Form is
                             --Anw: Fall
                             when Fall => Put_Line("-- (12 : If-Anweisung aufloesen)--");
      --(***)
      -- lse Fall auf und hnge die Komponenten vor zk (ohne Anw)
      Result.Akt_Zustand.Zk := new Teile_Rec'(new Teil_Rec'
                                (Bed,Result.Akt_Zustand.Zk.T.Teil_Anw.Bedingungs_Wert_Fall),
                                new Teile_Rec'(new Teil_Rec'(Form=>If_Case),
                                 new Teile_Rec'(new Teil_Rec'
                                  (Anw_F,Result.Akt_Zustand.Zk.T.Teil_Anw.Erst_Anweisung_Fall),
                                  new Teile_Rec'(new Teil_Rec'
                                   (Anw_F,Result.Akt_Zustand.Zk.T.Teil_Anw.Zweit_Anweisung_Fall),
                                    Result.Akt_Zustand.Zk.Rest))));

                             --Anw: Schleife
                             when Schleife => Put_Line("-- (15 : Schleife aufloesen)--");

      --(***)
      -- lse schleife auf und hnge die Teile vor zk (ohne die Schleife)
      Result.Akt_Zustand.Zk := new Teile_Rec'(new Teil_Rec'
                                (Bed,Result.Akt_Zustand.Zk.T.Teil_Anw.Bedingungs_Wert_Schleife),
                                new Teile_Rec'(new Teil_Rec'(Form=>Do_Case),
                                 new Teile_Rec'(new Teil_Rec'
                                  (Bed,Result.Akt_Zustand.Zk.T.Teil_Anw.Bedingungs_Wert_Schleife),
                                  new Teile_Rec'(new Teil_Rec'
                                   (Anw_F,Result.Akt_Zustand.Zk.T.Teil_Anw.Erst_Anweisung_Schleife),
                                   Result.Akt_Zustand.Zk.Rest))));

                           --               if result.akt_zustand.zk /= null then
                           --                 result.akt_zustand.zk := result.akt_zustand.zk.rest;
                           --               end if;

                             -- Anw: Skip
                             when Skip => Put_Line("-- (8 : Skipanweisung)--");
                                        -- mache nichts..
                                        if Result.Akt_Zustand.Zk /= null then
                                        Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                                        end if;
                             -- Anw: Zuweisung
                             when Zuweisung => Put_Line("-- (9 : Zuweisung aufloesen)--");
      
      --(***)
      -- Lse Zuweisung und ersetze sie durch ihren Inhalt
      Result.Akt_Zustand.Zk := new Teile_Rec'(new Teil_Rec'
                                (Ausdr,Result.Akt_Zustand.Zk.T.Teil_Anw.Links_Ausdruck),
                                new Teile_Rec'(new Teil_Rec'(Form=>Zuw),
                                 new Teile_Rec'(new Teil_Rec'
                                  (Bez,Result.Akt_Zustand.Zk.T.Teil_Anw.Zuweisungs_Name),
                                  Result.Akt_Zustand.Zk.Rest)));

                           end case;
            --Anw_F
            when Anw_F  => -- unterscheidung von 11) und 11)' (Teil_anw_f leer oder nicht)
                           if Result.Akt_Zustand.Zk.T.Teil_Anw_F = null then
                              Put_Line("-- (11' : leere Anweisungsfolge aufloesen)--");
                              -- leere Anweisungsfolge: wird gelscht
                              Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                           else
                              Put_Line("-- (11 : nichtleere Anweisungsfolge aufloesen)--");
                              -- hnge 1.Anweisung aus Anw_f vor Anw_f
                              Result.Akt_Zustand.Zk := new Teile_Rec'(new Teil_Rec'
                                                       (Anw,Result.Akt_Zustand.Zk.T.Teil_Anw_F.Anw),
                                                       Result.Akt_Zustand.Zk);
                              -- und lsche sie dort
                              if Result.Akt_Zustand.Zk.Rest.T.Teil_Anw_F /= null then
                                 Result.Akt_Zustand.Zk.Rest.T.Teil_Anw_F :=   --(***)
                                                      Result.Akt_Zustand.Zk.Rest.T.Teil_Anw_F.Rest;
                              end if;
                           end if;
            --Bed
            when Bed    => case Result.Akt_Zustand.Zk.T.Teil_Bed.Form is
                              --Bed: Wahr
                              when Wahr => Put_Line("-- (5 : Wahrheitswert lesen)--");
                                           
      --(***)
      -- kopiere Wahrheitswert von zk nach wk
      Result.Akt_Zustand.Wk := new Teile_Rec'(new Teil_Rec'
                                (Wwert,Result.Akt_Zustand.Zk.T.Teil_Bed.Wahr_Wert),
                                Result.Akt_Zustand.Wk);

                                           -- und lsche ihn in zk
                                           if Result.Akt_Zustand.Zk /= null then
                                              Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                                           end if;

                              --Bed: Vergleich
                              when Vergleich => Put_Line("-- (6 : Bedingung aufloesen)--");
                              
      --(***)
      -- lse die Verknpfung auf und hnge sie an (oder vor) zk (ohne die Verknpfung)
      Result.Akt_Zustand.Zk := new Teile_Rec'(new Teil_Rec'
                                (Ausdr,Result.Akt_Zustand.Zk.T.Teil_Bed.Links_Ausdruck),
                                new Teile_Rec'(new Teil_Rec'
                                 (Ausdr,Result.Akt_Zustand.Zk.T.Teil_Bed.Rechts_Ausdruck),
                                 new Teile_Rec'(new Teil_Rec'
                                  (Rop,Result.Akt_Zustand.Zk.T.Teil_Bed.Vergleichs_Wert),
                                  Result.Akt_Zustand.Zk.Rest)));
                           
                           end case;
            --Ausdr
            when Ausdr  => case Result.Akt_Zustand.Zk.T.Teil_Ausdr.Form is
                              --Ausdr: Zahl
                              when Zahl  => Put_Line("-- (1 : Zahl lesen) --");
                              
      --(***)
      -- kopiere die Zahl von zk nach wk
      Result.Akt_Zustand.Wk := new Teile_Rec'(new Teil_Rec'
                                (Zwert,Result.Akt_Zustand.Zk.T.Teil_Ausdr.Zahl_Wert),
                                Result.Akt_Zustand.Wk);
                                            
                                            -- und lsche sie in zk
                                            if Result.Akt_Zustand.Zk /= null then
                                               Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                                            end if;

                              --Ausdr: Name
                              when Name => Put_Line("-- (2 : Variable lesen, Wert aus Umgebung)--");
                              
      --(***)
      -- kopiere Belegung der Variable unter um nach wk
      Result.Akt_Zustand.Wk := new Teile_Rec'(new Teil_Rec'
                               (Zwert,--finde richtigen Wert fr Variable
                                Wert_Suche(Result.Akt_Zustand.Zk.T.Teil_Ausdr.Name_Wert,
                                           Result.Akt_Zustand.Um).Zahl_Wert),
                                           Result.Akt_Zustand.Wk);

                                            -- und lsche sie in zk
                                            if Result.Akt_Zustand.Zk /= null then
                                               Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                                            end if;
                              --Ausdr: Verkn
                              when Verkn => Put_Line("-- (3 : Verknuepfung aufloesen)--");
                              
      --(***)
      -- lse die Verknpfung auf und hnge sie an (oder vor) zk (ohne die Verknpfung)
      Result.Akt_Zustand.Zk := new Teile_Rec'(new Teil_Rec'
                                (Ausdr,Result.Akt_Zustand.Zk.T.Teil_Ausdr.Links_Ausdruck),
                                new Teile_Rec'(new Teil_Rec'
                                 (Ausdr,Result.Akt_Zustand.Zk.T.Teil_Ausdr.Rechts_Ausdruck),
                                 new Teile_Rec'(new Teil_Rec'
                                  (Op,Result.Akt_Zustand.Zk.T.Teil_Ausdr.Operator_Wert),
                                  Result.Akt_Zustand.Zk.Rest)));

                           end case;
            when Bez    => null;     --hier auf string umsteigen
            when Zwert  => null;
            when Wwert  => null;
            --Op
            when Op     => Put_Line("-- (4 : Arithmetische Operation)--");
                           -- hnge das Ergebnis der Operation mit den letzten beiden 
                           -- Werte von wk an wk statt der beiden Werte
                           Result.Akt_Zustand.Wk := new Teile_Rec'
                                                        (new Teil_Rec'(Zwert,Ada_Op
                                                           (Result.Akt_Zustand.Zk.T.Teil_Op,
                                                            Result.Akt_Zustand.Wk.Rest.T.Teil_Zwert,
                                                            Result.Akt_Zustand.Wk.T.Teil_Zwert)),
                                                        Result.Akt_Zustand.Wk.Rest.Rest);
                           -- und lsche den Operator in zk
                           if Result.Akt_Zustand.Zk /= null then
                              Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                           end if;
            -- Rop
            when Rop    => Put_Line("-- (7 : Vergleichsoperation)--");
                           -- hnge das Ergebnis der Operation mit den letzten beiden 
                           -- Werte von wk an wk statt der beiden Werte
                           Result.Akt_Zustand.Wk := new Teile_Rec'
                                                        (new Teil_Rec'(Wwert,Ada_Rel_Op
                                                           (Result.Akt_Zustand.Zk.T.Teil_Rop,
                                                            Result.Akt_Zustand.Wk.Rest.T.Teil_Zwert,
                                                            Result.Akt_Zustand.Wk.T.Teil_Zwert)),
                                                        Result.Akt_Zustand.Wk.Rest.Rest);
                           -- und lsche den Operator in zk
                           Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
            --If_case
            when If_Case => -- Unterscheidung zwischen 13) und 14)
                            -- lsche if
                            Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                            if Result.Akt_Zustand.Wk.T.Teil_Wwert then
                               Put_Line("-- (13 : If-Anweisung True-Fall)--");
                               -- lsche Anw_f2
                               Result.Akt_Zustand.Zk.Rest := Result.Akt_Zustand.Zk.Rest.Rest;
                            else
                               Put_Line("-- (14 : If-Anweisung False-Fall)--");
                               -- lsche Anw_f1
                               Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                            end if;
                            -- lsche WWert in wk
                            Result.Akt_Zustand.Wk := Result.Akt_Zustand.Wk.Rest;
            --Do_case
            when Do_Case => -- Unterscheidung zwischen 16) und 17)
                            -- lsche Do_case
                            Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest;
                            if Result.Akt_Zustand.Wk.T.Teil_Wwert then
                               Put_Line("-- (16 : Schleifenbedingung True)--");
                               
      --(***)
      -- extrahiere Anw_f und erstelle neue Schleife
      Result.Akt_Zustand.Zk := new Teile_Rec'(new Teil_Rec'
                                (Anw_F,Result.Akt_Zustand.Zk.Rest.T.Teil_Anw_F),
                                new Teile_Rec'(new Teil_Rec'
                                 (Anw,new Anweisung_Rec'(Schleife,
                                                         Result.Akt_Zustand.Zk.T.Teil_Bed,
                                                         Result.Akt_Zustand.Zk.Rest.T.Teil_Anw_F)),
                                                         Result.Akt_Zustand.Zk.Rest.Rest));
                            else
                               Put_Line("-- (17 : Schleifenbedingung False)--");
                               -- lsche SchleifenRest
                               if Result.Akt_Zustand.Zk /= null then
                                  Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest.Rest;
                               end if;
                            end if;
                            -- lsche WWert
                            if Result.Akt_Zustand.Wk /= null then
                               Result.Akt_Zustand.Wk := Result.Akt_Zustand.Wk.Rest;
                            end if;
            --Zuw
            when Zuw     => Put_Line("-- (10 : Zuweisung ausfuehren)--");
                            -- verndere Umgebung
                            Result.Akt_Zustand.Um := Wert_Eintrag
                                                         (Result.Akt_Zustand.Zk.Rest.T.Teil_Bez,
                                                          Result.Akt_Zustand.Wk.T.Teil_Zwert,
                                                          Result.Akt_Zustand.Um);
                            -- lsche ZWert in wk
                            Result.Akt_Zustand.Wk := Result.Akt_Zustand.Wk.Rest;
                            -- und lsche Zuweisung und Bezeichner in zk
                            Result.Akt_Zustand.Zk := Result.Akt_Zustand.Zk.Rest.Rest;

         end case;
      else
         Put_Line("-- (Ende der Auswertung)--");
         EndOfResult.Ende_Zustand := Result.Akt_Zustand.Um;
         Result := EndOfResult;
      end if;

      return Result;
   end Delta2;
-----------------------------------------------------------------------------------------------
--   function Regel (
--         Zust : Zustand )
--     return Unbounded_String is
--   begin
--      if Zust.Form = Akt then
--         if Zust.Akt_Zustand.Zk/= null then
--            if Zust.Akt_Zustand.Zk.T.Form=Ausdr then
--               if Zust.Akt_Zustand.Zk.T.Teil_Ausdr.Form = Zahl then
--                  return To_Unbounded_String("-- (1 : Zahl lesen)--") ;
--               end if;
--               if Zust.Akt_Zustand.Zk.T.Teil_Ausdr.Form = Name then
--                  return To_Unbounded_String("-- (2 : Variable lesen, Wert aus Umgebung)--");
--               end if;
--               if Zust.Akt_Zustand.Zk.T.Teil_Ausdr.Form = Verkn then
--                  return To_Unbounded_String("-- (3 : Verknuepfung aufloesen)--");
--               end if;
--            end if;
--         end if;
--         if (Zust.Akt_Zustand.Wk /= null) and then (Zust.Akt_Zustand.Wk.Rest = null) then
--            Put_Line("wk.rest = null");
--         end if;
--         if  Zust.Akt_Zustand.Wk /=null and Zust.Akt_Zustand.Zk /=null then
--            if Zust.Akt_Zustand.Wk.Rest /=null then
--               if Zust.Akt_Zustand.Wk.T.Form=Zwert  and
--                     Zust.Akt_Zustand.Wk.Rest.T.Form=Zwert and
--                     Zust.Akt_Zustand.Zk.T.Form=Op then
--                  return To_Unbounded_String("-- (4 : Arithmetische Operation)--");
--               end if;
--            end if;
--         end if;
--         if  Zust.Akt_Zustand.Zk /=null then
--            if Zust.Akt_Zustand.Zk.T.Form = Bed then
--               if Zust.Akt_Zustand.Zk.T.Teil_Bed.Form = Wahr then
--                  return To_Unbounded_String("-- (5 : Wahrheitswert lesen)--");
--               end if;
--               if Zust.Akt_Zustand.Zk.T.Teil_Bed.Form = Vergleich then
--                  return To_Unbounded_String("-- (6 : Bedingung aufloesen)--");
--               end if;
--            end if;
--         end if;
--         if  Zust.Akt_Zustand.Wk /=null and Zust.Akt_Zustand.Zk/=null then
--            if Zust.Akt_Zustand.Wk.Rest /=null then
--               if Zust.Akt_Zustand.Wk.T.Form=Zwert and
--                     Zust.Akt_Zustand.Wk.Rest.T.Form=Zwert and
--                     Zust.Akt_Zustand.Zk.T.Form=Rop then
--                  return To_Unbounded_String("-- (7 : Vergleichsoperation)--");
--               end if;
--            end if;
--         end if;
--         if Zust.Akt_Zustand.Zk /=null then
--            if Zust.Akt_Zustand.Zk.T.Form=Anw then
--               if Zust.Akt_Zustand.Zk.T.Teil_Anw.Form = Skip then
--                  return To_Unbounded_String("-- (8 : Skipanweisung)--");
--               end if;
--               if Zust.Akt_Zustand.Zk.T.Teil_Anw.Form = Zuweisung then
--                  return To_Unbounded_String("-- (9 : Zuweisung aufloesen)--");
--               end if;
--            end if;
--         end if;
--         if Zust.Akt_Zustand.Wk /= null and Zust.Akt_Zustand.Zk /= null then
--            if Zust.Akt_Zustand.Zk.Rest /= null then
--               if Zust.Akt_Zustand.Wk.T.Form=Zwert and
--                     Zust.Akt_Zustand.Zk.T.Form=Zuw and
--                     Zust.Akt_Zustand.Zk.Rest.T.Form=Bez then
--                  return To_Unbounded_String("-- (10 : Zuweisung ausfuehren)--");
--               end if;
--            end if;
--         end if;
--         if Zust.Akt_Zustand.Zk/= null then
--            if Zust.Akt_Zustand.Zk.T.Form = Anw_F then
--               if Zust.Akt_Zustand.Zk.T.Teil_Anw_F /= null then
--                  return To_Unbounded_String("-- (11 : nichtleere Anweisungsfolge aufloesen)--");
--               else
--                  return To_Unbounded_String("-- (11' : leere Anweisungsfolge aufloesen)--");
--               end if;
--            end if;
--         end if;

--         if Zust.Akt_Zustand.Zk /= null then
--            if Zust.Akt_Zustand.Zk.T.Form = Anw and then
--                  Zust.Akt_Zustand.Zk.T.Teil_Anw.Form = Fall then
--               return To_Unbounded_String("-- (12 : If-Anweisung aufloesen)--");
--            end if;
--         end if;

--         if Zust.Akt_Zustand.Zk/=null and Zust.Akt_Zustand.Wk/= null then
--            if Zust.Akt_Zustand.Zk.Rest/= null then
--               if Zust.Akt_Zustand.Wk.T.Form = Wwert and
--                     Zust.Akt_Zustand.Wk.T.Teil_Wwert and
--                     Zust.Akt_Zustand.Zk.T.Form = If_Case and
--                     Zust.Akt_Zustand.Zk.Rest.T.Form = Anw_F then
--                  return To_Unbounded_String("-- (13 : If-Anweisung True-Fall)--");
--               end if;
--            end if;
--         end if;

--         if Zust.Akt_Zustand.Wk/= null and Zust.Akt_Zustand.Zk/= null then
--            if Zust.Akt_Zustand.Zk.Rest/= null then
--               if Zust.Akt_Zustand.Wk.T.Form = Wwert and
--                     not Zust.Akt_Zustand.Wk.T.Teil_Wwert and
--                     Zust.Akt_Zustand.Zk.T.Form = If_Case and
--                     Zust.Akt_Zustand.Zk.Rest.T.Form = Anw_F then
--                  return To_Unbounded_String("-- (14 : If-Anweisung False-Fall)--");
--               end if;
--            end if;
--         end if;

--         if Zust.Akt_Zustand.Zk/= null then
--            if Zust.Akt_Zustand.Zk.Rest/= null then
--               if Zust.Akt_Zustand.Zk.T.Form=Anw and then
--                     Zust.Akt_Zustand.Zk.T.Teil_Anw.Form = Schleife then
--                  return To_Unbounded_String("-- (15 : Schleife aufloesen)--");
--               end if;
--            end if;
--         end if;

--         if Zust.Akt_Zustand.Zk/= null and Zust.Akt_Zustand.Wk/= null then
--            if Zust.Akt_Zustand.Zk.Rest/= null then
--               if Zust.Akt_Zustand.Zk.Rest.Rest/= null then
--                  if Zust.Akt_Zustand.Wk.T.Form = Wwert and
--                        Zust.Akt_Zustand.Wk.T.Teil_Wwert and
--                        Zust.Akt_Zustand.Zk.T.Form = Do_Case and
--                        Zust.Akt_Zustand.Zk.Rest.T.Form = Bed and
--                        Zust.Akt_Zustand.Zk.Rest.Rest.T.Form = Anw_F then
--                     return To_Unbounded_String("-- (16 : Schleifenbedingung True)--");
--                  end if;
--               end if;
--            end if;
--         end if;

--         if Zust.Akt_Zustand.Wk/= null and Zust.Akt_Zustand.Zk/= null then
--            if Zust.Akt_Zustand.Wk.T.Form = Wwert and
--                  not Zust.Akt_Zustand.Wk.T.Teil_Wwert and
--                  Zust.Akt_Zustand.Zk.T.Form = Do_Case then
--               return To_Unbounded_String("-- (17 : Schleifenbedingung False)--");
--            end if;
--         end if;

--         return To_Unbounded_String("-- (Ende der Auswertung)--");
--      end if;
--   end Regel;
-----------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------
   procedure Trace (Z : Zustand) is

   begin
      case Z.Form is
         when Akt   => Put(Zeige_Konf(Z.Akt_Zustand));
                    -- put_line(to_string(Regel(z)));
                       Trace(Delta2(Z));
         when Ende  => Put_Line("Endspeicherbelegung: {" & (Zeige_Vars(Z.Ende_Zustand)) & "}");
      end case;
   end Trace;

-- Die Funktion Trace wertet einen bergebenen Zustand schrittweise aus.
-- Dazu wird der aktuelle Zustand angezeigt und die entsprechende Regel
-- ausgegeben. Dann ruft sich die Funktion rekursiv selbst auf, diesmal
-- aber nach einfacher Ausfhrung der Funktion Delta2 auf den Zustand.
-- Wenn der Zustand fertig ausgewertet ist und nun nur noch eine Umgebung
-- statt einer kompletten Konfiguration beinhaltet, wird nur die End-
-- speicherbelegung ausgegeben.

-----------------------------------------------------------------------------------------------
------main-------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------

-- Hilfsvariablen fr die Aufrufe
--   umgeb:umgebung;
--   h : Wert;
--   b : Boolean;
--   w1: wert(Undef);
--   w2: wert(Def_Wert);
   
   Zw : Integer := 0;
begin

   loop
      Put("Welcher Trace soll angezeigt werden? 1-5 fuer Ende druecke 6! Eingabe: ");
      Get(Zw);
      case Zw is
         when 1      => Trace(Z1);
         when 2      => Trace(Z2);
         when 3      => Trace(Z3);
         when 4      => Trace(Z4);
         when 5      => Trace(Z5);
         when 6      => null;
         when others => Put_Line("Falsche Eingabe");
      end case;
      exit when Zw = 6;
   end loop;

-- Hier, im eigentlichen Programm wird lediglich das Menu realisiert,
-- dass der User nach dem Starten und nach jedem Durchlauf einer Konfiguration
-- sieht. Das Menu und damit auch das Programm wird ber die Eingabe: 6 verlassen.

end While_Interpreter;

